@@ -53,6 +53,9 @@ void * malloc_fprintf(int sz, const char *format, ...);
 #define swbis_free free
 #endif
 
+#define swbis_devnull_open uxfio_devnull_open	/* cache opens on "/dev/null" */
+#define swbis_devnull_close uxfio_devnull_close
+
 /*
 * =====================
 */
