@@ -150,4 +150,8 @@ gb_init(GB * G) {
 	G->g_do_dereferenceM = 0;
 	G->g_ignore_slack_installM = 0;
 	*(G->g_gpufieldM) = '\0';
+	G->g_do_unconfigM = 0;
+	G->g_config_postinstallM = 0;
+	G->g_send_envM = 0;
+	G->g_no_summary_reportM = 0;
 }
