@@ -1,6 +1,6 @@
 /*  iswverify.c -- helper program for installed software verification
 
- Copyright (C) 2007,2008 Jim Lowe
+ Copyright (C) 2007,2008,2009,2010 Jim Lowe
  All Rights Reserved.
   
  COPYING TERMS AND CONDITIONS:
@@ -56,7 +56,7 @@
 #define PROGNAME                "swverify"
 #define SW_UTILNAME             PROGNAME
 #define SWPROG_VERSION          SWVERIFY_VERSION
-#define WOPT_LAST		246   /* Highest option number */
+#define WOPT_LAST		247   /* Highest option number */
 
 static char progName[] = PROGNAME;
 static char * CHARTRUE = "True";
@@ -86,7 +86,7 @@ version_info(FILE * fp)
 "and comes with NO WARRANTY to the extent permitted by law.\n"
 "See the file named COPYING for details.\n");
 }
-					
+
 static
 void
 write_attr(char * name, char * value, int index_format)
@@ -197,11 +197,13 @@ fprintf(fp, "%s",
 "      --check-mtime=true|false\n"
 "      --check-scripts=true|false    Not implemented\n"
 "      --check-volatile=true|false\n"
+"      --check-owners=true|false\n"
 "      --show-info    show the matching comparison info to stdout\n"
 "      --catalog-info-name=NAME  write comparison info to filename NAME\n"
 "      --system-info-name=NAME  write comparison info to filename NAME\n"
 "      --dereference-symlinks  when obtaining file system info\n"
 "      --ignore-slack-install  ignore Slackware control files when comparing\n"
+"      --without-summary-report  ignore Slackware control files when comparing\n"
 "\n"
 "  Verification Options for verifying a directory:\n"
 "\n"
@@ -260,7 +262,7 @@ fprintf(fp, "%s",
 
 fprintf(fp, "%s",
 "\n"
-" Posix Extended Options:\n"
+" POSIX Extended Options:\n"
 "        File : <libdir>/swbis/swdefaults\n"
 "               ~/.swdefaults\n"
 "        Override values in defaults file.\n"
@@ -291,6 +293,7 @@ fprintf(fp, "%s",
 "  swverify.swbis_no_remote_kill	 = false\n"
 "  swverify.swbis_any_format             = false\n"
 "  swverify.swbis_forward_agent          = true\n"
+"  swverify.swbis_check_owners           = true\n"
 "\n"
 );
 
@@ -364,21 +367,25 @@ safe_sig_handler(int signum)
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGUSR1\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGTERM:
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGTERM\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGINT:
 			if (G->g_do_progressmeter) alarm(0);
 			/* sw_e_msg(G, "Caught SIGINT\n"); */
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGPIPE:
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGPIPE\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 	}
 }
@@ -532,11 +539,17 @@ main(int argc, char **argv)
 	char wopt_pgm_mode[30];
 	
 	char * eopt_distribution_target_directory;
+	char * eopt_allow_incompatible;
 	char * eopt_enforce_scripts;
 	char * eopt_logfile;
 	char * eopt_installed_software_catalog;
 	char * eopt_select_local;
 	char * eopt_verbose;
+	char * eopt_check_scripts;
+	char * eopt_check_contents;
+	char * eopt_check_permissions;
+	char * eopt_check_volatile;
+	char * eopt_check_requisites;
 
 	char * response_img;
 
@@ -652,6 +665,8 @@ main(int argc, char **argv)
 		{"testnum", 1, 0, 164},
 		{"debug-verbose", 0, 0, 165},
 		{"force-locks", 0, 0, 167},
+		{"without-summary-report", 0, 0, 168},
+		{"no-summary-report",	0, 0, 168},
 		{"remote-shell", 1, 0, 172},
 		{"local-pax-write-command", 1, 0, 175},
 		{"remote-pax-write-command", 1, 0, 176},
@@ -723,11 +738,20 @@ main(int argc, char **argv)
 		{"swbis-forward-agent", 	1, 0, 236},
 		{"swbis_check_mtime", 		1, 0, 246},
 		{"swbis-check-mtime", 		1, 0, 246},
+		{"swbis_check_owners", 		1, 0, 247},
+		{"swbis-check-owners", 		1, 0, 247},
+		{"check-owners", 		1, 0, 247},
 		{0, 0, 0, 0}
              };
 
 	struct option posix_extended_long_options[] =
              {
+		{"check_volatile", 		1, 0, 210},
+		{"check_scripts", 		1, 0, 211},
+		{"check_requisites", 		1, 0, 212},
+		{"check_permissions", 		1, 0, 213},
+		{"check_contents", 		1, 0, 214},
+		{"allow_incompatible", 		1, 0, 215},
 		{"enforce_scripts", 		1, 0, 216},
 		{"installed_software_catalog",	1, 0, 217},
 		{"autoselect_dependencies", 	1, 0, 231},
@@ -735,10 +759,9 @@ main(int argc, char **argv)
 		{"enforce_dependencies", 	1, 0, 233},
 		{"logfile", 			1, 0, 235},
 		{"verbose", 			1, 0, 240},
-		/* {"check_requisites", 	1, 0, 243}, */
                {0, 0, 0, 0}
              };
-	
+/* XXX */	
 	struct option std_long_options[] =
              {
 		{"distribution", 0, 0, 'd'},
@@ -755,58 +778,6 @@ main(int argc, char **argv)
 		{0, 0, 0, 0}
              };
 
-
-	/* ---------------------------------------------------
-
-	struct option impl_extensions_long_options[] =
-             {
-		{"noop", 0, 0, 154},
-		{"show-options", 0, 0, 158},
-		{"show-options-files", 0, 0, 159},
-		{"testnum", 1, 0, 164},
-		{"debug-verbose", 0, 0, 165},
-		{"force-locks", 0, 0, 167},
-		{"remote-shell", 1, 0, 172},
-		{"local-pax-write-command", 1, 0, 175},
-		{"remote-pax-write-command", 1, 0, 176},
-		{"pax-write-command", 1, 0, 177},
-		{"remote-pax-read-command", 1, 0, 178},
-		{"local-pax-read-command", 1, 0, 179},
-		{"pax-read-command", 1, 0, 180},
-		{"no-defaults-files", 0, 0, 181},
-		{"pax-command", 1, 0, 182},
-		{"no-remote-kill", 0, 0, 184},
-		{"no-getconf", 0, 0, 185},
-		{"use-getconf", 0, 0, 186},
-		{"shell-command", 1, 0, 188},
-		{"ssh-options", 1, 0, 195},
-		{"debug-events", 0, 0, 196},
-		{"source-script-name", 1, 0, 198},
-		{"target-script-name", 1, 0, 199},
-		{"swi-debug-name", 1, 0, 200},
-		{"debug-task-scripts", 0, 0, 201},
-		{"enable-ssh-agent-forwarding", 0, 0, 202},
-		{"A", 0, 0, 202},
-		{"disable-ssh-agent-forwarding", 0, 0, 203},
-		{"a", 0, 0, 203},
-		{"noscripts", 0, 0, 204},	
-		{"no-scripts", 0, 0, 204},	
-		{"ignore-scripts", 0, 0, 204},	
-		{"remote-pax-remove-command", 1, 0, 205},
-		{"local-pax-remove-command", 1, 0, 206},
-		{"pax-remove-command", 1, 0, 207},
-		{"allow-ambig", 0, 0, 208},
-		{"sig-level", 1, 0, 209},
-		{"check-mtime", 1, 0, 210},
-		{"check-volatile", 1, 0, 211},
-		{"check-contents", 1, 0, 212},
-		{"check-permissions", 1, 0, 213},
-		{"check-scripts", 1, 0, 214},
-		{0, 0, 0, 0}
-             };
-
-	--------------------------------------------------- */
-	
 	struct ugetopt_option_desc main_help_desc[] =
              {
 	{"", "", "causes swverify to operate on a distribution"},
@@ -841,6 +812,8 @@ main(int argc, char **argv)
 	G->g_save_fdar = save_stdio_fdar;
 	G->g_verbose_threshold = SWC_VERBOSE_1;
 	G->optaM = opta;
+	
+	uxfio_devnull_open("/dev/null", O_RDWR, 0);  /* initiallize the null fd */
 
 	pre_reqs = NULL;  /* used for testing, not by a published option */
 	ex_reqs = NULL;   /* used for testing, not by a published option */
@@ -977,6 +950,7 @@ main(int argc, char **argv)
 	set_opta_initial(opta, SW_E_check_scripts, CHARTRUE);
 	set_opta_initial(opta, SW_E_check_volatile, CHARTRUE);
 	set_opta_initial(opta, SW_E_swbis_check_mtime, CHARTRUE);
+	set_opta_initial(opta, SW_E_swbis_check_owners, CHARTRUE);
 
 	set_opta_initial(opta, SW_E_swbis_no_getconf, wopt_no_getconf);
 	set_opta_initial(opta, SW_E_swbis_remote_shell_client,
@@ -1158,7 +1132,32 @@ gotoStandardOptions:
 		if (c == -1) break;
 		
 		switch (c) {
-		
+	/*XXX */	
+		case 210:
+			eopt_check_volatile = strdup(optarg);
+			set_opta_boolean(opta, SW_E_check_volatile, eopt_check_volatile);
+			break;
+		case 211:
+			eopt_check_scripts = strdup(optarg);
+			set_opta_boolean(opta, SW_E_check_scripts, eopt_check_scripts);
+			break;
+		case 212:
+			eopt_check_requisites = strdup(optarg);
+			set_opta_boolean(opta, SW_E_check_requisites, eopt_check_requisites);
+			break;
+		case 213:
+			eopt_check_permissions = strdup(optarg);
+			set_opta_boolean(opta, SW_E_check_permissions, eopt_check_permissions);
+			break;
+		case 214:
+			eopt_check_contents = strdup(optarg);
+			set_opta_boolean(opta, SW_E_check_contents, eopt_check_contents);
+			break;
+		case 215:
+			eopt_allow_incompatible = strdup(optarg);
+			set_opta_boolean(opta, SW_E_allow_incompatible, eopt_allow_incompatible);
+			break;
+
 		case 232:
 			eopt_distribution_target_directory = strdup(optarg);
 			set_opta(opta, 
@@ -1196,8 +1195,13 @@ gotoStandardOptions:
 			sw_e_msg(G, "error processing extended option\n");
 			sw_e_msg(G, "Try `swverify --help' for more information.\n");
 		 	LCEXIT(1);
-               break;
-               }
+			break;
+		}
+		if (swextopt_get_status()) {
+			optionname = getLongOptionNameFromValue(posix_extended_long_options, c);
+			sw_e_msg(G, "bad value detected for extended option '%s': %s\n", optionname, optarg);
+			LCEXIT(1);
+		}
 	}
 	optind = 1;
 	optarg =  NULL;
@@ -1237,6 +1241,9 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 		case 167:
 			G->g_force_locks = 1;
 			break;
+		case 168:
+			G->g_no_summary_reportM = 1;
+			break;
 		case 172:
 			remote_shell_command = strdup(optarg);
 			set_opta(opta, SW_E_swbis_remote_shell_client, remote_shell_command);
@@ -1429,6 +1436,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 		case 231:
 		case 236:
 		case 246:
+		case 247:
 			optionname = getLongOptionNameFromValue(main_long_options , c);
 			SWLIB_ASSERT(optionname != NULL);
 			optionEnum = getEnumFromName(optionname, opta);
@@ -1524,8 +1532,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 	 */
 
 	eopt_distribution_target_directory = get_opta(opta, SW_E_distribution_target_directory);
-	eopt_installed_software_catalog = get_opta(opta, SW_E_installed_software_catalog);
-	swlib_squash_all_leading_slash(eopt_installed_software_catalog);
+	eopt_installed_software_catalog = get_opta_isc(opta, SW_E_installed_software_catalog);
 
 	eopt_select_local		= get_opta(opta, SW_E_select_local);
 	eopt_verbose			= get_opta(opta, SW_E_verbose);
@@ -1564,6 +1571,11 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 		LCEXIT(0);
 	}
 
+	if (swextopt_get_status()) {
+		sw_e_msg(G, "bad value detected for extended option: %s\n", optionname);
+		LCEXIT(1);
+	}
+
 	swc_set_shell_dash_s_command(G, wopt_shell_command);
 	
 	if (*remote_shell_command == '/') {
@@ -1601,6 +1613,11 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 	login_orig_termiosP = NULL;
 	login_sizeP = NULL;
 	
+	swparse_set_do_not_warn_utf();
+	if (G->g_verboseG >= SWC_VERBOSE_4) {
+		swparse_unset_do_not_warn_utf();
+	}
+	
 	/* Set the signal handlers.  */
 
 	swgp_signal(SIGINT, safe_sig_handler);
@@ -2215,6 +2232,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 
 			E_DEBUG("");
 			if (f_retval == 0) {
+				E_DEBUG("");
 				response_img = swpl_get_samepackage_query_response(G, swicol, target_path,
 					target_fdar[0], target_fdar[1], opta, swspecs,
 					&f_retval, 0 /* make_dummy_response */);
@@ -2232,6 +2250,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 
 			/* Now parse the response */
 			sl = NULL;		
+			E_DEBUG("");
 			req = swpl_analyze_samepackage_query_response(G, response_img, &sl);
 			if (req == NULL) {
 				raise(SIGINT);
@@ -2278,7 +2297,11 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 					continue;
 				} 
 
-				/* Loop over the responses to a query */
+				/*
+				 * --------------------------------------------
+				 * Loop over the responses to a query
+				 * --------------------------------------------
+				 */
 
 				while(
 					f_retval == 0 &&
@@ -2305,6 +2328,9 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 					}
 					rp_status = 1;
 					E_DEBUG2("signal_flag=%d", G->g_signal_flag);
+					E_DEBUG2("SWEVERID: %s", swverid_show_object_debug(sr->swspecM, NULL, "sr->swspecM:"));
+					E_DEBUG2("SWEVERID LOCATION: %s", swverid_get_verid_value(sr->swspecM, SWVERID_VERIDS_LOCATION, 1));
+
 					ret = swverify_looper_sr_payload(G, target_path, swicol,
 								sc, sr,
 								target_fdar[1],
@@ -2337,26 +2363,26 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 					E_DEBUG2("signal_flag=%d", G->g_signal_flag);
 					critical_region_end();
 					swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
+					if (compare_status != 0) {
+						v_retval++;
+						swc_stderr_fd2_set(G, STDERR_FILENO);
+							sw_l_msg(G, SWC_VERBOSE_1, "%s at sig_level=%s for %s: status=1\n",
+							swevent_code(SW_SOC_INTEGRITY_NOT_CONFIRMED), wopt_sig_level, sc->sqM->swspec_stringM);
+						swc_stderr_fd2_restore(G);
+					} else {
+						swc_stderr_fd2_set(G, STDERR_FILENO);
+						sw_l_msg(G, SWC_VERBOSE_2, "%s at sig_level=%s for %s: status=0\n",
+							swevent_code(SW_SOC_INTEGRITY_CONFIRMED), wopt_sig_level, sc->sqM->swspec_stringM);
+						swc_stderr_fd2_restore(G);
+					}
 				}  /* loop over responses */
 				E_DEBUG("");
 				swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
 				if (swicol_get_master_alarm_status(swicol)) {
 					break;
 				}
-				E_DEBUG("");
-				if (compare_status != 0) {
-					v_retval++;
-					swc_stderr_fd2_set(G, STDERR_FILENO);
-					sw_l_msg(G, SWC_VERBOSE_1, "%s at sig_level=%s for %s: status=1\n",
-						swevent_code(SW_SOC_INTEGRITY_NOT_CONFIRMED), wopt_sig_level, sc->sqM->swspec_stringM);
-					swc_stderr_fd2_restore(G);
-				} else {
-					swc_stderr_fd2_set(G, STDERR_FILENO);
-					sw_l_msg(G, SWC_VERBOSE_2, "%s at sig_level=%s for %s: status=0\n",
-						swevent_code(SW_SOC_INTEGRITY_CONFIRMED), wopt_sig_level, sc->sqM->swspec_stringM);
-					swc_stderr_fd2_restore(G);
-				}
 				swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
+				E_DEBUG("");
 			} /* loop over software selections */
 			
 			E_DEBUG2("f_retval=%d", f_retval);
@@ -2385,6 +2411,10 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 			}
 
 			if (s_retval) f_retval ++;
+
+			if (sl)
+				swicat_sl_delete(sl);
+
 		} else if ( target_write_pid > 0 ) {
 			/* 
 			 * unexpected internal error
@@ -2567,6 +2597,18 @@ TARGET1:
 	close(G->g_swi_event_fd);
 	f_retval = swc_shutdown_logger(G, 0);
 
+	/* 
+	 *  -------  Test  Area --------
+	 */
+	if (0) {
+			STROB * xx = strob_open(2);
+			E_DEBUG("");
+			strob_close(xx);
+		}
+	/* 
+	 *  -------  Test  Area --------
+	 */
+
 	E_DEBUG("");
 	if (f_retval) return 1;
 	if (reqs_failed) return 3;
