@@ -36,6 +36,9 @@ g_pax_write_command g_pax_write_commands[] = {
 	},
 	{"swbistar",
 		SWBISLIBEXECDIR "/swbis/swbistar"
+	},
+	{"detect",    /* Used by swlist, swverify */
+		"false"
 	},
 		{(char*)NULL, (char*)NULL}
 	};
