@@ -187,29 +187,6 @@ writeline(int do_stderr, int do_log, int logfd, int efd, char * msg_prefix, char
 	return ret;
 }
 
-static
-int
-test_verbose(int verbose_level, int swbis_event,
-			int is_swi_event, int event_status, int is_posix_event)
-{
-	E_DEBUG3("verbose_level=%d  swbis_event=%d", verbose_level, swbis_event);
-	E_DEBUG3("is_swi_event=%d  event_status=%d", is_swi_event, event_status);
-	if (
-		(verbose_level >= SWC_VERBOSE_1 && 
-				event_status != 0) ||
-		(verbose_level >= SWC_VERBOSE_3 && is_posix_event) ||
-		(verbose_level >= SWC_VERBOSE_4 && swbis_event) ||
-		(verbose_level >= SWC_VERBOSE_6 && is_swi_event) ||
-		(verbose_level >= SWC_VERBOSE_4 && !is_swi_event)
-	) {
-		E_DEBUG("returning 1");
-		return 1;
-	} else {
-		E_DEBUG("returning 0");
-		return 0;
-	}
-}
-
 static
 int
 is_event(char * line, int * status,
@@ -250,11 +227,12 @@ write_error_line(GB * G, struct sw_logspec * logspec, int vofd, int vefd, STROB
 	int event_status;
 	int is_swi_event = 0;
 	int is_posix_event = 0;
-	int swbis_event;
+	int is_swbis_event;
 	char * news;
 	char * sptr;
 	int do_log = 0;
 	int do_stderr = 0;
+	struct swEvents * ev = NULL;
 
 	E_DEBUG("");
 	E_DEBUG2("verbose_level=%d" , verbose_level);
@@ -264,7 +242,7 @@ write_error_line(GB * G, struct sw_logspec * logspec, int vofd, int vefd, STROB
 	detect_special_verbose_status(G, sptr);
 	E_DEBUG("");
 
-	if (is_event(sptr, &event_status, &swbis_event, &is_swi_event, &is_posix_event)) {
+	if (is_event(sptr, &event_status, &is_swbis_event, &is_swi_event, &is_posix_event)) {
 		E_DEBUG("");
 		if (swi_event_fd >= 0) {
 			/*
@@ -275,7 +253,26 @@ write_error_line(GB * G, struct sw_logspec * logspec, int vofd, int vefd, STROB
 			E_DEBUG("");
 			news = strdup(sptr);
 			E_DEBUG2("sptr=[%s]", news);
+
+			if ((ev=swevents_get_struct_by_message(sptr, swevent_get_events_array())) == NULL) {
+				fprintf(stderr, "swbis internal error: EVENT NOT Found for [%s]\n", sptr);
+			} else {
+				;
+				/*
+				fprintf(stderr, "FOUND EVENT [%s] with threshhold of %d\n", ev->codeM, ev->verbose_threshholdM);
+				*/
+			}
+
+			/*
+			 * The next function is very important, it reads the human readable
+			 * messages and forms machine readable ascii text consisting only of
+			 * CODE:STATUS (e.g. 262:0) separated by newlines. This form is written
+			 * into the event_fd and read by the main process in the G->g_swi_event_fd
+			 * descriptor.
+			 */
+
 			evret = swevent_write_rpsh_event(swi_event_fd, news, len);
+
 			E_DEBUG("");
 			if (evret < 0) {
 				fprintf(stderr,
@@ -293,7 +290,7 @@ write_error_line(GB * G, struct sw_logspec * logspec, int vofd, int vefd, STROB
 			do_log = 0;
 		}
 
-		do_stderr = swlib_test_verbose(verbose_level, swbis_event,
+		do_stderr = swlib_test_verbose(ev, verbose_level, is_swbis_event,
 					is_swi_event, event_status, is_posix_event);
 		if (
 			do_log || do_stderr
@@ -1159,6 +1156,11 @@ swc_run_ssh_command (GB * G,
 
 	*pump_pid = 0;
 	
+	E_DEBUG("");
+	E_DEBUG2("path=[%s]", path);
+	E_DEBUG2("nhops=[%d]", nhops);
+	E_DEBUG2("is_no_fork_ok=[%d]", is_no_fork_ok);
+
 	if (source_file_size) *source_file_size = 0;
 	
 	if (is_local_fd) *is_local_fd = 0;
@@ -1173,6 +1175,7 @@ swc_run_ssh_command (GB * G,
 		*/
 		struct stat st;
 
+		E_DEBUG("Local File");
 		if (stat(path, &st) < 0) {
 			/*
 			* Source Access error
@@ -1184,6 +1187,7 @@ swc_run_ssh_command (GB * G,
 			/* fprintf(stderr, "%s : %s\n", path, strerror(errno)); */
 			return -2;
 		}	
+		E_DEBUG("");
 		if (
 			S_ISREG(st.st_mode) &&
 			G->g_do_createM == 0 &&
@@ -1214,11 +1218,13 @@ swc_run_ssh_command (GB * G,
 		}
 	}
 
+	E_DEBUG2("did_open=%d", did_open);
 	if (did_open == 0) {
 	 	i_fdm[0] = i_fdar[0];
 		i_fdm[1] = i_fdar[1];
 		i_fdm[2] = i_fdar[2];
 
+		E_DEBUG("");
 		ret = swc_form_command_args(G,
 			"target", 
 			path, 
@@ -1239,6 +1245,7 @@ swc_run_ssh_command (GB * G,
 	}
 
 	if (nhops == 0) {
+		E_DEBUG("Set sigdelset(ignoremask, SIGINT);");
 		sigdelset(ignoremask, SIGINT);
 	}
 
@@ -1254,6 +1261,7 @@ swc_run_ssh_command (GB * G,
 		/*
 		 * Fork happened.
 		 */
+		E_DEBUG("Fork Happened");
 		if (ts_pid == 0) {
 			/*
 			 * Execute the ssh pipeline.
@@ -1284,6 +1292,7 @@ swc_run_ssh_command (GB * G,
 		}
 	} else {
 		;
+		E_DEBUG("No Fork Happened");
 		/*
 		 * No fork required.
 		 */
