@@ -20,6 +20,7 @@
 
 #include "swuser_config.h"
 #include "swuser_assert_config.h"
+#include "sw.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -90,7 +91,7 @@ volatile int global_lastsig;
 #define SWBIS_TS_Load_fileset           "load fileset"
 #define SWBIS_TS_Load_catalog           "Load catalog and attributes"
 #define SWBIS_TS_Load_INSTALLED         "Load INSTALLED file"
-#define SWBIS_TS_Load_control_sh        "load control.sh"
+#define SWBIS_TS_Load_control_sh        "load " SW_A_CONTROL_SH
 #define SWBIS_TS_Load_session_options   "load session options"
 #define SWBIS_TS_Get_catalog_selections   "Get catalog selections"
 #define SWBIS_TS_Abort  		"Programmed Abort" /* Do nothing unsucessfully */
@@ -119,7 +120,10 @@ volatile int global_lastsig;
 #define SWBIS_TS_remove_files		"Remove files"
 #define SWBIS_TS_retrieve_files_archive "Retrive files archive"
 #define SWBIS_TS_post_verify		"Run post Verify"
-#define SWBIS_TS_get_catalog_perms		"Get catalog permissions"
+#define SWBIS_TS_get_catalog_perms	"Get catalog permissions"
+#define SWBIS_TS_run_configure		"Run configure script"
+#define SWBIS_TS_Load_INSTALLED_transient  SWBIS_TS_Load_INSTALLED " transient"
+#define SWBIS_TS_Load_management_host_status "Load manangement host status"
 
 
 #define SWPARSE_AT_LEVEL	0
