@@ -73,7 +73,7 @@ construct_controlsh_script(GB * G, STROB * buf, SWI * swi)
 	taglist = strob_open(100);
 	tmp = strob_open(100);
 	strob_strcpy(buf, "");
-	swicat_write_auto_comment(buf, "control.sh");
+	swicat_write_auto_comment(buf, SW_A_CONTROL_SH);
 	swicat_env(buf, swi, NULL, NULL);
 
 	swicat_construct_controlsh_taglist(swi, "*", taglist);
@@ -85,7 +85,7 @@ construct_controlsh_script(GB * G, STROB * buf, SWI * swi)
 		"	2)\n"
 		"		;;\n"
 		"	*)\n"
-		"		echo \"usage: control.sh selection tag\" 1>&2\n"
+		"		echo \"usage: " SW_A_CONTROL_SH " selection tag\" 1>&2\n"
 		"		exit 1\n"
 		"		;;\n"
 		"esac\n"
@@ -170,8 +170,8 @@ write_tar_controlsh_file(GB * G, SWI * swi, int ofd,
 	name = strob_open(100);
 	data = strob_open(100);
 
-	strob_strcpy(name, swi->swi_pkgM->catalog_entryM);
-	swlib_unix_dircat(name, SW_A_controlsh);
+	strob_strcpy(name, SW_A_controlsh);
+
 	ret = construct_controlsh_script(G, data, swi);
 	SWLIB_ASSERT(ret == 0);
 
@@ -184,7 +184,7 @@ write_tar_controlsh_file(GB * G, SWI * swi, int ofd,
 		event_fd,
 		id_str,
 		strob_str(name),
-		strob_str(data));
+		strob_str(data), (AHS*)NULL);
 
 	strob_close(name);
 	strob_close(data);
@@ -208,9 +208,8 @@ write_tar_session_options_file(GB * G, SWI * swi, int ofd,
 	name = strob_open(100);
 	data = strob_open(100);
 
-	swpl_write_session_options_filename(name, swi);
+	strob_strcpy(name, SW_A_session_options);
 	swpl_write_session_options_file(data, swi);
-
 	ret = swpl_load_single_file_tar_archive(G,
 		swi,
 		ofd, 
@@ -220,7 +219,7 @@ write_tar_session_options_file(GB * G, SWI * swi, int ofd,
 		event_fd,
 		id_str,
 		strob_str(name),
-		strob_str(data));
+		strob_str(data), (AHS*)NULL);
 
 	strob_close(name);
 	strob_close(data);
@@ -530,7 +529,7 @@ get_directory(SWI * swi)
 
 	/* FIXME, select the correct product, not the first */
 	product = swi_package_get_product(swi->swi_pkgM, 0); /* FIXME */
-	swheader_set_current_offset(indexheader, product->baseM.header_indexM);
+	swheader_set_current_offset(indexheader, product->p_baseM.header_indexM);
 
 	value = swheader_get_single_attribute_value(indexheader, SW_A_directory);
 	if (value == NULL) 
@@ -800,14 +799,6 @@ send_catalog_tarfile(SWI * swi, int ofd,
 	 * Here is the directory that is chdir'ed into.
 	 */
 	strob_strcpy(namebuf, catalog_path);
-	if (alt_catalog_root == 0) {
-		/*
-	 	 * Unless alt_catalog_root is true,
-		 * the catalog path is relative to the
-		 * target path.
-		 */
-		swlib_squash_all_leading_slash(strob_str(namebuf));
-	}
 
 	swicol_set_task_idstring(swi->swicolM, SWBIS_TS_Load_catalog);
 	ret = swicol_rpsh_task_send_script2(
@@ -871,33 +862,30 @@ send_catalog_tarfile(SWI * swi, int ofd,
 
 static
 int
-construct_execution_phase_script(GB * G, SWI * swi,
-			STROB * buf,
-			char * pax_read_command
-			)
+construct_execution_phase_script(GB * G, CISF_PRODUCT * cisf, SWI * swi,
+			STROB * buf, char * pax_read_command)
 {
 	int do_configure;
 	int ret;
 	char * xxx;
-	STROB * tmp;
 	char * v_target;
 	char * v_command;
-
-	tmp = strob_open(1000);
-
-	strob_sprintf(buf, STROB_DO_APPEND,
-	"( # subshell 001\n"
-	);
+	
 
 	/*
 	 * determine if its proper to run the configure scripts
 	 */
 	do_configure = determine_if_do_configure(swi, G->optaM);
 
+	strob_sprintf(buf, STROB_DO_APPEND,
+	"( # subshell 001\n"
+		CSHID
+	);
+
 	/*
 	 * construct the preinstall portion of the execution script
 	 */
-	ret = swpl_construct_script(G, buf, swi, SW_A_preinstall);
+	ret = swpl_construct_script(G, cisf, buf, swi, SW_A_preinstall);
 
 	strob_sprintf(buf, STROB_DO_APPEND,
 	"# FIXME: why isn't sw_retval being set\n"
@@ -951,11 +939,12 @@ construct_execution_phase_script(GB * G, SWI * swi,
 
 	strob_sprintf(buf, STROB_DO_APPEND,
 	"( # subshell 002\n"
+		CSHID
 	);
 	
-	ret = swpl_construct_script(G, buf, swi, SW_A_postinstall);
+	ret = swpl_construct_script(G, cisf,  buf, swi, SW_A_postinstall);
 
-	swpl_construct_configure_script(G, buf, swi, do_configure);
+	swpl_construct_configure_script(G, cisf, buf, swi, do_configure);
 
 	strob_sprintf(buf, STROB_DO_APPEND,
 	"exit \"$sw_retval\"\n"
@@ -970,7 +959,7 @@ construct_execution_phase_script(GB * G, SWI * swi,
 
 	return 0;
 }
-	
+
 static
 void
 construct_directory_ownership_script(SWI * swi, STROB * dir_owner_buf, char * isc_path)
@@ -989,7 +978,7 @@ construct_directory_ownership_script(SWI * swi, STROB * dir_owner_buf, char * is
 	strob_strcpy(dir_owner_buf, "true");
 	
 	strob_strcpy(tmp, swi->swi_pkgM->catalog_entryM);
-	swlib_squash_leading_slash(strob_str(tmp));
+	E_DEBUG2("swi->swi_pkgM->catalog_entryM is [%s]", swi->swi_pkgM->catalog_entryM);
 
 	s = strstr(strob_str(tmp), isc_path);
 	if (s != strob_str(tmp)) {
@@ -997,7 +986,6 @@ construct_directory_ownership_script(SWI * swi, STROB * dir_owner_buf, char * is
 		return;
 	}
 	s += (strlen(isc_path));
-	swlib_squash_leading_slash(isc_path);
 	swlib_squash_leading_slash(s);
 
         mode = swi->swi_pkgM->installed_catalog_modeM;
@@ -1178,93 +1166,10 @@ send_files_with_no_storage(GB * G, SWI * swi, SWHEADER * infoheader, XFORMAT * x
 	return retval;
 }
 
-int
-run_analysis_script(GB * G, SWI * swi, int ofd, int event_fd, int * p_checkinstall_status)
-{
-	int ret;
-	STROB * buf;
-	SWI_CONTROL_SCRIPT * checkinstall_script;
-	int stdin_file_size;
-	int this_index;
-	int script_status;
-	char * checkinstall_message;
-
-	buf = strob_open(400);
-
-	ret = swpl_construct_analysis_script(G, buf, swi, &checkinstall_script);
-	SWLIB_ASSERT(ret == 0);
-	
-	stdin_file_size = 512;
-
-	swicol_set_task_idstring(swi->swicolM, SWBIS_TS_Analysis_002);
-	ret = swicol_rpsh_task_send_script2(
-		swi->swicolM,
-		ofd, 
-		512,
-		".",
-		strob_str(buf),
-		SWBIS_TS_Analysis_002
-		);
-
-	if (ret == 0) {
-		/* 
-		 * Send the payload, in this case, it is a gratuitous block of nuls.
-		 */
-		ret = etar_write_trailer_blocks(NULL, ofd, 1);
-
-		if (ret < 0) return -1;
-
-		/*
-		 * wait for the script to finish
-		 */
-		ret = swicol_rpsh_task_expect(swi->swicolM,	
-				event_fd,
-				SWICOL_TL_9 /*timelimit*/);
-		if (ret < 0) return -1;
-		/*
-		 * this shows only the events for this task script
-		 */	
-		if (swi->debug_eventsM)
-			swicol_show_events_to_fd(swi->swicolM, STDERR_FILENO, -1);
-
-	
-		checkinstall_message = swicol_rpsh_get_event_message(swi->swicolM,
-       	         			SW_CONTROL_SCRIPT_BEGINS,
-					-1, 
-					&this_index);
-
-		*p_checkinstall_status = -1;  /* not set */
-		if (checkinstall_message) {
-			/*
-			 * Perform a sanity check on the message
-			 */
-	
-			SWLIB_ASSERT(strstr(checkinstall_message, SW_A_checkinstall) != NULL);
-	
-			/*
-			 * record the result
-			 */
-	
-			script_status = swicol_rpsh_get_event_status(swi->swicolM, NULL, SW_CONTROL_SCRIPT_ENDS, this_index, &this_index);
-			checkinstall_script->resultM = script_status;
-			*p_checkinstall_status = script_status;
-		} else {
-			/*
-			 * This is OK
-			 * There must not have been a checkinstall script
-			 */
-			;
-		}
-	}
-	strob_close(buf);
-	return ret;
-}
-
 int
 swinstall_lib_determine_catalog_path(STROB * buf, SWI_DISTDATA * distdata,
 		int product_number, char * isc_path, int instance_id)
 {
-	char * isc_relpath;
 	char * prodtag;
 	char * prodrev;
 	char * prodvendortag;
@@ -1272,9 +1177,10 @@ swinstall_lib_determine_catalog_path(STROB * buf, SWI_DISTDATA * distdata,
 	int ret;
 	STROB * tmp;
 	
+	E_DEBUG("");
+	E_DEBUG2("fp: isc_path: %s", isc_path);
 	tmp = strob_open(32);
-	isc_relpath = swlib_return_relative_path(isc_path);
-	strob_strcpy(tmp, isc_relpath);
+	strob_strcpy(tmp, isc_path);
 	swlib_unix_dircat(tmp, distdata->catalog_bundle_dir1M);
 
 	idx = product_number;  /* for now, always zero */
@@ -1393,7 +1299,9 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	char * p_qualifier;
 	char * p_directory;
 	SWVERID * swverid;
+	CISF_PRODUCT * cisf;
 
+	E_DEBUG2("fp: catalog_path=[%s]", catalog_path);
 	signal_flag = 0;
 	p_signal_flag = &signal_flag;
 	if (g_signal_flag)
@@ -1479,13 +1387,28 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	/*
 	 * FIXME: Eventually support multiple products and filesets.
 	 */
+
 	swpl_enforce_one_prod_one_fileset(swi);
 
 	/*
 	 * Set the current product and fileset
 	 */
-	current_product = swi_package_get_product(swi->swi_pkgM, 0 /* the first one */);
-	current_fileset = swi_product_get_fileset(current_product, 0 /* the first one */);
+	current_product = swi_package_get_product(swi->swi_pkgM, D_ZERO /* the first one */);
+	current_fileset = swi_product_get_fileset(current_product, D_ZERO /* the first one */);
+
+	/*
+	 * Construct the CISF_PRODUCT object
+	 */
+
+	cisf = swpl_cisf_product_create(current_product);
+
+	/*
+	 * Initialize the CISF object according to assumptions
+	 * of one product and one fileset  FIXME
+	 */	
+
+	swpl_cisf_init_single_single(cisf, swi);
+	swpl2_audit_cisf_bases(G, swi, cisf);
 
 	/*
 	 * Get the fileset size.
@@ -1545,7 +1468,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	}
 
 	construct_directory_ownership_script(swi, dir_owner_buf,
-			get_opta(G->optaM, SW_E_installed_software_catalog));
+			get_opta_isc(G->optaM, SW_E_installed_software_catalog));
 
 	E_DEBUG2("swi->swi_pkgM->catalog_entryM=[%s]", swi->swi_pkgM->catalog_entryM);
 	if (opt_preview == 0 && retval == 0 && G->g_to_stdout == 0) {
@@ -1814,7 +1737,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	if (opt_preview == 0 && retval == 0) {
 		E_DEBUG("HERE");
 		/* TS_catalog_unpack */
-		ret = swpl_unpack_catalog_tarfile(swi, vofd, catalog_path, pax_read_command, alt_catalog_root, event_fd);
+		ret = swpl_unpack_catalog_tarfile(G, swi, vofd, catalog_path, pax_read_command, alt_catalog_root, event_fd);
 		if (ret) {
 			retval++;
 			E_DEBUG2("retval = %d", retval);
@@ -1880,7 +1803,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		 * TS_analysis_002
 		 */
 		E_DEBUG("HERE");
-		ret = run_analysis_script(G, swi, vofd, event_fd, &checkinstall_status);
+		ret = swpl_run_check_script(G, SW_A_checkinstall, SWBIS_TS_Analysis_002, swi, vofd, event_fd, &checkinstall_status);
 		if (ret < 0) {
 			/* internal error */
 			retval++;
@@ -1918,13 +1841,13 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		if (up_e && do_check_abort && retval == 0) {
 			/* restore the catalog entry */
 			ret = swpl_restore_catalog_entry(G, swi->swicolM, up_e,
-				vofd, get_opta(G->optaM, SW_E_installed_software_catalog));
+				vofd, get_opta_isc(G->optaM, SW_E_installed_software_catalog));
 			retval++;
 		} else {
 			/* fulfill the task with nothing successfully */			
 			ret = swpl_send_nothing_and_wait(swi->swicolM, vofd, event_fd,
 					SWBIS_TS_restore_catalog_entry,
-					SWICOL_TL_4, SW_SUCCESS);
+					SWICOL_TL_8, SW_SUCCESS);
 		}
 
 		/* SWBIS_TS_continue */
@@ -1936,7 +1859,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 
 		}
 		ret = swpl_send_nothing_and_wait(swi->swicolM, vofd, event_fd,
-			SWBIS_TS_continue, SWICOL_TL_4, ret);
+			SWBIS_TS_continue, SWICOL_TL_8, ret);
 
 		if (ret != 0) retval++;
 	}
@@ -1949,7 +1872,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 
 	E_DEBUG("HERE");
 	if (opt_preview == 0 && retval == 0) {
-		/* TS_load_index_file_90 */
+		/* SWBIS_TS_Load_INSTALLED_90 */
 		/*
 		 * Audit the control_scripts that must be run
 		 */
@@ -1964,7 +1887,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 			pax_read_command,
 			alt_catalog_root,
 			event_fd,
-			SWBIS_TS_Load_INSTALLED_90);
+			SWBIS_TS_Load_INSTALLED_90, SW_A__INSTALLED, NULL);
 		if (ret) {
 			retval++;
 			E_DEBUG2("retval = %d", retval);
@@ -1984,7 +1907,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	if (opt_preview == 0 && retval == 0) {
 		/* TS_execution_phase */
 		E_DEBUG("HERE");
-		ret = construct_execution_phase_script(G, swi,
+		ret = construct_execution_phase_script(G, cisf, swi,
 				 execution_phase_script,
 				pax_read_command);
 
@@ -2411,8 +2334,6 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 					uxfio_write(G->g_t_efd, strob_str(tmp), strob_strlen(tmp));
 				} else {
 					/* send to logger process */
-					/* fprintf(stderr, "%d v=%d JLJL=%s", do_write_at_level, swi->verboseM, strob_str(tmp));
-					*/
 					swlib_doif_writef(swi->verboseM, do_write_at_level,
 						&G->g_logspec, G->g_t_efd, "%s", strob_str(tmp));
 				}
@@ -2557,7 +2478,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		E_DEBUG("HERE");
 		ret = swicol_rpsh_task_expect(swi->swicolM,
 				event_fd,
-				SWICOL_TL_6 /*time limit*/);
+				SWICOL_TL_1d /*time limit of 1 day*/);
 		if (ret != 0) {
 			E_DEBUG("HERE");
 			sw_e_msg(G, "error loading fileset\n");
@@ -2575,6 +2496,9 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		E_DEBUG("HERE");
 		swpl_update_execution_script_results(swi, swi->swicolM, swpl_get_script_array());
 		E_DEBUG("HERE");
+		swpl2_update_execution_script_results(swi, swi->swicolM, cisf); 
+
+		E_DEBUG("HERE");
 
 	} else {
 		E_DEBUG("HERE");
@@ -2591,7 +2515,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 			 */
 			/* TS_preview */
 			E_DEBUG("HERE at " SWBIS_TS_preview_task);
-			ret = swpl_send_nothing_and_wait(swi->swicolM, ofd, event_fd, SWBIS_TS_preview_task , SWICOL_TL_3, SW_SUCCESS);
+			ret = swpl_send_nothing_and_wait(swi->swicolM, ofd, event_fd, SWBIS_TS_preview_task , SWICOL_TL_8, SW_SUCCESS);
 			E_DEBUG("HERE at " SWBIS_TS_preview_task);
 			if (ret) { 
 				retval++;
@@ -2627,7 +2551,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 
 	E_DEBUG("HERE");
 	if (opt_preview == 0 && retval == 0) {
-		/* TS_load_index_file_91 */
+		/*  SWBIS_TS_Load_INSTALLED_91 */
 		swpl_update_fileset_state(swi, "*", SW_STATE_INSTALLED);
 		ret = swpl_write_tar_installed_software_index_file(G, swi,
 			vofd,
@@ -2635,7 +2559,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 			pax_read_command,
 			alt_catalog_root,
 			event_fd,
-			SWBIS_TS_Load_INSTALLED_91);
+			SWBIS_TS_Load_INSTALLED_91, SW_A__INSTALLED, NULL);
 		if (ret) {
 			retval++;
 			E_DEBUG2("retval = %d", retval);
@@ -2650,52 +2574,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	 */
 
 	if (opt_preview == 0 && retval == 0) {
-		swicol_set_task_idstring(swi->swicolM, SWBIS_TS_make_live_INSTALLED);
-		strob_sprintf(tmp, 0,
-			"cd \"%s\"\n"
-			"case \"$?\" in\n"
-			"	0)\n"
-			"		sw_retval=0\n"
-			"		;;\n"
-			"	*)\n"
-			"		sw_retval=1\n"
-			"		;;\n"
-			"esac\n"
-
-			"case \"$sw_retval\" in\n"
-			"	0)\n"
-			"		mv -f "	SW_A__INSTALLED " " SW_A_INSTALLED "\n"
-			"		sw_retval=$?\n"
-			"		;;\n"
-			"esac\n"
-			"dd count=1 bs=512 of=/dev/null 2>/dev/null\n"
-			,
-			swi->swi_pkgM->catalog_entryM);
-
-		ret = swicol_rpsh_task_send_script2(
-			swi->swicolM,
-			ofd, 
-			512, 			/* stdin_file_size */
-			target_path, 		/* directory to run in*/
-			strob_str(tmp),  	/* the task script */
-			SWBIS_TS_make_live_INSTALLED
-			);
-
-		if (ret != 0) {
-			return -1;
-		}
-
-		ret = etar_write_trailer_blocks(NULL, ofd, 1);
-		if (ret < 0) {
-			SWBIS_ERROR_IMPL();
-			SWI_internal_error();
-			return -1;
-		}
-
-		ret = swicol_rpsh_task_expect(swi->swicolM,
-					event_fd,
-					SWICOL_TL_5 /*time limit*/);
-
+		ret = swpl_run_make_installed_live(G, swi, ofd, target_path);
 		if (ret != 0) {
 			return -1;
 		}
@@ -2776,7 +2655,6 @@ write_target_install_script(GB * G,
 	STROB * subsh, * subsh2;
 	STROB * shell_lib_buf;
 	char * target_dirname, *source_basename, *target_basename;
-	char * isc_relpath;
 	char * targetpath;
 	char * x_targetpath;
 	char umaskbuf[10];
@@ -2834,8 +2712,7 @@ write_target_install_script(GB * G,
 	 * as being relative to the target_path
 	 */
 
-	isc_relpath = swlib_return_relative_path(installed_software_catalog);
-	strob_strcpy(catlockbuffer, isc_relpath);
+	strob_strcpy(catlockbuffer, installed_software_catalog);
 	swlib_unix_dircat(catlockbuffer, distdata->catalog_bundle_dir1M);
 
 	/*
@@ -3221,7 +3098,7 @@ write_target_install_script(GB * G,
 /*_% */	strob_str(catlockbuffer),
 /*_% */	umaskbuf,
 /*_% */	blocksize,
-/*_% */	isc_relpath,
+/*_% */	installed_software_catalog,
 /*_% */	x_targetpath, 
 /*_% */	g_sh_dash_s,
 /*_% */	debug_task_shell,	/* debug_task_shell */	
@@ -3343,13 +3220,6 @@ swinstall_arfinstall(GB * G, SWI * swi, int ofd, int * g_signal_flag,
 	swi->swi_pkgM->target_pathM = strdup(target_path);
 	swi->swi_pkgM->catalog_entryM = strdup(catalog_prefix);
 
-	/*
-	 * catalog_prefix is nominally:
-	 *     var/lib/swbis/catalog/<name>/<version>/<seq_no>
-	 * 
-	 * target path is the absolute path
-	 */
-	
 	E_DEBUG("HERE");
 	if (swi->swc_idM == SWC_U_L) {
 		;
