@@ -1,6 +1,6 @@
 /*  swlist.c -- The swbis catalog listing utility.
 
- Copyright (C) 2006,2007 Jim Lowe
+ Copyright (C) 2006,2007,2010 Jim Lowe
  All Rights Reserved.
   
  COPYING TERMS AND CONDITIONS:
@@ -383,21 +383,25 @@ safe_sig_handler(int signum)
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGUSR1\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGTERM:
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGTERM\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGINT:
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGINT\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGPIPE:
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGPIPE\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 	}
 }
@@ -557,6 +561,7 @@ main(int argc, char **argv)
 	int w_argc = 1;
 	int swi_event_fd = -1;
 	int ism;
+	int no_of_selections_processed;
 	int jsm;
 	char * tmp_s;
 	pid_t ss_pid = 0;
@@ -641,7 +646,6 @@ main(int argc, char **argv)
 	char * target_fork_type;
 	char * target_script_pid = (char*)NULL;
 	char * tmpcharp = (char*)NULL;
-	/* char * selections_filename = (char*)NULL; */
 	char * exoption_arg;
 	char * exoption_value;
 	char * catalog_entry_directory;
@@ -689,6 +693,7 @@ main(int argc, char **argv)
 	sigset_t * fork_blockmask;
 	sigset_t * fork_defaultmask;
 	sigset_t * currentmask;
+	SWVERID * swverid;
 	SWLOG * swlog;
 	SWICOL * swicol;
 	SWUTS * uts;
@@ -836,6 +841,8 @@ main(int argc, char **argv)
 	G->g_verbose_threshold = SWC_VERBOSE_1;
 	G->optaM = opta;
 
+	uxfio_devnull_open("/dev/null", O_RDWR, 0);  /* initiallize the null fd */
+
 	BL = &g_blist;
 	blist_init(BL);
 
@@ -1520,7 +1527,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 	    the defaults file(s). */
 
 	eopt_distribution_target_directory = get_opta(opta, SW_E_distribution_target_directory);
-	eopt_installed_software_catalog = get_opta(opta, SW_E_installed_software_catalog);
+	eopt_installed_software_catalog = get_opta_isc(opta, SW_E_installed_software_catalog);
 	eopt_select_local		= get_opta(opta, SW_E_select_local);
 	eopt_one_liner			= get_opta(opta, SW_E_one_liner);
 	eopt_verbose			= get_opta(opta, SW_E_verbose);
@@ -1595,6 +1602,11 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 		dup2(nullfd, STDERR_FILENO);
 	}
 
+	swparse_set_do_not_warn_utf();
+	if (G->g_verboseG >= SWC_VERBOSE_4) {
+		swparse_unset_do_not_warn_utf();
+	}
+
 	if (G->g_verboseG >= SWC_VERBOSE_7) {
 		swlib_set_verbose_level(G->g_verboseG);
 	}
@@ -1715,6 +1727,7 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 				wopt_ssh_options,
 				swc_is_option_true(wopt_forward_agent));
 		SWLIB_ASSERT(target_nhops >= 0);
+		E_DEBUG2("Target nhops is %d", target_nhops);
 
 		/* Set the archive writing utility */
 
@@ -1921,7 +1934,7 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 		} else {
 			/* Non-remote target, i.e. the localhost
 			   Here is the target piping for local stdin */
-			E_DEBUG("");
+			E_DEBUG("********* Not a remote target");
 
 			ss_pid = 0;
 			target_fdar[0] = target_stdio_fdar[0];
@@ -2217,8 +2230,11 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 			G->g_swicolM = swicol;
 
 			/* Handle the SWBIS_TS_uts task shell */
-			if (swpl_get_utsname_attributes(G, swicol, uts, target_fdar[1], G->g_swi_event_fd)) {
-				SWLIB_ASSERT(0);
+			ret = swpl_get_utsname_attributes(G, swicol, uts, target_fdar[1], G->g_swi_event_fd);
+			if (ret != 0) {
+				abort_script(swicol, G, target_fdar[1]);
+				ret = swpl_send_abort(swicol, target_fdar[1], G->g_swi_event_fd, "");
+				f_retval++;
 			}
 			swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
 
@@ -2274,7 +2290,7 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 				swicat_write_isc_script(isc_script_buf, G, swspecs, NULL, NULL, NULL,
 					target_path, SWICAT_FORM_TAR1);
 				strob_strcpy(target_catalog_dir, target_path);
-				swlib_unix_dircat(target_catalog_dir, get_opta(opta, SW_E_installed_software_catalog));
+				swlib_unix_dircat(target_catalog_dir, get_opta_isc(opta, SW_E_installed_software_catalog));
 				/* swicol_set_task_idstring(swicol, SWBIS_TS_Get_iscs_listing); */
 				ret = swicol_rpsh_task_send_script2(swicol,
 					target_fdar[1],			/* file descriptor of the task shell's stdin */
@@ -2332,12 +2348,25 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 				*/
 
 				/*
-				 * ===========================================================
-				 * ===========================================================
 				 * ===========================================================
 				 * ===========================================================
 				 */
 
+				if (! does_have_sw_selections(swspecs)) {
+					E_DEBUG("No selections given on command line");
+					/* Per spec, If no selections are given,
+					   process all selections. Hence we need to add
+					   as swverid for "*" */
+					swverid = swverid_open(NULL, "*");
+					if (swverid == NULL) {
+						/* This should never happen */
+						SWLIB_FATAL("");
+					} else {
+						E_DEBUG("Adding * as default selection");
+						vplob_add(swspecs, (void*)swverid);
+					}
+				}
+
 				E_DEBUG("");
 				if (f_retval == 0) {
 					response_img = swpl_get_samepackage_query_response(G, swicol, target_path,
@@ -2371,7 +2400,6 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 				if (G->g_do_debug_events)
 					swicol_show_events_to_fd(swicol, STDERR_FILENO, -1);
 
-				ism = 0;
 
 				/*
 				 * --------------------------------------------
@@ -2379,8 +2407,10 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 				 * --------------------------------------------
 				 */
 	
-				E_DEBUG("");
+				E_DEBUG("Looping over selections");
 				G->in_shls_looperM = 1;
+				ism = 0;
+				no_of_selections_processed = 0;
 	
 				/* loop over the queries, which are actually the whitespace delimited
 				   software selections from the command line */
@@ -2392,8 +2422,9 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 					G->g_signal_flag == 0 &&
 					1
 				) {
+					E_DEBUG2("START OF SELECTION LOOP for %s", sc->sqM->swspec_stringM);
 					jsm = 0;
-					E_DEBUG("");
+					no_of_selections_processed++;
 					nmatches = sc_selections_check(G, opta, sc);
 					if (nmatches == 0) {
 						strob_sprintf(btmp2, 0,
@@ -2413,25 +2444,14 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 						1
 					) {
 						critical_region_begin();
+						E_DEBUG2("START OF RESPONSE for %s", swverid_print(sr->swspecM, NULL));
 						E_DEBUG("");
 						catalog_entry_directory = swicat_sr_form_catalog_path(sr,
 										eopt_installed_software_catalog, NULL);
-	
+						E_DEBUG("");
 						E_DEBUG2("catalog_entry_directory=%s", catalog_entry_directory);
-						if ( 0 && /* disable */
-							nmatches > 1 && swc_is_option_true(wopt_allow_ambig) == 0
-						) {
-							/* don't allow ambiguaous selections (e.g. wildcards) unless
-							   specifically allowed via the --allow-ambig option */
-							E_DEBUG("");
-							swicat_sr_form_swspec(sr, btmp2);
-							sw_e_msg(G, "SW_SELECTION_NOT_FOUND_AMBIG: %s: status=1\n",
-								strob_str(btmp2));
-							found_ambig = 1;
-							s_retval++;
-							continue;
-						}
 						rp_status = 1;
+						E_DEBUG("");
 						E_DEBUG2("signal_flag=%d", G->g_signal_flag);
 						ret = swlist_looper_sr_payload(G, target_path, swicol,
 									sr,
@@ -2466,6 +2486,7 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 						E_DEBUG2("signal_flag=%d", G->g_signal_flag);
 						critical_region_end();
 						swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
+						E_DEBUG("END OF RESPONSE");
 					}  /* loop over responses */
 					E_DEBUG("");
 					swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
@@ -2473,9 +2494,11 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 						break;
 					}
 					E_DEBUG("");
-				swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
+					swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
+					E_DEBUG2("END OF SELECTION LOOP for %s", sc->sqM->swspec_stringM);
+					/* swevent_s_arr_reset(); */
 				} /* loop over software selections */
-			
+				E_DEBUG("finished looping over selections");
 				E_DEBUG2("f_retval=%d", f_retval);
 	
 				/* this terminates the shls_looper() loop */
@@ -2508,7 +2531,8 @@ E_DEBUG2("current_arg=[%s]", current_arg);
 				 * ===========================================================
 				 */
 
-
+				if (sl)
+					swicat_sl_delete(sl);
 			} else if (
 				swpl_test_pgm_mode(wopt_pgm_mode, SWLIST_PMODE_PROD) == 0 ||
 				swpl_test_pgm_mode(wopt_pgm_mode, SWLIST_PMODE_DIR) == 0 ||
@@ -2732,6 +2756,7 @@ TARGET1:
 						G->g_targetfd_array,
 					eopt_distribution_target_directory, &num_remains);
 		swi_distdata_delete(distdataO);
+		/* swevent_s_arr_reset(); */
 	} /* target loop */
 	E_DEBUG("");
 
@@ -2749,6 +2774,7 @@ TARGET1:
 			&G->g_logspec, swc_get_stderr_fd(G), "SWI_NORMAL_EXIT: status=%d\n",
 		sw_exitval(G, target_loop_count, target_success_count));
 
+	/* swevent_s_arr_delete(); */
 	strob_close(tmp);
 	strob_close(btmp2);
 	E_DEBUG("");
