@@ -51,41 +51,6 @@
 #include "shlib.h"
 #include "ls_list.h"
 
-/*
-static
-int
-make_verbose_tar_listing(int ofd, int ifd)
-{
-	while (
-		xformat_read_header(xformat) > 0
-	) {
-		E_DEBUG("HERE: Storage");
-		if (xformat_is_end_of_archive(xformat)){
-			break;
-		}
-		xformat_get_name(xformat, namebuf);
-		name = strob_str(namebuf);
-		if (strstr(name, topsf->format_desc_->pathname_prefixM) == name) {
-			name += strlen(topsf->format_desc_->pathname_prefixM);
-			swlib_squash_leading_slash(name);
-		}
-		if (strlen(name) == 0) {
-			if (xformat_file_has_data(xformat)) { 
-				xformat_copy_pass_by_dst(xformat, nullfd);
-			}
-		} else {
-			ahsStaticSetTarFilename(ahs_vfile_hdr(xformat_ahs_object(xformat)), name);
-			xformat_write_header(xformat);
-			if (xformat_file_has_data(xformat)) { 
-				xformat_copy_pass_thru(xformat);
-			}
-		}
-	}
-	xformat_close(xformat);
-	etar_write_trailer_blocks(NULL, STDOUT_FILENO, 2);
-}
-*/
-
 static
 void
 looper_abort(GB * G, SWICOL * swicol, int ofd)
@@ -93,6 +58,18 @@ looper_abort(GB * G, SWICOL * swicol, int ofd)
 	swpl_send_abort(swicol, ofd, G->g_swi_event_fd,  SWBIS_TS_Abort);
 }
 
+static
+int
+show_nopen(void)
+{
+	int ret;
+	ret = open("/dev/null",O_RDWR);
+	if (ret<0)
+		fprintf(stderr, "fcntl error: %s\n", strerror(errno));
+	close(ret);
+	return ret;
+}
+
 static
 char *
 looper_locked_region(STROB * buf, int vlv)
@@ -111,8 +88,8 @@ looper_locked_region(STROB * buf, int vlv)
 		"				read swspec_string\n"                  /* SWBIS_DT_0001 */
 		"				%s\n" /* SW_SELECTION_EXECUTION_BEGINS */
 
-		"				# echo Processing Now \"$xxb\" 1>&2\n"
-		"				echo \"$xxb\"\n"			/* SWBIS_DT_0002 */
+		"				# echo Processing Now \"$catentdir\" 1>&2\n"
+		"				echo \"$catentdir\"\n"			/* SWBIS_DT_0002 */
 
 		"				tar cbf 1 - \"$catalog_entry_dir\" 2>/dev/null\n"
 		"				sw_retval=$?\n"
@@ -379,7 +356,7 @@ swlist_write_source_copy_script2(GB * G,
 	/* Make a nice message for the SOURCE_ACCESS_BEGINS event when
 	   reading the catalog */
 
-	strob_sprintf(isc_msg, 0, "installed catalog at %s", get_opta(G->optaM, SW_E_installed_software_catalog));
+	strob_sprintf(isc_msg, 0, "installed catalog at %s", get_opta_isc(G->optaM, SW_E_installed_software_catalog));
 
 
 	/* Make a final adjustment if the path is '/', FIXME */
@@ -744,17 +721,12 @@ swlist_list_distribution_archive(
 	swi->opt_alt_catalog_rootM = G->g_opt_alt_catalog_root;
 	swi->swi_pkgM->target_pathM = strdup(target_path);
 	swi->swi_pkgM->installed_software_catalogM =
-		strdup(get_opta(opta, SW_E_installed_software_catalog));
+		strdup(get_opta_isc(opta, SW_E_installed_software_catalog));
 
 	if (swlib_check_clean_path(swi->swi_pkgM->installed_software_catalogM)) {
 		SWLIB_FATAL("tainted path");
 	}
 		
-	/* FIXME, ?? eventually need to support a catalog location that is
-	   outside of the target path? */
-
-	swlib_squash_all_leading_slash(swi->swi_pkgM->installed_software_catalogM);
-
 	swicol_set_targetpath(swi->swicolM, target_path);
 	swicol_set_delaytime(swi->swicolM, SWC_SCRIPT_SLEEP_DELAY);
 	swicol_set_nhops(swi->swicolM, target_nhops);
@@ -769,7 +741,6 @@ swlist_list_distribution_archive(
 			"error decoding source INDEX file: swbis_code=%d\n", ret);
 
 			/* Close and return */
-			xformat_close(swi->xformatM);
 			G->g_xformat = NULL;
 			swi_delete(swi);
 			return -2;
@@ -828,7 +799,6 @@ swlist_list_distribution_archive(
 			 
 	swlib_pipe_pump(G->g_nullfd,  dup_of_target_fdar0);
 
-	xformat_close(swi->xformatM);
 	G->g_xformat = NULL;
 	swi_delete(swi);
 	E_DEBUG("");
@@ -860,6 +830,8 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	int do_skip_entry;
 	int ls_verbose;
 	int compare_status;
+	int result;
+	int sig_level;
 	char * pax_read_command;
 	char * epath;
 	char * installed_software_catalog;
@@ -887,11 +859,16 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	swgpg = NULL;
 	ee = NULL;
 
+	sig_level = swlib_atoi(get_opta(G->optaM, SW_E_swbis_sig_level), &result);
+	if (result) {
+		sig_level = 0;
+	}
+
 	pax_read_command = swc_get_pax_read_command(G->g_pax_read_commands,
 		"tar", G->g_verboseG >= SWC_VERBOSE_3,
 		0 /*keep_old_files*/, DEFAULT_PAX_R);
 	E_DEBUG("");
-	installed_software_catalog = get_opta(G->optaM, SW_E_installed_software_catalog);
+	installed_software_catalog = get_opta_isc(G->optaM, SW_E_installed_software_catalog);
 	catalog_entry_directory = swicat_sr_form_catalog_path(sr, installed_software_catalog, NULL);
 
 	E_DEBUG2("catalog_entry_directory = [%s]", catalog_entry_directory);
@@ -900,6 +877,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	E_DEBUG("");
 	*p_rp_status = 0;
+	E_DEBUG("setting *p_rp_status = 0");
 	if (strlen(catalog_entry_directory) == 0) {
 		/* return with no error
 		   this happens for `the empty response` to a query */
@@ -964,6 +942,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		sw_e_msg(G, "swpl_session_lock lock failed for %s, ret=%d\n", strob_str(btmp), ret);
 		/* sw_d_msg(G, "swpl_session_lock lock failed for %s, ret=%d\n", strob_str(btmp), ret); */
 		*p_rp_status = 1;
+		E_DEBUG("setting *p_rp_status = 1");
 		return 1;
 	} 
 
@@ -992,17 +971,20 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	/* SWBIS_DT_0002 */
 
+	E_DEBUG("");
 	swgp_read_line(ifd, btmp2, DO_APPEND);
 	if (G->devel_verboseM)
 		fprintf(stderr, "rp_status is %d\n", ret);
 
 	/* Create the catalog entry control structure */
 
+	E_DEBUG("");
 	ee = swicat_e_create();
 
 	/* Now read an archive of the installed catalog entry on
 	   file descriptor ifd */
 
+	E_DEBUG("");
 	ret = swicat_e_open_entry_tarball(ee, ifd);
 	if (ret < 0) {
 		sw_e_msg(G, "error opening catalog entry tarball, ret=%d\n", ret);
@@ -1015,25 +997,23 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	/* ee->entry_prefixM contains the installed_software_catalog
 	  attribute already prefixed */
 
-	epath = swicat_e_form_catalog_path(ee, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_ACTIVE_ENTRY);
+	E_DEBUG("");
+	epath = swicat_e_form_catalog_path(ee, btmp, NULL, SWICAT_ACTIVE_ENTRY);
 	if (G->devel_verboseM)
 		SWLIB_INFO2("catalog entry (active) path = %s", epath);
 	
-	epath = swicat_e_form_catalog_path(ee, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_DEACTIVE_ENTRY);
+	E_DEBUG("");
+	epath = swicat_e_form_catalog_path(ee, btmp, NULL, SWICAT_DEACTIVE_ENTRY);
 	if (G->devel_verboseM)
 		SWLIB_INFO2("catalog entry path (inactive) path = %s", epath);
 
-	epath = swicat_e_form_catalog_path(ee, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_ACTIVE_ENTRY);
+	E_DEBUG("");
+	epath = swicat_e_form_catalog_path(ee, btmp, NULL, SWICAT_ACTIVE_ENTRY);
 
 	/* epath is now a clean relative path to such as:
 		var/lib/swbis/catalog/emacs/emacs/21.2.1/0  */
 
+	E_DEBUG("");
 	ret = swicat_e_reset_fd(ee);
 	if (ret < 0) {
 		rstatus = 2;
@@ -1046,21 +1026,34 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	/* Now obtain the verification status for each signature */
 	if (G->devel_verboseM)
 		SWLIB_INFO2("NOPEN=%d", uxfio_uxfio_get_nopen());
+	
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
+	E_DEBUG("");
 	swgpg = swgpg_create();
 	if (G->devel_verboseM)
 		SWLIB_INFO2("NOPEN=%d", uxfio_uxfio_get_nopen());
-	ret = swicat_e_verify_gpg_signature(ee, swgpg);
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
+	E_DEBUG("");
+
+	ret = 0;
+	if (sig_level >= 0)
+		ret = swicat_e_verify_gpg_signature(ee, swgpg);
 	if (ret != 0) {
 		rstatus = 3;
 		retval = 0;
 		sw_e_msg(G, "error from swicat_e_verify_gpg_signature, ret=%d\n", ret);
 	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 	/* Now interpret the verification results according to extended option
 	   requirements */
 
-	ret = swpl_signature_policy_accept(G, swgpg, G->g_verboseG, strob_str(swspec_string));
+	E_DEBUG("");
+	ret = 0;
+	if (sig_level >= 0)
+		ret = swpl_signature_policy_accept(G, swgpg, G->g_verboseG, strob_str(swspec_string));
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	E_DEBUG2("ret=%d", ret);
 	if (ret != 0) {
 		/* Bad signature or not enough good signatures
@@ -1074,13 +1067,17 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		/* Signatures OK, or don't care */
 		;
 	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 	/*
 	 * If we're here its OK to decode the catalog.tar file to create (SWI*)swi
 	 * because its authenticated, or we don't care.
 	 */
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
+	E_DEBUG("");
 	swi = swicat_e_open_swi(ee);
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	if (swi == NULL) {
 		/* swi might be NULL due to file system permission,
 		   handle NULL gracefully */
@@ -1092,6 +1089,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		swi->swi_pkgM->catalog_entryM = strdup(epath); /* FIXME, this must set explicitly */
 		swi->swi_pkgM->target_pathM = strdup(target_path); /* FIXME, this must set explicitly */
 	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 
 	if (G->devel_verboseM)
@@ -1108,8 +1106,10 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	the remote script to report a problem and to re-verify script-data
 	sychronization. */
 	
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	ret = swpl_report_status(swicol, ofd, G->g_swi_event_fd);
 	*p_rp_status = ret;
+	E_DEBUG2("setting *p_rp_status = %d", ret);
 	if (ret) {
 		/* looper_abort(G, swicol, ofd); */
 		rstatus = 6;
@@ -1121,8 +1121,10 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	/* wait for clear-to-send */
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	ret = swicol_rpsh_wait_cts(swicol, G->g_swi_event_fd);
 	*p_rp_status = ret;
+	E_DEBUG2("setting *p_rp_status = %d", ret);
 	if (ret) {
 		looper_abort(G, swicol, ofd);
 	}
@@ -1131,12 +1133,14 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		swicol_show_events_to_fd(swicol, STDERR_FILENO, -1);
 
 	/* Make the file list for the product from the catalog metadata */
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 	if (swi) { 
 		file_list = swicat_e_make_file_list(ee, uts, swi);
 	} else {
 		file_list = NULL;
 	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 	if (swi && file_list == NULL) {
 		rstatus = 7;
@@ -1163,6 +1167,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	skip:
 
 	/* SWBIS_TS_Catalog_unpack */
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 	if (
 		G->g_opt_previewM ||
@@ -1179,7 +1184,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			retval = -1;
 		}
 	} else {
-		ret = swpl_unpack_catalog_tarfile(swi, ofd,
+		ret = swpl_unpack_catalog_tarfile(G, swi, ofd,
 			catalog_entry_directory,
 			pax_read_command,
 			0 /*alt_catalog_root*/,
@@ -1193,6 +1198,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	/* SWBIS_TS_retrieve_files_archive */
 	ls_verbose = G->g_verboseG > SWC_VERBOSE_1 ? LS_LIST_VERBOSE_L1 : LS_LIST_VERBOSE_L0;
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 	if (
 		G->g_opt_previewM ||
@@ -1233,6 +1239,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 				pax_write_command_key,
 				-1, -1, (FILE_DIGS*)NULL);
 		}
+		E_DEBUG2("LOWEST FD=%d", show_nopen());
 		E_DEBUG("");
 		if (ret < 0) {
 			rstatus = 8;
@@ -1247,11 +1254,13 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		}
 	}
 	E_DEBUG("");
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 	/* --------------------------------------------------*/
 	/* Now, read the payload which is a tar archive      */
 	/* --------------------------------------------------*/
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	if (wopt_file_sys) {
 		E_DEBUG("List files from file system (above)");
 		/* see swpl_retrieve_files() invocation above */
@@ -1269,6 +1278,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		}
 	}
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	archive_fd = -1;
 
 	E_DEBUG("");
@@ -1308,6 +1318,7 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		}
 	}
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	E_DEBUG("");
 	error_out:
 	E_DEBUG("");
@@ -1322,8 +1333,10 @@ swlist_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	if (G->g_do_debug_events)
 		swicol_show_events_to_fd(swicol, STDERR_FILENO, -1);
-	if (rstatus)
+	if (rstatus) {
 		*p_rp_status = rstatus;
+		E_DEBUG2("setting *p_rp_status = %d", rstatus);
+	}
 	if (swi) swi_delete(swi);
 	if (swgpg) swgpg_delete(swgpg);
 	if (ee) swicat_e_delete(ee);
