@@ -192,7 +192,7 @@ fprintf(fp, "%s",
 "   enforce_dependencies        = false      # Not Implemented\n"
 "   enforce_scripts             = false      # Not Implemented\n"
 "   installed_software_catalog  = var/lib/swbis/catalog\n"
-"   logfile                     = /var/log/swinstall.log\n"
+"   logfile                     = /var/log/swremove.log\n"
 "   loglevel                    = 0\n"
 "   select_local		= false      # Not Implemented\n"
 "   verbose			= 1\n"
@@ -323,21 +323,25 @@ safe_sig_handler(int signum)
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGUSR1\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGTERM:
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGTERM\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGINT:
 			if (G->g_do_progressmeter) alarm(0);
 			/* sw_e_msg(G, "Caught SIGINT\n"); */
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 		case SIGPIPE:
 			if (G->g_do_progressmeter) alarm(0);
 			sw_e_msg(G, "Caught SIGPIPE\n");
 			G->g_signal_flag = signum;
+			if (G->g_swicolM) swicol_set_master_alarm(G->g_swicolM);
 			break;
 	}
 }
@@ -684,60 +688,6 @@ main(int argc, char **argv)
                {0, 0, 0, 0}
              };
 
-	/* -------------------------------------------------
-	-------------------------------------------------
-	-------------------------------------------------
-	-------------------------------------------------
-	struct option impl_extensions_long_options[] =
-             {
-		{"noop", 0, 0, 154},
-		{"show-options", 0, 0, 158},
-		{"show-options-files", 0, 0, 159},
-		{"testnum", 1, 0, 164},
-		{"debug-verbose", 0, 0, 165},
-		{"force-locks", 0, 0, 167},
-		{"remote-shell", 1, 0, 172},
-		{"local-pax-write-command", 1, 0, 175},
-		{"remote-pax-write-command", 1, 0, 176},
-		{"pax-write-command", 1, 0, 177},
-		{"remote-pax-read-command", 1, 0, 178},
-		{"local-pax-read-command", 1, 0, 179},
-		{"pax-read-command", 1, 0, 180},
-		{"no-defaults-files", 0, 0, 181},
-		{"pax-command", 1, 0, 182},
-		{"no-remote-kill", 0, 0, 184},
-		{"no-getconf", 0, 0, 185},
-		{"use-getconf", 0, 0, 186},
-		{"shell-command", 1, 0, 188},
-		{"ssh-options", 1, 0, 195},
-		{"debug-events", 0, 0, 196},
-		{"source-script-name", 1, 0, 198},
-		{"target-script-name", 1, 0, 199},
-		{"swi-debug-name", 1, 0, 200},
-		{"debug-task-scripts", 0, 0, 201},
-		{"enable-ssh-agent-forwarding", 0, 0, 202},
-		{"A", 0, 0, 202},
-		{"disable-ssh-agent-forwarding", 0, 0, 203},
-		{"a", 0, 0, 203},
-		{"noscripts", 0, 0, 204},	
-		{"no-scripts", 0, 0, 204},	
-		{"ignore-scripts", 0, 0, 204},	
-		{"remote-pax-remove-command", 1, 0, 205},
-		{"local-pax-remove-command", 1, 0, 206},
-		{"pax-remove-command", 1, 0, 207},
-		{"allow-ambig", 0, 0, 208},
-		{"sig-level", 1, 0, 209},
-		{"remove-volatile", 1, 0, 210},
-		{"keep-volatile-files", 0, 0, 211},
-		{"cleansh", 0, 0, 212},
-		{"justdb", 0, 0, 213},
-		{0, 0, 0, 0}
-             };
-	------------------------------------------------------
-	------------------------------------------------------
-	------------------------------------------------------
-	------------------------------------------------------ */	
-
 	struct option std_long_options[] =
              {
 		{"distribution", 0, 0, 'd'},
@@ -788,6 +738,8 @@ main(int argc, char **argv)
 
 	pre_reqs = NULL;  /* used for testing, not by a published option */
 	ex_reqs = NULL;   /* used for testing, not by a published option */
+	
+	uxfio_devnull_open("/dev/null", O_RDWR, 0);  /* initiallize the null fd */
 
 	target_fdar[0] = -1;
 	target_fdar[1] = -1;
@@ -1130,8 +1082,13 @@ gotoStandardOptions:
 			sw_e_msg(G, "error processing extended option\n");
 			sw_e_msg(G, "Try `swremove --help' for more information.\n");
 		 	LCEXIT(1);
-               break;
-               }
+		break;
+		}
+		if (swextopt_get_status()) {
+			optionname = getLongOptionNameFromValue(posix_extended_long_options, c);
+			sw_e_msg(G, "bad value detected for extended option '%s': %s\n", optionname, optarg);
+			LCEXIT(1);
+		}
 	}
 	optind = 1;
 	optarg =  NULL;
@@ -1434,8 +1391,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 	 */
 
 	eopt_distribution_target_directory = get_opta(opta, SW_E_distribution_target_directory);
-	eopt_installed_software_catalog = get_opta(opta, SW_E_installed_software_catalog);
-	swlib_squash_all_leading_slash(eopt_installed_software_catalog);
+	eopt_installed_software_catalog = get_opta_isc(opta, SW_E_installed_software_catalog);
 
 	eopt_select_local		= get_opta(opta, SW_E_select_local);
 	eopt_verbose			= get_opta(opta, SW_E_verbose);
@@ -1500,6 +1456,11 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 		dup2(nullfd, STDOUT_FILENO);
 		dup2(nullfd, STDERR_FILENO);
 	}
+	
+	swparse_set_do_not_warn_utf();
+	if (G->g_verboseG >= SWC_VERBOSE_4) {
+		swparse_unset_do_not_warn_utf();
+	}
 
 	if (G->g_verboseG >= SWC_VERBOSE_7) {
 		swlib_set_verbose_level(G->g_verboseG);
@@ -2182,6 +2143,13 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 			/* loop over the queries, which are actually the whitespace delimited
 			   software selections from the command line */
 
+			if (! does_have_sw_selections(swspecs)) {
+				E_DEBUG("No selections given on command line");
+				/* Per spec, If no selections are given, do nothing */
+				sw_e_msg(G, "SW_SELECTION_NOT_FOUND for nil (empty) selection list\n");
+				s_retval ++;
+			}
+
 			while (
 				/* s_retval == 0 && */
 				f_retval == 0 &&
@@ -2299,6 +2267,10 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 
 			E_DEBUG("");
 			if (s_retval) f_retval ++;
+
+			if (sl)
+				swicat_sl_delete(sl);
+
 		} else if ( target_write_pid > 0 ) {
 			/* 
 			 * unexpected internal error
