@@ -53,7 +53,98 @@
 #include "swicat.h"
 #include "swicat_e.h"
 
-#define SWREMOVE_ARG_MAX_THRESHOLD 900
+#define SWREMOVE_ARG_MAX_THRESHOLD 800
+
+static
+int
+make_remove_glob_list_for(int do_dir, GB * G, SWI * swi, STROB * buf, SWI_FILELIST * fl, int do_remove_volatile, int do_rename_volatile)
+{
+	STROB * rm_tmp;
+	STROB * suffix_tmp;
+	char * sx;
+	int type;
+	int ix;
+	int action;
+
+	rm_tmp = strob_open(32);
+	ix = 0;
+	while ((sx=swi_fl_get_path(fl, ix)) != NULL) {
+		if (G->devel_verboseM) {
+			SWLIB_INFO4("INFO; file=[%s] type=[%c] is_volatile=[%c]", sx,
+					(int)swi_fl_get_type(fl, ix),
+					(int)swi_fl_is_volatile(fl, ix));
+		}
+
+		if (G->g_justdbM == 1) {
+			action = 0;
+		} else if (swi_fl_is_volatile(fl, ix))	{
+			/* apply volatile policy */
+			if (do_remove_volatile == 1) {
+				/* keep going, just do it */
+				action = 1;
+			} else if (do_remove_volatile == 0) {
+				/* skip it */
+				action = 0;
+			} else {
+				/* rename it */
+				action = 2;
+			}
+		} else {
+			action = 1;
+		}
+
+		if (action == 1) {
+			/* remove it */
+			type = swi_fl_get_type(fl, ix);
+			if (do_dir == 0 && type != DIRTYPE) {
+				strob_sprintf(rm_tmp, STROB_DO_APPEND, " \"%s\"", sx);
+				if (strob_strlen(rm_tmp) > SWREMOVE_ARG_MAX_THRESHOLD) {
+					strob_sprintf(buf, STROB_DO_APPEND, "rm -f %s\n", strob_str(rm_tmp));
+					strob_sprintf(buf, STROB_DO_APPEND, "case $? in 0) ;; *) rm_retval=$? ;; esac\n");
+					strob_strcpy(rm_tmp, "");
+				}
+			} else if (do_dir && type == DIRTYPE) {
+				if (strcmp(sx, "/") == 0 || strcmp(sx, "./") == 0) {
+					; /* skip */
+				} else {
+					swlib_squash_embedded_dot_slash(sx);
+					strob_sprintf(rm_tmp, STROB_DO_APPEND, " \"%s\"", sx);
+				}
+				E_DEBUG2("HAVE dir to remove: %s", sx);
+				if (strob_strlen(rm_tmp) > SWREMOVE_ARG_MAX_THRESHOLD) {
+					E_DEBUG2("AT SWREMOVE_ARG_MAX_THRESHOLD with %s\n", sx);
+					strob_sprintf(buf, STROB_DO_APPEND, "rmdir %s\n", strob_str(rm_tmp));
+					strob_sprintf(buf, STROB_DO_APPEND, "case $? in 0) ;; *) rm_retval=$? ;; esac\n");
+					E_DEBUG2("HAVE rmdir command args: %s", strob_str(rm_tmp));
+					strob_strcpy(rm_tmp, "");
+				}
+			}
+		} else if (action == 0) {
+			/* skip it */
+			;
+		}  else {
+			/* rename it */
+			strob_sprintf(buf, STROB_DO_APPEND, "mv -f \"%s\" \"%s%s\"\n",
+				sx, sx, SWREMOVE_VOLATILE_SUFFIX);
+
+			/* Do not trap this error, just let it go
+			strob_sprintf(buf, STROB_DO_APPEND, "case $? in 0) ;; *) rm_retval=$? ;; esac\n");
+			*/
+		}
+		ix++;
+	}
+
+	if (do_dir == 0) {
+		if (strob_strlen(rm_tmp) > 0)
+			strob_sprintf(buf, STROB_DO_APPEND, "\nrm -f %s\n", strob_str(rm_tmp));
+	} else {
+		if (strob_strlen(rm_tmp) > 0)
+			strob_sprintf(buf, STROB_DO_APPEND, "\nrmdir %s\n", strob_str(rm_tmp));
+	}
+	
+	strob_close(rm_tmp);
+	return 0;
+}
 
 static
 void
@@ -80,7 +171,7 @@ show_file_list(GB * G, SWI_FILELIST * fl)
 
 static
 int
-make_remove_command_script_fragment(GB * G, SWI * swi, STROB * buf, SWI_FILELIST * fl)
+make_remove_command_script_fragment(GB * G, CISF_PRODUCT * cisf, SWI * swi, STROB * buf, SWI_FILELIST * fl)
 {
 	STROB * suffix_tmp;
 	STROB * rm_tmp;
@@ -125,7 +216,7 @@ make_remove_command_script_fragment(GB * G, SWI * swi, STROB * buf, SWI_FILELIST
 		"sw_retval=0\n"
 		);
 
-	swpl_construct_script(G, buf, swi, SW_A_preremove);
+	swpl_construct_script(G, cisf, buf, swi, SW_A_preremove);
 
 	strob_sprintf(buf, STROB_DO_APPEND,
 	"exit \"$sw_retval\"\n"
@@ -135,82 +226,14 @@ make_remove_command_script_fragment(GB * G, SWI * swi, STROB * buf, SWI_FILELIST
 
 	strob_sprintf(buf, STROB_DO_APPEND, "rm_retval=0\n");
 
-	ix = 0;
-	while ((sx=swi_fl_get_path(fl, ix)) != NULL) {
-		if (G->devel_verboseM) {
-			SWLIB_INFO4("INFO; file=[%s] type=[%c] is_volatile=[%c]", sx,
-					(int)swi_fl_get_type(fl, ix),
-					(int)swi_fl_is_volatile(fl, ix));
-		}
-
-		if (G->g_justdbM == 1) {
-			action = 0;
-		} else if (swi_fl_is_volatile(fl, ix))	{
-			/* apply volatile policy */
-			if (do_remove_volatile == 1) {
-				/* keep going, just do it */
-				action = 1;
-			} else if (do_remove_volatile == 0) {
-				/* skip it */
-				action = 0;
-			} else {
-				/* rename it */
-				action = 2;
-			}
-		} else {
-			action = 1;
-		}
-
-		if (action == 1) {
-			/* remove it */
-			if (swi_fl_get_type(fl, ix) != DIRTYPE) {
-				strob_sprintf(rm_tmp, STROB_DO_APPEND, " \"%s\"", sx);
-				if (strob_strlen(rm_tmp) > SWREMOVE_ARG_MAX_THRESHOLD) {
-					strob_sprintf(buf, STROB_DO_APPEND, "rm -f %s\n", strob_str(rm_tmp));
-					strob_sprintf(buf, STROB_DO_APPEND, "case $? in 0) ;; *) rm_retval=$? ;; esac\n");
-					strob_strcpy(rm_tmp, "");
-				}
-			} else {
-				if (strcmp(sx, "/") == 0 || strcmp(sx, "./") == 0) {
-					; /* skip */
-				} else {
-					strob_sprintf(rmdir_tmp, STROB_DO_APPEND, " \"%s\"", sx);
-				}
-				if (strob_strlen(rmdir_tmp) > SWREMOVE_ARG_MAX_THRESHOLD) {
-					strob_sprintf(buf, STROB_DO_APPEND, "rmdir %s\n", strob_str(rmdir_tmp));
-					strob_sprintf(buf, STROB_DO_APPEND, "case $? in 0) ;; *) rm_retval=$? ;; esac\n");
-					strob_strcpy(rmdir_tmp, "");
-				}
-			}
-		} else if (action == 0) {
-			/* skip it */
-			;
-		}  else {
-			/* rename it */
-			/*
-			strob_sprintf(buf, STROB_DO_APPEND, "mv -f \"%s\" \"%s.%s\"\n",
-				sx, sx, swpl_rename_suffix(suffix_tmp));
-			*/
-			strob_sprintf(buf, STROB_DO_APPEND, "mv -f \"%s\" \"%s%s\"\n",
-				sx, sx, SWREMOVE_VOLATILE_SUFFIX);
-
-			/* Do not trap this error, just let it go
-			strob_sprintf(buf, STROB_DO_APPEND, "case $? in 0) ;; *) rm_retval=$? ;; esac\n");
-			*/
-		}
-		ix++;
-	}
-
-	if (strob_strlen(rm_tmp) > 0)
-		strob_sprintf(buf, STROB_DO_APPEND, "\nrm -f %s\n", strob_str(rm_tmp));
-	if (strob_strlen(rmdir_tmp) > 0)
-		strob_sprintf(buf, STROB_DO_APPEND, "\nrmdir %s\n", strob_str(rmdir_tmp));
+	make_remove_glob_list_for(0 /* not DIRTYPE */, G, swi, buf, fl, do_remove_volatile, do_rename_volatile);
+	make_remove_glob_list_for(1 /* now the DIRTYPE */, G, swi, buf, fl, do_remove_volatile, do_rename_volatile);
 	
 	strob_sprintf(buf, STROB_DO_APPEND,
 		"( # subshell 002_remove\n"
 		);
 
-	swpl_construct_script(G, buf, swi, SW_A_postremove);
+	swpl_construct_script(G, cisf, buf, swi, SW_A_postremove);
 
 	strob_sprintf(buf, STROB_DO_APPEND,
 		"exit \"$sw_retval\"\n"
@@ -226,16 +249,17 @@ make_remove_command_script_fragment(GB * G, SWI * swi, STROB * buf, SWI_FILELIST
 
 static
 int
-remove_files(GB * G, SWI * swi, SWICOL * swicol, SWICAT_E * e, SWI_FILELIST * file_list, int ofd)
+remove_files(GB * G, CISF_PRODUCT * cisf, SWI * swi, SWICOL * swicol, SWICAT_E * e, SWI_FILELIST * file_list, int ofd)
 {
 	STROB * buf;
 	int ret;
 
+	E_DEBUG("");
 	buf = strob_open(300);
 
 	strob_sprintf(buf, STROB_DO_APPEND, "rm_retval=0\n");
 
-	make_remove_command_script_fragment(G, swi, buf, file_list);
+	make_remove_command_script_fragment(G, cisf, swi, buf, file_list);
 
 	/* Add the requisite dd bs=512 count=1 of=/dev/null */
 		
@@ -261,6 +285,7 @@ remove_files(GB * G, SWI * swi, SWICOL * swicol, SWICAT_E * e, SWI_FILELIST * fi
 	}
 
 	ret = swicol_rpsh_task_expect(swicol, G->g_swi_event_fd, SWICOL_TL_10);
+	E_DEBUG2("swicol_rpsh_task_expect returned %d", ret);
 
 	strob_close(buf);
 	return ret;
@@ -284,15 +309,20 @@ looper_locked_region(STROB * buf, int vlv)
 		"				read swspec_string\n"                  /* SWBIS_DT_0001 */
 		"				%s\n" /* SW_SELECTION_EXECUTION_BEGINS */
 
-		"				# echo Processing Now \"$xxb\" 1>&2\n"
-		"				echo \"$xxb\"\n"			/* SWBIS_DT_0002 */
+		"				# echo Processing Now \"$catentdir\" 1>&2\n"
+		"				echo \"$catentdir\"\n"			/* SWBIS_DT_0002 */
+		
+		"				test -d \"$catalog_entry_dir\" 2>/dev/null\n"
+		"				case $? in \n"
+		"					0) tar cbf 1 - \"$catalog_entry_dir\" 2>/dev/null; ;;\n"
+		"					*) dd if=/dev/zero count=2 2>/dev/null; ;;\n"
+		"				esac\n"
 
-		"				tar cbf 1 - \"$catalog_entry_dir\" 2>/dev/null\n"
 		"				sw_retval=$?\n"
 		"				# echo \"here is the swspec string: [$swspec_string]\" 1>&2\n"
 
 		"				rp_status=$sw_retval\n"
-		"				# rp_status=8\n"
+
 		"				shls_bashin2 \"" SWBIS_TS_report_status "\"\n"
 		"				sw_retval=$?\n"
 		"				case $sw_retval in 0) $sh_dash_s " SWBIS_TS_report_status ";; *) shls_false_;; esac\n"
@@ -306,7 +336,13 @@ looper_locked_region(STROB * buf, int vlv)
 		"				case $sw_retval in 0) $sh_dash_s " SWBIS_TS_Catalog_unpack  ";; *) shls_false_;; esac\n"
 		"				sw_retval=$?\n"
 		"				swexec_status=$sw_retval\n"
-	
+
+		"				shls_bashin2 \"" SWBIS_TS_Analysis_002 "\"\n"
+		"				sw_retval=$?\n"
+		"				case $sw_retval in 0) $sh_dash_s " SWBIS_TS_Analysis_002 ";; *) shls_false_;; esac\n"
+		"				sw_retval=$?\n"
+		"				swexec_status=$sw_retval\n"
+
 		"				shls_bashin2 \"" SWBIS_TS_remove_files "\"\n"
 		"				sw_retval=$?\n"
 		"				case $sw_retval in 0) $sh_dash_s " SWBIS_TS_remove_files  ";; *) shls_false_;; esac\n"
@@ -483,7 +519,7 @@ swremove_write_source_copy_script2(GB * G,
 	/* Make a message for the SOURCE_ACCESS_BEGINS event when
 	   reading the catalog */
 
-	strob_sprintf(isc_msg, 0, "installed catalog at %s", get_opta(G->optaM, SW_E_installed_software_catalog));
+	strob_sprintf(isc_msg, 0, "installed catalog at %s", get_opta_isc(G->optaM, SW_E_installed_software_catalog));
 
 
 	/* Make a final adjustment if the path is '/', FIXME */
@@ -506,8 +542,6 @@ swremove_write_source_copy_script2(GB * G,
 
 	/* Now here is the script */
 	strob_sprintf(buffer_new, STROB_DO_APPEND,
-	/* ret = swlib_writef(ofd, buffer,  */
-									/* ism_begin */
 		"trap '/bin/rm -f ${LOCKPATH}.lock; exit 1' 1 2 15\n"
 		"echo " SWBIS_TARGET_CTL_MSG_125 ": " KILL_PID "\n"
 		CSHID
@@ -557,7 +591,7 @@ swremove_write_source_copy_script2(GB * G,
 		"export LOCKPATH\n"
 		"# Here is the override of the shls_looper_payload function\n"
 		"#\n"
-		"%s\n"    /* <<<--- shls_looper_payload */
+		"%s\n"    /* <<<--- shls_looper_payload looper_routine */
 		
 		/* Here, in this first if-then statement we will classify and
 		   type-test the target path */
@@ -764,100 +798,6 @@ swremove_write_source_copy_script2(GB * G,
 	return !(ret > 0);
 }
 
-static  /* Not used at present */
-int
-swremove_ls_fileset_from_iscat(SWI * swi)
-{
-	int retval;
-	SWHEADER_STATE state1;
-	SWHEADER * indexheader;
-	SWHEADER * infoheader;
-	SWI_PRODUCT * current_product;
-	SWI_XFILE * current_fileset;
-	char * next_line;
-	char * next_attr;
-	char * keyword;
-	char * value;
-	AHS * info_ahs2;
-	int ret;
-	STROB * ls_line;
-	int taru_ls_verbose_level;
-
-	E_DEBUG("");
-	retval = 0;
-	info_ahs2 = ahs_open();
-	ls_line = strob_open(60);
-
-	/* FIXME: Eventually support multiple products and filesets  */
-
-	swpl_enforce_one_prod_one_fileset(swi);
-
-	/* Get the current product and fileset, for now, the first ones */
-
-	current_product = swi_package_get_product(swi->swi_pkgM, 0 /* the first one */);
-	current_fileset = swi_product_get_fileset(current_product, 0 /* the first one */);
-
-	indexheader = swi_get_global_index_header(swi);
-
-	/* Loop over the INFO file definitions, these are found
-		in the SWHEADER for the fileset */
-
-	infoheader = swi_get_fileset_info_header(swi, 0, 0); /* FIXME */
-	swheader_store_state(infoheader, NULL);
-	swheader_reset(infoheader);
-
-        while ((next_line=swheader_get_next_object(infoheader, (int)UCHAR_MAX, (int)UCHAR_MAX))) {
-
-		E_DEBUG2("next_line=%s\n", next_line);
-		if (swheaderline_get_type(next_line) != SWPARSE_MD_TYPE_OBJ) {
-			/* Sanity check */
-			SWBIS_IMPL_ERROR_DIE(1);
-		}
-
-		keyword = swheaderline_get_keyword(next_line);
-
-		/* skip control files */
-
-		SWLIB_ASSERT(keyword != NULL);
-		if (strcmp(keyword, SW_A_control_file) == 0) {
-			continue;
-		}
-
-		ahs_init_header(info_ahs2);
-		swheader_store_state(infoheader, &state1);
-		ret = swheader_fileobject2filehdr(infoheader, ahs_vfile_hdr(info_ahs2));
-		if (ret) {
-			SWLIB_FATAL("");
-		}
-		swheader_restore_state(infoheader, &state1);
-
-		/*  Here is some example code for accessing and
-			printing the file attributes */
-		/******
-		value = swheaderline_get_value(next_line, (int*)(NULL));
-		swlib_doif_writef(swi->verboseM, SWC_VERBOSE_1,
-			(struct sw_logspec *)(NULL), STDERR_FILENO,
-			"%s %s\n", keyword, value?value:"");
-		while((next_attr=swheader_get_next_attribute(infoheader))) {
-			keyword = swheaderline_get_keyword(next_attr);
-			value = swheaderline_get_value(next_attr, (int*)(NULL));
-			swlib_doif_writef(swi->verboseM, SWC_VERBOSE_1,
-				(struct sw_logspec *)(NULL), STDERR_FILENO,
-				"%s %s\n", keyword, value?value:"");
-		}	
-		******/
-		taru_ls_verbose_level = swpl_determine_tar_listing_verbose_level(swi);	
-		taru_print_tar_ls_list(ls_line, ahs_vfile_hdr(info_ahs2), taru_ls_verbose_level);
-	
-		uxfio_unix_atomic_write(1, strob_str(ls_line), strob_strlen(ls_line));
-        }
-
-	swheader_restore_state(infoheader, NULL);
-	ahs_close(info_ahs2);
-	strob_close(ls_line);
-	return retval;
-}
-
 int
 swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		SWICAT_SR * sr, int ofd, int ifd, int * p_rp_status, SWUTS * uts)
@@ -867,6 +807,9 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	int rstatus;
 	int ret;
 	int do_skip_entry;
+	int checkremove_status;
+	int do_check_abort;
+	int do_enforce_scripts;
 	char * pax_read_command;
 	char * epath;
 	char * name;
@@ -880,6 +823,8 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	SWI * swi;
 	SWI_FILELIST * file_list;
 	SWGPG_VALIDATE * swgpg;
+	CISF_PRODUCT * cisf;
+	SWI_PRODUCT * current_product;
 	
 	do_skip_entry = 0;
 	retval = 0;
@@ -891,12 +836,14 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	file_list = NULL;
 	swgpg = NULL;
 	e = NULL;
+	checkremove_status = -1; /* unset value */
 
+	do_enforce_scripts = swextopt_is_option_true(SW_E_enforce_scripts, G->optaM);
 	pax_read_command = swc_get_pax_read_command(G->g_pax_read_commands,
 		"tar", G->g_verboseG >= SWC_VERBOSE_3,
 		0 /*keep_old_files*/, DEFAULT_PAX_R);
 	E_DEBUG("");
-	installed_software_catalog = get_opta(G->optaM, SW_E_installed_software_catalog);
+	installed_software_catalog = get_opta_isc(G->optaM, SW_E_installed_software_catalog);
 	catalog_entry_directory = swicat_sr_form_catalog_path(sr, installed_software_catalog, NULL);
 
 	E_DEBUG2("catalog_entry_directory = [%s]", catalog_entry_directory);
@@ -1000,25 +947,26 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		retval = -1;  /* protocol internal error */
 		looper_abort(G, swicol, ofd);
 		goto error_out;
+	} else if (ret == SWICAT_RETVAL_NULLARCHIVE) {
+		/* this is the case for a missing catalog
+		   entry which may happen */
+		do_skip_entry = 1;
+		goto GL_skip;
+	} else {
+		; /* Normal path */
 	}
 
 	/* e->entry_prefixM has the installed_software_catalog
 	  path already prefixed */
-	epath = swicat_e_form_catalog_path(e, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_ACTIVE_ENTRY);
+	epath = swicat_e_form_catalog_path(e, btmp, NULL, SWICAT_ACTIVE_ENTRY);
 	if (G->devel_verboseM)
 		SWLIB_INFO2("catalog entry (active) path = %s", epath);
 	
-	epath = swicat_e_form_catalog_path(e, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_DEACTIVE_ENTRY);
+	epath = swicat_e_form_catalog_path(e, btmp, NULL, SWICAT_DEACTIVE_ENTRY);
 	if (G->devel_verboseM)
 		SWLIB_INFO2("catalog entry path (inactive) path = %s", epath);
 
-	epath = swicat_e_form_catalog_path(e, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_ACTIVE_ENTRY);
+	epath = swicat_e_form_catalog_path(e, btmp, NULL, SWICAT_ACTIVE_ENTRY);
 
 	/* epath is now a clean relative path to such as:
 		var/lib/swbis/catalog/emacs/emacs/21.2.1/0  */
@@ -1074,7 +1022,7 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		do_skip_entry = 1;
 		sw_e_msg(G, "catalog read error, possible permission denied: %s/%s\n", target_path, epath);
 	}
-
+	
 	if (swi) swi->swi_pkgM->catalog_entryM = strdup(epath); /* FIXME, this must set explicitly */
 	if (swi) swi->swi_pkgM->target_pathM = strdup(target_path); /* FIXME, this must set explicitly */
 
@@ -1084,7 +1032,21 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	if (G->devel_verboseM)
 		SWLIB_INFO2("NOPEN=%d", uxfio_uxfio_get_nopen());
 
-	error_out_and_report:
+	/* Create the CISF_PRODUCT object */
+	current_product = swi_package_get_product(swi->swi_pkgM, D_ZERO /* FIXME, assumes the first one */);
+	cisf = swpl_cisf_product_create(current_product);
+
+	/*
+	 * FIXME
+	 * Initialize the CISF object according to assumptions
+	 * of one product and one fileset  FIXME
+	 */	
+
+	/* FIXME, this should be based on INSTALL, not the single
+	   product and single fileset assumption */
+	swpl_cisf_init_single_single(cisf, swi);
+
+	GL_skip:
 
 	/* SWBIS_TS_report_status
 	The purpose of this is solely to provide an opportunity for
@@ -1093,6 +1055,7 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	
 	ret = swpl_report_status(swicol, ofd, G->g_swi_event_fd);
 	*p_rp_status = ret;
+	E_DEBUG2("rp_status is %d", ret);
 	if (ret) {
 		/* looper_abort(G, swicol, ofd); */
 		rstatus = 6;
@@ -1106,6 +1069,7 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	ret = swicol_rpsh_wait_cts(swicol, G->g_swi_event_fd);
 	*p_rp_status = ret;
+	E_DEBUG2("rp_status is %d", ret);
 	if (ret) {
 		looper_abort(G, swicol, ofd);
 	}
@@ -1140,8 +1104,6 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	if (G->devel_verboseM)
 		SWLIB_INFO2("%s", strob_str(btmp));
 
-	skip:
-
 	if (
 		G->g_opt_previewM ||
 		do_skip_entry ||
@@ -1157,7 +1119,7 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			retval = -1;
 		}
 	} else {
-		ret = swpl_unpack_catalog_tarfile(swi, ofd,
+		ret = swpl_unpack_catalog_tarfile(G, swi, ofd,
 			catalog_entry_directory,
 			pax_read_command,
 			0 /*alt_catalog_root*/,
@@ -1169,10 +1131,70 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		}
 	}
 
+	/* Analysis Phase and checremove script */
+	/* SWBIS_TS_Analysis_002 */
+
+	E_DEBUG("HERE");
+	do_check_abort = 0;
+	if (
+		G->g_opt_previewM ||
+		do_skip_entry ||
+		0
+	) {
+		ret = swpl_send_nothing_and_wait(swicol, ofd, G->g_swi_event_fd,
+                        SWBIS_TS_Analysis_002,
+			SWICOL_TL_8,
+			SW_SUCCESS);
+		if (ret != 0) {
+			looper_abort(G, swicol, ofd);
+			goto error_out;
+			retval = -1;
+		}
+	} else {
+		/*
+		 * TS_analysis_002
+		 */
+		E_DEBUG("HERE");
+		ret = swpl_run_check_script(G, SW_A_checkremove, SWBIS_TS_Analysis_002, swi, ofd, G->g_swi_event_fd, &checkremove_status);
+		if (ret < 0) {
+			/* internal error */
+			retval++;
+			E_DEBUG2("retval = %d", retval);
+		} else {
+			/* process the checkremove enforcement policy 
+			   based on the script exit status */
+			
+			if (checkremove_status == SW_SUCCESS) {
+				;	
+			} else if (checkremove_status == SW_WARNING) {
+				;
+			} else if (checkremove_status == SW_ERROR) {
+				if (do_enforce_scripts) {
+					do_check_abort = 1;
+				}
+			} else if (checkremove_status == SW_DESELECT) {
+				/* even if do_enforce_scripts == 0, enforce this status
+				   as meaning deselect, only if forced override */
+				if (G->g_force == 0) {
+					do_check_abort = 1;
+				}
+			} else if (checkremove_status == -1) {
+				/* OK, probably no checkremove script */
+				;
+			} else {
+				/* unrecognized status */
+				sw_e_msg(G, "unrecognized exit status for checkremove: %d\n", checkremove_status);
+				retval++;
+			}
+
+		}
+	}
+
 	/* SWBIS_TS_remove_files */
 	if (
 		G->g_opt_previewM ||
 		do_skip_entry ||
+		do_check_abort ||
 		0
 	) {
 		ret = swpl_send_nothing_and_wait(swicol, ofd, G->g_swi_event_fd,
@@ -1185,25 +1207,26 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			retval = -1;
 		}
 	} else {
-		ret = remove_files(G, swi, swicol, e, file_list, ofd);
+		E_DEBUG("remove files");
+		ret = remove_files(G, cisf, swi, swicol, e, file_list, ofd);
 		if (ret < 0) {
 			/* internal error */
+			E_DEBUG("remove files: internal error");
 			rstatus = 8;
 			retval = -1;
 			looper_abort(G, swicol, ofd);
 			goto error_out;
 		} else if (ret > 0) {
 			/* script error */
+			E_DEBUG("remove files: script error");
 			rstatus = 9;
-			/* retval = -1; */
-			/* looper_abort(G, swicol, ofd); */
 			goto error_out;
 		}
 	}
 
 	/* SWBIS_TS_Catalog_dir_remove
-	   This removes the .../export/ directory
-	   for example: var/lib/swbis/catalog/foo/foo/1.1/0/export */
+	   This removes the .../export/name-version  directory
+	   i.e. the opposite of Catalog_unpack */
 
 	if (
 		G->g_opt_previewM ||
@@ -1237,6 +1260,7 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	if (
 		G->g_opt_previewM ||
 		do_skip_entry ||
+		do_check_abort ||
 		0
 	) {
 		ret = swpl_send_nothing_and_wait(swicol, ofd, G->g_swi_event_fd,
@@ -1250,11 +1274,13 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		ret = swpl_remove_catalog_entry(G, swicol, e, ofd, NULL);
 		if (ret < 0) {
 			/* internal error */
+			E_DEBUG("remove catalog: internal error");
 			rstatus = 8;
 			retval = -1;
 			goto error_out;
 		} else if (ret > 0) {
 			/* script error */
+			E_DEBUG("remove catalog: script error");
 			rstatus = 9;
 		}
 	}
@@ -1269,6 +1295,7 @@ swremove_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	if (G->g_do_debug_events)
 		swicol_show_events_to_fd(swicol, STDERR_FILENO, -1);
+	E_DEBUG2("rp_status is now rstatus: %d", rstatus);
 	if (rstatus)
 		*p_rp_status = rstatus;
 	if (swi) swi_delete(swi);
