@@ -1,6 +1,6 @@
 /* swverify_lib.c -- routines for iswverify.
 
- Copyright (C) 2007,2008 James H. Lowe, Jr. 
+ Copyright (C) 2007,2008,2009,2010 James H. Lowe, Jr. 
  All Rights Reserved.
  
  COPYING TERMS AND CONDITIONS:
@@ -57,6 +57,8 @@
 
 #define SWREMOVE_ARG_MAX_THRESHOLD 900
 
+static INFOLINE AA;  /* dummy object used for sizing */
+
 static
 void
 print_lines(char * name, char * pref, STRAR * ar)
@@ -64,6 +66,7 @@ print_lines(char * name, char * pref, STRAR * ar)
 	int fd;
 	char * s;
 	int i;
+	E_DEBUG("START");
 	i = 0;
 
 	if (strcmp(name, "-") == 0) {
@@ -93,6 +96,7 @@ discern_name(char * line, int * len)
 	char * s;
 	char * delim =  "[linkname=";
 
+	E_DEBUG("START");
 	*len = 0;
 	s = strstr(line, delim);
 	if (s == NULL) {
@@ -114,6 +118,7 @@ remove_duplicate_files(STRAR * meta)
 	int len;
 	int prevlen;
 
+	E_DEBUG("START");
 	prevname = NULL;
 	prevlen = 0;
 	im = 0;
@@ -131,10 +136,10 @@ remove_duplicate_files(STRAR * meta)
 				   indicate the first file is the INFO
 				   file. */
 				if (name < prevname) {
-					fprintf(stderr, "REMOVING [%s] index %d\n", name, im-1);
+					/* fprintf(stderr, "REMOVING [%s] index %d\n", name, im-1); */
 					strar_remove_index(meta, im-2);
 				} else {
-					fprintf(stderr, "REMOVING [%s] index %d\n", name, im-2);
+					/* fprintf(stderr, "REMOVING [%s] index %d\n", name, im-2); */
 					strar_remove_index(meta, im-1);
 				}
 				im--;
@@ -250,8 +255,8 @@ looper_locked_region(STROB * buf, int vlv)
 		"				read swspec_string\n"                  /* SWBIS_DT_0001 */
 		"				%s\n" /* SW_SELECTION_EXECUTION_BEGINS */
 
-		"				# echo Processing Now \"$xxb\" 1>&2\n"
-		"				echo \"$xxb\"\n"			/* SWBIS_DT_0002 */
+		"				# echo Processing Now \"$catentdir\" 1>&2\n"
+		"				echo \"$catentdir\"\n"			/* SWBIS_DT_0002 */
 
 		"				tar cbf 1 - \"$catalog_entry_dir\" 2>/dev/null\n"
 		"				sw_retval=$?\n"
@@ -291,12 +296,19 @@ looper_locked_region(STROB * buf, int vlv)
 		"				sw_retval=$?\n"
 		"				swexec_status=$sw_retval\n"
 
+		"				#shls_bashin2 \"" SWBIS_TS_Load_management_host_status "\"\n"
+		"				#sw_retval=$?\n"
+		"				#case $sw_retval in 0) $sh_dash_s;; *) shls_false_;; esac\n"
+		"				#sw_retval=$?\n"
+		"				#swexec_status=$sw_retval\n"
+
 		"				shls_bashin2 \"" SWBIS_TS_post_verify "\"\n"
 		"				sw_retval=$?\n"
 		"				case $sw_retval in 0) $sh_dash_s;; *) shls_false_;; esac\n"
 		"				sw_retval=$?\n"
 		"				swexec_status=$sw_retval\n"
 
+
 		CSHID
 		"				%s\n" /* SW_SELECTION_EXECUTION_ENDS */
 		,
@@ -309,165 +321,615 @@ looper_locked_region(STROB * buf, int vlv)
 
 static
 int
-get_name_len(char * line)
+inl_get_name_len(char * line)
 {
+	int ret;
+	int status;
 	char * s;
 	char * l;
-
-	E_DEBUG("");
-	l = NULL;	
-	s = strstr(line, " " LS_LIST_LINKNAME_MARK);
-	while(s) {
-		E_DEBUG("");
-		/* FIXME: this is supposed to get the last occurance but
-			  is not immune to all attacks */
-		l = s;
-		s = strstr(s+1, " " LS_LIST_LINKNAME_MARK);
-	}
-	if (! l) {
-fprintf(stderr, "[%s]\n", line);
+	/* E_DEBUG2("LINE: [%s]", line); */
+	s = strstr(line,  LS_LIST_NAME_LENGTH);
+	if (!s) {
 		SWLIB_FATAL("");
 	}
-	E_DEBUG2("returning %d", (int)(l - line));
-	return (int)(l - line); /* length of the filename */
+	s = s + strlen(LS_LIST_NAME_LENGTH);
+	if (*s != '=') {
+		SWLIB_FATAL("");
+	}	
+	s++;
+	ret = swlib_atoi(s, &status);
+	if (status) {
+		SWLIB_FATAL("");
+	}
+	return ret;
+}
+
+static
+char *
+inl_get_name(char * line, int * plen)
+{
+	char * s;
+	s = strchr(line, ' ');
+	/* E_DEBUG2("LINE: [%s]", line); */
+	if (!s) SWLIB_FATAL("");
+	s++;
+	if (plen) *plen = inl_get_name_len(line);
+	return s;
+}
+
+
+static
+char *
+inl_get_linkname(char * line, int * plen)
+{
+	int len;
+	char * s;
+	int ret;
+	int status;
+	int linknamelen;
+
+	s = inl_get_name(line, &len);
+	s += len;
+	s = strstr(s,  LS_LIST_LINKNAME_LENGTH);
+	/* E_DEBUG2("LINE: [%s]", line); */
+	if (!s)  SWLIB_FATAL("");
+	s = s + strlen(LS_LIST_LINKNAME_LENGTH);
+	if (*s != '=')  SWLIB_FATAL("");
+	s++;
+	linknamelen = swlib_atoi(s, &status);
+	if (status) SWLIB_FATAL("");
+
+	s = strstr(s,  LS_LIST_LINKNAME_MARK);
+	if (!s)  SWLIB_FATAL("");
+	s = s + strlen(LS_LIST_LINKNAME_MARK);
+	if (plen) *plen = linknamelen;
+	return s;	
+}
+
+static
+char *
+inl_get_filesize(char * line, int * plen) {
+	int len;
+	char * ret;
+	char * e;
+	char * s;
+
+	/* E_DEBUG2("LINE: [%s]", line); */
+	s = inl_get_linkname(line, &len);
+	if (!s) SWLIB_FATAL("");
+	s = s + len;
+	s++; s++;
+	if (*s != ' ') SWLIB_FATAL("");
+	s++;
+	if (*s != '[') SWLIB_FATAL("");
+	s++;
+	ret = s;
+	e = s;
+	while (*e && *e != ']') e++;
+	if (*e != ']') SWLIB_FATAL("");
+	if (plen) *plen = (int)(e - s);
+	return ret;
+}
+
+static
+char *
+inl_get_permissions(char * line, int * plen)
+{
+	int len;
+	char * ret;
+	char * e;
+	char * s;
+
+	E_DEBUG2("LINE: [%s]", line);
+	s = inl_get_filesize(line, &len);
+	if (!s) SWLIB_FATAL("");
+	s = s + len;
+	if (*s != ']') SWLIB_FATAL("");
+	if (*(++s) != ' ') SWLIB_FATAL("");
+	if (*(++s) != '[') SWLIB_FATAL("");
+	s++;
+
+	/* Now at mode string */
+	ret = s;
+	e = s;
+	while (*e && *e != ']') e++;
+	if (*e != ']') SWLIB_FATAL("");
+	if (plen) *plen = (int)(e - s);
+	return ret;
+}
+
+static
+int
+inl_is_regfile(char * line)
+{
+	char * s;
+	int ret;
+	int len;
+	E_DEBUG2("LINE: [%s]", line);
+	s = inl_get_permissions(line, &len);
+	if (*s == '-')
+		return 1;
+	else
+		return 0;
+}
+
+static
+char *
+inl_get_ownernames(char * line, int * plen)
+{
+	int len;
+	int status;
+	char * ret;
+	char * e;
+	char * s;
+
+	E_DEBUG2("LINE: [%s]", line);
+	s = inl_get_permissions(line, &len);
+	if (!s) SWLIB_FATAL("");
+	s = s + len;
+	if (*s != ']') SWLIB_FATAL("");
+	if (*(++s) != ' ') SWLIB_FATAL("");
+	if (*(++s) != '[') SWLIB_FATAL("");
+	s++;
+
+	/* Now at "len=NNN" */
+	if (strstr(s, "len=") != s) SWLIB_FATAL("");
+	s += strlen("len=");
+
+	len = swlib_atoi(s, &status);
+	while (*s && *s != ' ') s++;
+	s++;
+	if (*s != '[') SWLIB_FATAL("");
+	s++;
+	if (plen) *plen = len;
+	ret = s;
+	return ret;
+}
+
+static
+char *
+inl_get_mdate(char * line, int * plen) {
+	int len;
+	char * ret;
+	char * e;
+	char * s;
+
+	E_DEBUG2("LINE: [%s]", line);
+	s = inl_get_ownernames(line, &len);
+	if (!s) SWLIB_FATAL("");
+	s = s + len;
+	s++; s++;
+	if (*s != ' ') SWLIB_FATAL("");
+	s++;
+	if (*s != '[') SWLIB_FATAL("");
+	s++;
+	ret = s;
+	e = s;
+	while (*e && *e != ']') e++;
+	if (*e != ']') SWLIB_FATAL("");
+	if (plen) *plen = (int)(e - s);
+	return ret;
+}
+
+static
+char *
+inl_get_md5(char * line, int * plen) {
+	int len;
+	char * ret;
+	char * e;
+	char * s;
+
+	s = inl_get_mdate(line, &len);
+	if (!s) SWLIB_FATAL("");
+	s = s + len;
+	s++;
+	if (*s != ' ') SWLIB_FATAL("");
+	s++;
+	if (*s != '[') SWLIB_FATAL("");
+	s++;
+	if (strstr(s, "MD5=") == s) 
+		s += strlen ("MD5=");
+	ret = s;
+	e = s;
+	while (*e && *e != ']') e++;
+	if (*e != ']') SWLIB_FATAL("");
+	if (plen) *plen = (int)(e - s);
+	return ret;
+}
+
+static
+char *
+inl_get_sha1(char * line, int * plen) {
+	int len;
+	char * ret;
+	char * e;
+	char * s;
+
+	s = inl_get_md5(line, &len);
+	if (!s) SWLIB_FATAL("");
+	while (*s && *s != ']') s++;
+	s++;
+	if (*s != ' ') SWLIB_FATAL("");
+	s++;
+	if (*s != '[') SWLIB_FATAL("");
+	s++;
+
+	if (strstr(s, "SHA1=") == s) 
+		s += strlen ("SHA1=");
+	ret = s;
+	e = s;
+	while (*e && *e != ']') e++;
+	if (*e != ']') SWLIB_FATAL("");
+	if (plen) *plen = (int)(e - s);
+	return ret;
+}
+
+static
+char *
+inl_get_sha512(char * line, int * plen) {
+	int len;
+	char * ret;
+	char * e;
+	char * s;
+
+	s = inl_get_sha1(line, &len);
+	if (!s) SWLIB_FATAL("");
+	while (*s && *s != ']') s++;
+	s++;
+	if (*s != ' ') SWLIB_FATAL("");
+	s++;
+	if (*s != '[') SWLIB_FATAL("");
+	s++;
+
+	if (strstr(s, "SHA512=") == s) 
+		s += strlen("SHA512=");
+	ret = s;
+	e = s;
+	while (*e && *e != ']') e++;
+	if (*e != ']') SWLIB_FATAL("");
+	if (plen) *plen = (int)(e - s);
+	return ret;
 }
 
 static
 int
 compare_names(char * mname, char * sname)
 {
+	char * mname_start;
+	char * sname_start;
+	char * s;
 	int ml;
 	int sl;
 	E_DEBUG("");
-	ml = get_name_len(mname);
+	ml = inl_get_name_len(mname);
 	E_DEBUG("");
-	sl = get_name_len(sname);
+	sl = inl_get_name_len(sname);
 	E_DEBUG("");
 	if (ml != sl) return 1;
 	E_DEBUG("");
-	return strncmp(mname, sname, ml);
+
+	s = strchr(mname, ' ');  /* the space after 'name_length=NNN' */
+	if (!s) SWLIB_FATAL("");	
+	mname_start = s + 1;
+	
+	s = strchr(sname, ' ');  /* the space after 'name_length=NNN' */
+	if (!s) SWLIB_FATAL("");	
+	sname_start = s + 1;
+
+	return strncmp(mname_start, sname_start, ml);
 }
 
 static
 int
-compare_diff(GB * G, char * host, char * swspec_string, char * target_path, STRAR * meta, STRAR * sys, int do_show)
+determine_inl_field_condition(INFOLINE * info_inl, INFOLINE * syst_inl, int offset)
+{
+	int ret;
+	INFOLINE * status_inl;
+	INFOITEM * ti = (INFOITEM*)(((char*)info_inl)+offset);
+	INFOITEM * ts = (INFOITEM*)(((char*)syst_inl)+offset);
+
+	ret = INL_STATUS_UNSET;
+
+	if ( 
+		/* if lengths are equal and field is identical */
+		ti->lenM > 0 &&
+		ti->lenM == ts->lenM &&
+		strncmp(ti->addrM, ts->addrM, ti->lenM) == 0
+	) {
+		ret = INL_STATUS_1_1_EQ;
+	} else if ( 
+		/* present and unequal */
+		ti->lenM > 0 &&
+		ts->lenM > 0 &&
+		strncmp(ti->addrM, ts->addrM, ti->lenM)
+	) {
+		ret = INL_STATUS_1_1_NEQ;
+	} else if ( 
+		ti->lenM == 0 &&
+		ts->lenM == 0
+	) {
+		ret = INL_STATUS_0_0_NA;
+	} else if ( 
+		/* present in INFO only */
+		ti->lenM > 0
+	) {
+		ret = INL_STATUS_1_0_NA;	
+	} else if ( 
+		/* present in SYSTEM only */
+		ts->lenM > 0
+	) {
+		ret = INL_STATUS_0_1_NA;
+	} else {
+		/* error, should never get here */
+		SWLIB_FATAL("");
+	}
+
+	/* return -2, if present in INFO and empty in system */
+	/* return -1, if present in INFO and system and unequal */
+	/* return 0, if identical */
+	/* return 1, if present in system, but missing from INFO */
+	/* return 2, missing from system and INFO */
+
+	return ret;
+}
+
+static
+int
+get_inl_field_status(INFOLINE * status_inl, int offset)
+{
+	int current_status;
+	INFOITEM * t = (INFOITEM*)(((char*)status_inl)+offset);
+	E_DEBUG("");
+	current_status = t->statusM;
+	return current_status;
+}
+
+static
+void
+set_inl_condition(INFOLINE * status_inl, int offset, int incoming_status)
+{
+	int current_status;
+	INFOITEM * t = (INFOITEM*)(((char*)status_inl)+offset);
+
+	E_DEBUG("");
+	current_status = t->statusM;
+
+	if ( incoming_status < current_status) {
+
+		/* By design, the worse the status is the lower the number,
+		   hence it is safe to overwrite a previous status with a higher number */
+
+		t->statusM = incoming_status;
+	}
+}
+
+static
+void
+inl_compare_infoline(INFOLINE * status_inl, int line_status, INFOLINE * info_inl, INFOLINE * system_inl)
+{
+	/* line_status: 0 means good, lines identical
+			1 means bad, mismatch */
+	int field_status;
+	E_DEBUG("");
+
+		/* find out which field mis-matched */
+
+		field_status = determine_inl_field_condition(info_inl, system_inl, INL_LINKNAME_OFFSET);
+		set_inl_condition(status_inl, INL_LINKNAME_OFFSET, field_status);
+
+		field_status = determine_inl_field_condition(info_inl, system_inl, INL_SIZE_OFFSET);
+		set_inl_condition(status_inl, INL_SIZE_OFFSET, field_status);
+
+		field_status = determine_inl_field_condition(info_inl, system_inl, INL_OWNERS_OFFSET);
+		set_inl_condition(status_inl, INL_OWNERS_OFFSET, field_status);
+
+		field_status = determine_inl_field_condition(info_inl, system_inl, INL_PERMISSIONS_OFFSET);
+		set_inl_condition(status_inl, INL_PERMISSIONS_OFFSET, field_status);
+
+		field_status = determine_inl_field_condition(info_inl, system_inl, INL_MDATE_OFFSET);
+		set_inl_condition(status_inl, INL_MDATE_OFFSET, field_status);
+
+		field_status = determine_inl_field_condition(info_inl, system_inl, INL_MD5_OFFSET);
+		set_inl_condition(status_inl, INL_MD5_OFFSET, field_status);
+
+		field_status = determine_inl_field_condition(info_inl, system_inl, INL_SHA1_OFFSET);
+		set_inl_condition(status_inl, INL_SHA1_OFFSET, field_status);
+
+		field_status = determine_inl_field_condition(info_inl, system_inl, INL_SHA512_OFFSET);
+		set_inl_condition(status_inl, INL_SHA512_OFFSET, field_status);
+	E_DEBUG("");
+	return;
+}
+
+
+static
+char *
+inl_status_msg(int status)
+{
+
+/*
+#define INL_STATUS_UNSET        5  
+#define INL_STATUS_0_0_NA       4  
+#define INL_STATUS_1_0_NA       3 
+#define INL_STATUS_1_1_EQ       2 
+#define INL_STATUS_0_1_NA       1 
+#define INL_STATUS_1_1_NEQ      0 
+*/
+
+	if (status == INL_STATUS_1_1_EQ) return "PASSED";
+	if (status == INL_STATUS_1_1_NEQ) return "FAILED";
+	if (status == INL_STATUS_0_1_NA) return  "MISSING FROM INFO File";
+	if (status == INL_STATUS_0_0_NA) return  "Not Checked";
+	if (status == INL_STATUS_1_0_NA) return  "MISSING FROM SYSTEM";
+	if (status == INL_STATUS_MISSING_FILE) return  "ONE OR MORE FILES MISSING";
+	if (status == INL_STATUS_UNSET) return  "Not determined";
+	return "Internal Error";
+}
+
+static
+void	
+write_summary_report(GB * G, INFOLINE * status_inl, STROB * rpt)
+{
+	int ret;
+	strob_strcpy(rpt, "");
+	
+	ret = get_inl_field_status(status_inl, INL_NAME_OFFSET);
+	strob_sprintf(rpt, 1, "%s: INFO check: Fileset file list: %s\n", swlib_utilname_get(), inl_status_msg(ret));
+
+	ret = get_inl_field_status(status_inl, INL_SIZE_OFFSET);
+	strob_sprintf(rpt, 1, "%s: INFO check: file sizes: %s\n", swlib_utilname_get(), inl_status_msg(ret));
+
+	ret = get_inl_field_status(status_inl, INL_OWNERS_OFFSET);
+	strob_sprintf(rpt, 1, "%s: INFO check: POSIX.1 file ownerships: %s\n", swlib_utilname_get(), inl_status_msg(ret));
+
+	ret = get_inl_field_status(status_inl, INL_MDATE_OFFSET);
+	strob_sprintf(rpt, 1, "%s: INFO check: File Modification times: %s\n", swlib_utilname_get(), inl_status_msg(ret));
+
+	ret = get_inl_field_status(status_inl, INL_PERMISSIONS_OFFSET);
+	strob_sprintf(rpt, 1, "%s: INFO check: POSIX.1 File permissions: %s\n", swlib_utilname_get(), inl_status_msg(ret));
+	
+	ret = get_inl_field_status(status_inl, INL_MD5_OFFSET);
+	strob_sprintf(rpt, 1, "%s: INFO check: File Contents MD5 digests: %s\n", swlib_utilname_get(), inl_status_msg(ret));
+	
+	ret = get_inl_field_status(status_inl, INL_SHA1_OFFSET);
+	strob_sprintf(rpt, 1, "%s: INFO check: File Contents SHA1 digests: %s\n", swlib_utilname_get(), inl_status_msg(ret));
+	
+	ret = get_inl_field_status(status_inl, INL_SHA512_OFFSET);
+	strob_sprintf(rpt, 1, "%s: INFO check: File Contents SHA512 digests: %s\n", swlib_utilname_get(), inl_status_msg(ret));
+}
+
+static
+int
+compare_diff(GB * G, STROB * rpt_diff, STROB * rpt_summary, STRAR * meta, STRAR * sys, int do_show)
 {
 	int retval;
 	char * m_s;
 	char * s_s;
+	char * name;
+	int name_len;
 	int did_print_header;
-	STROB * diff_header;
-	STROB * diff;
+	INFOLINE * m_s_inl;
+	INFOLINE * s_s_inl;
+	INFOLINE * status_inl;
 	int is;
 	int im;
 
+	E_DEBUG("START");
 	retval = 0;
 	did_print_header = 0;
 
-	diff_header = strob_open(80);
-	diff = strob_open(80);
-
-	strob_sprintf(diff_header, 0,
-			"1c1\n"
-			"< INFO %s @ %s:%s using  installed_software_catalog=%s\n"
-			"---\n"
-			"> SYSTEM located at %s:%s\n",
-		 	swspec_string, host, target_path, get_opta(G->optaM, SW_E_installed_software_catalog),
-		 	host, target_path
-			);
+	strob_strcpy(rpt_diff, "");
+	strob_strcpy(rpt_summary, "");
 
 	/* Make a psuedo diff-like output, this is easy because
 	   we can assume that the files are sorted, and, only files will
 	   be missing from the system list (STRAR*)sys or else they will
 	   be changed 1-to-1 with (STRAR*)meta */
 
+	status_inl = swverify_inl_create();
+	m_s_inl = swverify_inl_create();
+	s_s_inl = swverify_inl_create();
+
 	im = 0;
 	is = 0;
 	m_s = strar_get(meta, im);
 	s_s = strar_get(sys, is);
+	status_inl->nameM.statusM = INL_STATUS_1_1_EQ;
 	while (m_s) {
+		E_DEBUG2("NAME m_s=%s", m_s);
+		if (G->g_ignore_slack_installM) {
+			name = inl_get_name(m_s, &name_len);		
+			SWLIB_ASSERT(name != NULL);
+			SWLIB_ASSERT(name_len > 0);
+			if (
+			 	(strstr(name, "./install") == name && name_len == 9) ||
+			 	strstr(name, "./install/") == name
+			) {
+				im++;
+				E_DEBUG("goto slack_end");
+				goto slack_end;
+			}
+		}
+		/* parse the data to prove its what we expect */
 		E_DEBUG("");
-		if (
-			G->g_ignore_slack_installM &&
-			(
-			 	strstr(m_s, "./install/") == m_s ||
-				strstr(m_s, "./install ") == m_s
-			)
-		) {
-			im++;
-		} else if (m_s && s_s && strcmp(m_s, s_s) == 0) {
+		if (G->g_no_summary_reportM == 0) {
+			if (m_s) swverify_inl_parse(m_s_inl, m_s); 
+			if (s_s) swverify_inl_parse(s_s_inl, s_s); 
+		}
+
+		if (m_s && s_s && strcmp(m_s, s_s) == 0) {
 			/* good, lines match */
-			im++;			
-			is++;			
+			im++;
+			is++;
 			E_DEBUG("match");
 			if (do_show) {
-				strob_sprintf(diff, 0,
+				strob_sprintf(rpt_diff, 1/*do append*/,
 					"= %s\n",
 		 			s_s);
-				fprintf(stdout, "%s", strob_str(diff));
 			}
-			;
+			if (G->g_no_summary_reportM == 0)
+				inl_compare_infoline(status_inl, 0, m_s_inl, s_s_inl);
 		} else if (m_s && s_s && strcmp(m_s, s_s) != 0) {
-			/* either changed of deleted file */
-			if (did_print_header++ == 0) {
-				fprintf(stderr, "%s", strob_str(diff_header));	
-			}
+			/* either changed or deleted file */
+			retval++;
 			E_DEBUG("mis match");
 			if (compare_names(m_s, s_s) == 0) {
 				/* changed file */
-				strob_sprintf(diff, 0,
+				if (G->g_no_summary_reportM == 0)
+					inl_compare_infoline(status_inl, 1, m_s_inl, s_s_inl);
+				strob_sprintf(rpt_diff, 1,
 					"%dc%d\n"
 					"< %s\n"
 					"---\n"
 					"> %s\n",
 		 			im, im, m_s, s_s);
-				fprintf(stdout, "%s", strob_str(diff));
 				im++;
 				is++;
 			} else {
 				/* missing file */
 				E_DEBUG("missing file");
-				if (did_print_header++ == 0) {
-					fprintf(stderr, "%s", strob_str(diff_header));	
-				}
-				strob_sprintf(diff, 0,
+				status_inl->nameM.statusM = INL_STATUS_MISSING_FILE;
+				strob_sprintf(rpt_diff, 1 /*do append */,
 					"%dd%d\n"
 					"< %s\n",
 	 				im, is, m_s);
-				fprintf(stdout, "%s", strob_str(diff));
 				im++;
 			}
 			E_DEBUG("diff");
-			retval++;
 		} else if (m_s && !s_s) {
 			/* files missing at end */
-			if (did_print_header++ == 0) {
-				fprintf(stderr, "%s", strob_str(diff_header));	
-			}
-			strob_sprintf(diff, 0,
+			retval++;
+			status_inl->nameM.statusM = INL_STATUS_MISSING_FILE;
+			strob_sprintf(rpt_diff, 1 /*do append*/,
 				"%dd%d\n"
 				"< %s\n",
 	 			im, is, m_s);
-			fprintf(stdout, "%s", strob_str(diff));
-			retval++;
 			im++;
 			E_DEBUG("missing2");
 		} else {
 			E_DEBUG("never here");
+			retval++;
 			im++;
 		}
+slack_end:
 		m_s = strar_get(meta, im);
 		s_s = strar_get(sys, is);
+		E_DEBUG("");
 	}
 	E_DEBUG2("retunring %d", retval);
-	strob_close(diff);
-	strob_close(diff_header);
+
+	if (G->g_no_summary_reportM == 0)
+		write_summary_report(G, status_inl, rpt_summary);
+
+	swverify_inl_delete(status_inl);
+	swverify_inl_delete(m_s_inl);
+	swverify_inl_delete(s_s_inl);
 	return retval;
 }
 
 static
 int
-compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, SWI * swi, int archive_fd, int * compare_status)
+compare_fileset(GB * G, STROB * rpt_diff, STROB * rpt_summary, SWI * swi, int archive_fd, int * p_compare_status)
 {
 	int nullfd;
 	int ret;
@@ -489,21 +951,25 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 	int check_permissions;
 	int check_contents;
 	int compare_status_is_set;
+	int check_owners;
 
+	E_DEBUG("START");
 	compare_status_is_set = 0;
-	*compare_status = -1;
+	*p_compare_status = -1;
 	meta_fd = swlib_open_memfd();
 	sys_fd = swlib_open_memfd();
-	nullfd = swlib_open_nullfd();
+	nullfd = swbis_devnull_open("/dev/null", O_RDWR, 0);
 	tmp = strob_open(100);
 
 	available_attributes = 0;
 	/* list the catalog metadata */
 
+	E_DEBUG("");
 	check_volatile = swextopt_is_option_true(SW_E_check_volatile, G->optaM);
 	check_mtime = swextopt_is_option_true(SW_E_swbis_check_mtime, G->optaM);
 	check_permissions = swextopt_is_option_true(SW_E_check_permissions, G->optaM);
 	check_contents = swextopt_is_option_true(SW_E_check_contents, G->optaM);
+	check_owners = swextopt_is_option_true(SW_E_swbis_check_owners, G->optaM);
 
 	if (check_contents == 0)
 		check_mtime = 0; /* per spec */
@@ -517,6 +983,15 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 		ls_attributes &= ~(LS_LIST_VERBOSE_WITH_REG_DATES|LS_LIST_VERBOSE_WITH_ALL_DATES);
 	}
 
+	E_DEBUG("");
+	if (!check_owners)
+		ls_attributes |= (LS_LIST_VERBOSE_WITHOUT_OWNERS);
+	
+	E_DEBUG("");
+	if (!check_permissions)
+		ls_attributes |= (LS_LIST_VERBOSE_WITHOUT_PERMISSIONS);
+	
+	E_DEBUG("");
 	swpl_ls_fileset_from_iscat(swi, meta_fd, ls_attributes,
 			&available_attributes, !check_volatile, check_contents);
 
@@ -538,13 +1013,22 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 		available_attributes &= ~(LS_LIST_VERBOSE_WITH_REG_DATES|LS_LIST_VERBOSE_WITH_ALL_DATES);
 	}
 
+	if (!check_owners)
+		available_attributes |= (LS_LIST_VERBOSE_WITHOUT_OWNERS);
+
+	if (!check_permissions)
+		available_attributes |= (LS_LIST_VERBOSE_WITHOUT_PERMISSIONS);
+
+	E_DEBUG("");
 	xformat = xformat_open(-1, -1, arf_ustar);
 	xformat_set_ifd(xformat, archive_fd);
 
+	E_DEBUG("");
 	file_hdr = ahs_vfile_hdr(xformat_ahs_object(xformat));
 	taru_init_header_digs(file_hdr);
 	digs = file_hdr->digsM;
 
+	E_DEBUG("");
 	if (check_contents == 0) {
 		digs->do_md5 = DIGS_ENABLE_OFF;
 		digs->do_sha1 = DIGS_ENABLE_OFF;
@@ -574,7 +1058,7 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 			/* The the package contains no digests, verification
 			   must therefore fail */
 			compare_status_is_set = 1;
-			*compare_status = 1;
+			*p_compare_status = 1;
 			fprintf(stderr, "%s: error: the package does not contain file digests\n", swlib_utilname_get());
 		}
 
@@ -598,9 +1082,11 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 		uxfio_write(sys_fd, strob_str(tmp), strob_strlen(tmp));
 	}
 	
+	E_DEBUG("");
 	meta_text = uxfio_get_fd_mem(meta_fd, (int *)NULL);
 	sys_text = uxfio_get_fd_mem(sys_fd, (int *)NULL);
 
+	E_DEBUG("");
 	/* split into lines */
 	meta_str = linize(meta_text);
 	sys_str = linize(sys_text);
@@ -618,21 +1104,22 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 	remove_duplicate_files(meta_str);
 
 	/* Now compare these line by line */
-	ret = compare_diff(G, G->g_target_terminal_host, swspec_string,
-				target_path, meta_str, sys_str, G->g_do_show_linesM);
+	ret = compare_diff(G, rpt_diff, rpt_summary, meta_str, sys_str, G->g_do_show_linesM);
 
 	if (compare_status_is_set == 0) {
 		compare_status_is_set = 1;
-		*compare_status = ret;
+		*p_compare_status = ret;
 	}
 
+	E_DEBUG("");
 	strar_close(meta_str);
 	strar_close(sys_str);
 	uxfio_close(sys_fd);
 	uxfio_close(meta_fd);
-	uxfio_close(nullfd);
+	swbis_devnull_close(nullfd);
 	strob_close(tmp);
 	xformat_close(xformat);
+	E_DEBUG("");
 	return 0;
 }
 
@@ -771,7 +1258,7 @@ swverify_write_source_copy_script2(GB * G,
 	/* Make a message for the SOURCE_ACCESS_BEGINS event when
 	   reading the catalog */
 
-	strob_sprintf(isc_msg, 0, "installed catalog at %s", get_opta(G->optaM, SW_E_installed_software_catalog));
+	strob_sprintf(isc_msg, 0, "installed catalog at %s", get_opta_isc(G->optaM, SW_E_installed_software_catalog));
 
 
 	/* Make a final adjustment if the path is '/', FIXME */
@@ -1027,28 +1514,35 @@ swverify_write_source_copy_script2(GB * G,
 
 int
 swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
-		SWICAT_SC * sc, SWICAT_SR * sr, int ofd, int ifd, int * p_rp_status,
-		SWUTS * uts, char * pax_write_command_key, int * compare_status)
+		SWICAT_SC * sc, SWICAT_SR * entry_sr, int ofd, int ifd, int * p_rp_status,
+		SWUTS * uts, char * pax_write_command_key, int * p_compare_status)
 {
 	int retval;
 	int rstatus;
 	int ret;
 	int do_skip_entry;
+	char * ctmp;
 	char * pax_read_command;
 	char * epath;
 	char * installed_software_catalog;
 	char * catalog_entry_directory;
+	int result;
+	int sig_level;
 	STROB * tmp;
 	STROB * btmp;
 	STROB * btmp2;
 	STROB * swspec_string;
+	STROB * rpt_diff;
+	STROB * rpt_summary;
+	STROB * diff_header;
 	SWICAT_E * e;
 	SWI * swi;
 	SWI_FILELIST * file_list;
 	SWGPG_VALIDATE * swgpg;
 	int archive_fd;
 	
-	*compare_status = -1;
+	E_DEBUG("START");
+	*p_compare_status = -1;
 	archive_fd = -1;
 	do_skip_entry = 0;
 	retval = 0;
@@ -1057,21 +1551,29 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	btmp = strob_open(100);
 	btmp2 = strob_open(100);
 	swspec_string = strob_open(100);
+	rpt_diff = strob_open(100);
+	rpt_summary = strob_open(100);
+	diff_header = strob_open(100);
 	swi = NULL;
 	file_list = NULL;
 	swgpg = NULL;
 	e = NULL;
 
+	sig_level = swlib_atoi(get_opta(G->optaM, SW_E_swbis_sig_level), &result);
+	if (result) {
+		sig_level = 0;
+	}
+
 	pax_read_command = swc_get_pax_read_command(G->g_pax_read_commands,
 		"tar", G->g_verboseG >= SWC_VERBOSE_5,
 		0 /*keep_old_files*/, DEFAULT_PAX_R);
 	E_DEBUG("");
-	installed_software_catalog = get_opta(G->optaM, SW_E_installed_software_catalog);
-	catalog_entry_directory = swicat_sr_form_catalog_path(sr, installed_software_catalog, NULL);
+	installed_software_catalog = get_opta_isc(G->optaM, SW_E_installed_software_catalog);
+	catalog_entry_directory = swicat_sr_form_catalog_path(entry_sr, installed_software_catalog, NULL);
 
 	E_DEBUG2("catalog_entry_directory = [%s]", catalog_entry_directory);
 
-	swicat_sr_form_swspec(sr, swspec_string);
+	swicat_sr_form_swspec(entry_sr, swspec_string);
 
 	*p_rp_status = 0;
 	if (strlen(catalog_entry_directory) == 0) {
@@ -1174,21 +1676,15 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 
 	/* e->entry_prefixM has the installed_software_catalog
 	  path already prefixed */
-	epath = swicat_e_form_catalog_path(e, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_ACTIVE_ENTRY);
+	epath = swicat_e_form_catalog_path(e, btmp, NULL, SWICAT_ACTIVE_ENTRY);
 	if (G->devel_verboseM)
 		SWLIB_INFO2("catalog entry (active) path = %s", epath);
 	
-	epath = swicat_e_form_catalog_path(e, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_DEACTIVE_ENTRY);
+	epath = swicat_e_form_catalog_path(e, btmp, NULL, SWICAT_DEACTIVE_ENTRY);
 	if (G->devel_verboseM)
 		SWLIB_INFO2("catalog entry path (inactive) path = %s", epath);
 
-	epath = swicat_e_form_catalog_path(e, btmp,
-			NULL /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_ACTIVE_ENTRY);
+	epath = swicat_e_form_catalog_path(e, btmp, NULL, SWICAT_ACTIVE_ENTRY);
 
 	/* epath is now a clean relative path to such as:
 		var/lib/swbis/catalog/emacs/emacs/21.2.1/0  */
@@ -1211,7 +1707,10 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	swgpg = swgpg_create();
 	if (G->devel_verboseM)
 		SWLIB_INFO2("NOPEN=%d", uxfio_uxfio_get_nopen());
-	ret = swicat_e_verify_gpg_signature(e, swgpg);
+
+	ret = 0;	
+	if (sig_level >= 0)
+		ret = swicat_e_verify_gpg_signature(e, swgpg);
 	if (ret != 0) {
 		rstatus = 3;
 		retval = 0;
@@ -1222,7 +1721,9 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	   requirements */
 
 	E_DEBUG("");
-	ret = swpl_signature_policy_accept(G, swgpg, G->g_verboseG, strob_str(swspec_string));
+	ret = 0;
+	if (sig_level >= 0)
+		ret = swpl_signature_policy_accept(G, swgpg, G->g_verboseG, strob_str(swspec_string));
 	if (ret != 0) {
 		/* Bad signature or not enough good signatures
 		   Can't use the blob to make a removal file list
@@ -1251,8 +1752,16 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	}
 
 	E_DEBUG("");
-	if (swi) swi->swi_pkgM->catalog_entryM = strdup(epath); /* FIXME, this must set explicitly */
-	if (swi) swi->swi_pkgM->target_pathM = strdup(target_path); /* FIXME, this must set explicitly */
+	if (swi) {
+		swi->swi_pkgM->catalog_entryM = strdup(epath); /* FIXME, this must set explicitly */
+		swi->swi_pkgM->target_pathM = strdup(target_path); /* FIXME, this must set explicitly */
+
+		ctmp = swverid_get_verid_value(entry_sr->swspecM, SWVERID_VERIDS_LOCATION, 1);
+		if (ctmp) swi->swi_pkgM->locationM = strdup(ctmp);
+
+		ctmp = swverid_get_verid_value(entry_sr->swspecM, SWVERID_VERIDS_QUALIFIER, 1);
+		if (ctmp) swi->swi_pkgM->qualifierM = strdup(ctmp);
+	}
 
 	if (G->devel_verboseM)
 		SWLIB_INFO2("NOPEN=%d", uxfio_uxfio_get_nopen());
@@ -1341,7 +1850,7 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 			retval = -1;
 		}
 	} else {
-		ret = swpl_unpack_catalog_tarfile(swi, ofd,
+		ret = swpl_unpack_catalog_tarfile(G, swi, ofd,
 			catalog_entry_directory,
 			pax_read_command,
 			0 /*alt_catalog_root*/,
@@ -1409,13 +1918,36 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	E_DEBUG("");
 
 	if (rstatus) {
-		*compare_status = 1;
+		*p_compare_status = 1;
 		ret = 0;
 	} else {
 		if (archive_fd > 0) {
 			/* compare the archive to the catalog */
-			ret = compare_fileset(G, strob_str(swspec_string), target_path, uts, swi, archive_fd, compare_status);
+			ret = compare_fileset(G, rpt_diff, rpt_summary, swi, archive_fd, p_compare_status);
 			uxfio_close(archive_fd);
+
+			strob_sprintf(diff_header, 0,
+				"1c1\n"
+				"< INFO %s @ %s:%s using  installed_software_catalog=%s\n"
+				"---\n"
+				"> SYSTEM located at %s:%s\n",
+			 	strob_str(swspec_string), G->g_target_terminal_host, target_path,
+				get_opta_isc(G->optaM, SW_E_installed_software_catalog),
+			 	G->g_target_terminal_host, target_path
+				);
+
+			if (*p_compare_status) {
+				fprintf(stderr, "%s", strob_str(diff_header));
+				fprintf(stderr, "%s", strob_str(rpt_diff));
+			} else {
+				if (G->g_do_show_linesM) {
+					fprintf(stdout, "%s", strob_str(rpt_diff));
+				}
+			}
+
+			fprintf(stderr, "%s", strob_str(rpt_summary));
+
+
 		}
 	}
 	archive_fd = -1;
@@ -1454,6 +1986,28 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 		}
 	}
 
+	/* DISABLED; NOT USED   SWBIS_TS_Load_management_host_status */
+	if (0) {
+	if (
+		G->g_opt_previewM ||
+		do_skip_entry ||
+		0
+	) {
+		ret = swpl_send_nothing_and_wait(swicol, ofd, G->g_swi_event_fd,
+                        SWBIS_TS_Catalog_dir_remove,
+			SWICOL_TL_8,
+			SW_SUCCESS);
+		if (ret != 0) {
+			looper_abort(G, swicol, ofd);
+			goto error_out;
+			retval = -1;
+		}
+        } else {
+		ret = swpl_load_single_status_value (G, swi, ofd, G->g_swi_event_fd,
+			SWBIS_TS_Load_management_host_status, 0 /**p_compare_status*/);
+        } 
+	}
+
 	E_DEBUG("");
 	/* SWBIS_TS_post_verify */
 	if (
@@ -1481,6 +2035,9 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	strob_close(btmp);
 	strob_close(btmp2);
 	strob_close(swspec_string);
+	strob_close(rpt_diff);
+	strob_close(rpt_summary);
+	strob_close(diff_header);
 	E_DEBUG("");
 	if (file_list)
 		swi_fl_delete(file_list);
@@ -1497,3 +2054,104 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	E_DEBUG("");
 	return retval;
 }
+
+static
+void
+init_infoitem(INFOLINE * inl, int offset)
+{
+	INFOITEM * it;
+	it = (INFOITEM*)((char*)inl + offset);
+	it->addrM = NULL;
+	it->lenM = 0;
+	it->statusM = INL_STATUS_UNSET;
+}
+
+INFOLINE *
+swverify_inl_create(void)
+{
+	INFOLINE * inl;
+	inl = (INFOLINE*)malloc(sizeof(INFOLINE));
+
+	init_infoitem(inl, INL_NAME_OFFSET);
+	init_infoitem(inl, INL_LINKNAME_OFFSET);
+	init_infoitem(inl, INL_SIZE_OFFSET);
+	init_infoitem(inl, INL_PERMISSIONS_OFFSET);
+	init_infoitem(inl, INL_OWNERS_OFFSET);
+	init_infoitem(inl, INL_MDATE_OFFSET);
+	init_infoitem(inl, INL_MD5_OFFSET);
+	init_infoitem(inl, INL_SHA1_OFFSET);
+	init_infoitem(inl, INL_SHA512_OFFSET);
+
+	return inl;
+}
+
+void
+swverify_inl_delete(INFOLINE * inl) {
+	free((void*)inl);
+}
+
+int
+swverify_inl_parse(INFOLINE * inl, char * line) {
+	char * start;
+	char * s;	
+	char * e;
+	int len;
+
+	/* filename */
+	s = inl_get_name(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->nameM.addrM = s;
+	inl->nameM.lenM = len;
+	E_DEBUG3("linkname: %*s", len, s);
+	
+	/* linkname */
+	s = inl_get_linkname(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->linknameM.addrM = s;
+	inl->linknameM.lenM = len;
+	E_DEBUG3("name: %*s", len, s);
+
+	/* filesize */
+	s = inl_get_filesize(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->sizeM.addrM = s;
+	inl->sizeM.lenM = len;
+
+	/* mode string */
+	s = inl_get_permissions(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->permissionsM.addrM = s;
+	inl->permissionsM.lenM = len;
+
+	/* owner names and ids */
+	s = inl_get_ownernames(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->ownersM.addrM = s;
+	inl->ownersM.lenM = len;
+
+	/* modification date */
+	s = inl_get_mdate(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->mdateM.addrM = s;
+	inl->mdateM.lenM = len;
+
+	/* md5 */
+	s = inl_get_md5(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->md5M.addrM = s;
+	inl->md5M.lenM = len;
+
+	/* sha1 */
+	s = inl_get_sha1(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->sha1M.addrM = s;
+	inl->sha1M.lenM = len;
+
+	/* sha512 */
+	s = inl_get_sha512(line, &len);
+	if (!s) SWLIB_FATAL("");
+	inl->sha512M.addrM = s;
+	inl->sha512M.lenM = len;
+
+	return 0;
+}
