@@ -23,9 +23,10 @@
 #ifndef swexcat_1_h
 #define swexcat_1_h
 
-extern "C" {
 #define FILENEEDDEBUG 1
-#undef FILENEEDDEBUG
+#undef FILENEEDDEBUG 
+
+extern "C" {
 #include "swuser_config.h"
 #include "swextopt.h"
 #include "swuser_assert_config.h"
@@ -48,12 +49,16 @@ extern "C" {
 class swExHost;
 class swExDistribution;
 
-/*
-// #define E_DEBUG(format) SWBISERROR("DEBUG", format)
-// #define E_DEBUG2(format, arg) SWBISERROR2("DEBUG", format, arg)
-// #define E_DEBUG3(format, arg, arg1) \
-// 			SWBISERROR3("DEBUG", format, arg, arg1)
-*/
+#ifdef FILENEEDDEBUG
+#define E_DEBUG(format) SWBISERROR("DEBUG", format)
+#define E_DEBUG2(format, arg) SWBISERROR2("DEBUG", format, arg)
+#define E_DEBUG3(format, arg, arg1) \
+ 			SWBISERROR3("DEBUG", format, arg, arg1)
+#else
+#define E_DEBUG(format)
+#define E_DEBUG2(format, arg)
+#define E_DEBUG3(format, arg, arg1)
+#endif
 
 class swExCat: public swObjFiles_i
 {
@@ -200,7 +205,6 @@ class swExCat: public swObjFiles_i
 		current_swmd = static_cast<swDefinition*>(swmd);
 		setReferer(static_cast<swDefinition*>(current_swmd));
 		last_def = current_swmd;
-	
 		E_DEBUG("ENTERING");
 		E_DEBUG3("this=[%s] parserline address=[%p]", this->getObjectName(), current_swmd->get_parserline());
 
