@@ -748,7 +748,7 @@ class swObjFiles_i:  public swExStruct_i
 			return;
 		}
 
-		size = info->swdeffile_linki_write_fd(SWBIS_NULL_FD);
+		size = info->swdeffile_linki_write_fd(UXFIO_NULL_FD);
 
 		SWLIB_ASSERT(size > 0);
 		if (size <= 0) {
