@@ -53,6 +53,8 @@ write_fd (int uxfio_fd) {
    int eret;
    int ret = 0; 
    char * kw;
+
+   squash_duplicates();
  
    if (swAttribute::debug_writeM) {
    	swDefinition::write_fd_debug(uxfio_fd, "");
@@ -80,7 +82,7 @@ write_fd (int uxfio_fd) {
 		ctime_r(&tm, ctbuf);
 		if (strchr(ctbuf, '\n')) *strchr(ctbuf, '\n') = '\0';
 		if (strchr(ctbuf, '\r')) *strchr(ctbuf, '\r') = '\0';
-		strob_sprintf(tmp, 0, "  " SW_A_create_time " %s # %s UTC\n", calstring, ctbuf);
+		strob_sprintf(tmp, 0, SW_A_create_time " %s # %s UTC\n", calstring, ctbuf);
 		if (oldtzenv) putenv(oldtzenv);
         	eret = uxfio_write(uxfio_fd, strob_str(tmp), strob_strlen(tmp));
 		strob_close(tmp);
