@@ -627,6 +627,7 @@ public: // =====================================================================
 	int
 	processExcludes(swDefinitionFile* psf, char * value) {
 		int i;
+		STROB * f_source;
 		STROB * excl_pat;
 		STRAR * pat_list;
 		swDefinition * swdef;
@@ -637,23 +638,86 @@ public: // =====================================================================
 		int is_dir;
 		int t_dflag;
 		int l_dflag;
+		int does_have_trailing_slash;
 		int first_match;
 		char * type;
 
+		does_have_trailing_slash = 0;
 		is_dir = 0;
 		t_dflag = 0;
 		l_dflag = 0;
 		i = excl_sindexM;
 		first_match = 0;
+		f_source = strob_open(100);
 		excl_pat = strob_open(100);
 		pat_list = strar_open();
+
+
 	
+		SWEXTDEF_DEBUG2("value is [%s]", value);
+
+		//
+		// /* This just cleans up 'value' of extraneous whitespace etc.*/
+		// /* 'isource' is not used */
+		//
 		parse_SourcePath(value, &source, &isource);
 
+		SWEXTDEF_DEBUG2("source is now [%s]", source);
+	
+		//
+		// 'source' is the name to exclude from the PSF
+		//
+		// Apply a special case of trailing slash, DIR/, which is treated as DIR/*
+		//
+		if (source && strlen(source) && source[strlen(source)-1] == '/') {
+			//
+			// we will treat this as a pattern to exclude the 
+			// entire directory contents and the directory entry itself
+			//
+			does_have_trailing_slash = 1;
+		} else {
+			//
+			// do not augment with .../* to exclude contents
+			// if indeed the source is a directory
+			//
+			;
+		}
+
+		//
+		// Now form the pattern
+		//
+
+		//
+		// filter out the common path components between
+		// source and dest_path of the directory mapping directive, if active.
+		//
+
+		if (directoryDestActive() && *source == '/') {
+			//
+			// If directoryDestActive() AND the exclude source is an absolute path
+			// then strip away the leading commong component
+			//
+			char * t;
+
+			t = strstr(source, strob_str(directory_pathM));
+			if (t) {
+				t += strob_strlen(directory_pathM);
+			} else {
+				t = source;
+			}
+			strob_strcpy(f_source, t);	
+		} else {
+			strob_strcpy(f_source, source);	
+		}
+
+		SWEXTDEF_DEBUG3("concatenating [%s] [%s]", strob_str(excl_pat), strob_str(f_source));
 		strob_strcpy(excl_pat, strob_str(directory_sourceM));
-		swlib_unix_dircat(excl_pat, source);
+		swlib_unix_dircat(excl_pat, strob_str(f_source));
 		swlib_toggle_trailing_slashdot("drop", strob_str(excl_pat), &t_dflag);
 		swlib_toggle_leading_dotslash("drop", strob_str(excl_pat), &l_dflag);
+		
+		SWEXTDEF_DEBUG2("excl_pat is now [%s]", strob_str(excl_pat));
+		strar_add(pat_list, strob_str(excl_pat));
 
 		while ((swdef=psf->swdeffile_get_pointer_from_index(i++)) != static_cast<swDefinition*>(NULL)) {
 			if(strcmp("file", swdef->get_keyword())) {
@@ -682,7 +746,6 @@ public: // =====================================================================
 			}
 
 			SWEXTDEF_DEBUG3("comparing for exclusion : fnmatch [%s] [%s]", strob_str(excl_pat), mdef_source);
-			strar_add(pat_list, strob_str(excl_pat));
 			// if (fnmatch(strob_str(excl_pat), mdef_source, 0) == 0) {
 			if (process_fnmatch_list(pat_list, mdef_source) == 0) {
 				//
@@ -697,8 +760,9 @@ public: // =====================================================================
 					fprintf(stderr,"swpackage: error: type attribute not found in processExcludes().\n");
 					return -1;
 				}
-				if (first_match == 0 && *type == 'd') {
-					strob_strcat(excl_pat, "/*");
+				if (first_match == 0 && *type == 'd' && (directoryDestActive()==0 || does_have_trailing_slash)) {
+					SWEXTDEF_DEBUG2("ADDING /* to pattern list, now [%s]", strob_str(excl_pat));
+					swlib_unix_dircat(excl_pat, "/*");
 					strar_add(pat_list, strob_str(excl_pat));
 				}
 				if (first_match == 0)
@@ -711,8 +775,6 @@ public: // =====================================================================
 		}
 		strob_close(excl_pat);
 		strar_close(pat_list);
-		// swlib_toggle_trailing_slashdot("restore", source, &t_dflag);
-		// swlib_toggle_leading_dotslash("restore", source, &l_dflag);
 		return 0;
 	}
 
@@ -813,10 +875,12 @@ private: // ================================================================
 		char * pat;
 		int ret;
 
+		SWEXTDEF_DEBUG2("mdef_source is [%s]", mdef_source);
 		ret = 1;
 		i = 0;
 		pat = strar_get(pat_list, i++);
 		while(pat && ret) {
+			SWEXTDEF_DEBUG3("pat[%d] is [%s]", i-1, pat);
 			//
 			// Check for the case
 			//   where pat=/dir/A/B/C/  and mdef_source=/dir/A/B/C
@@ -833,13 +897,28 @@ private: // ================================================================
 			) {
 				// Got a match for the special case to exlude
 				// a directory but not its contents
+				SWEXTDEF_DEBUG("Got special match 0001");
 				ret = 0;
 				break;
 			}
+			if (
+				strcmp(".", pat) == 0 &&
+				(
+					strcmp(mdef_source, "." ) == 0 ||
+					strcmp(mdef_source, "/" ) == 0 ||
+					strcmp(mdef_source, "/." ) == 0
+				)
+			) {
+				SWEXTDEF_DEBUG("Got special match for [.]");
+				ret = 0;
+				break;	
+			}
+
+			SWEXTDEF_DEBUG3("---- fnmatch(%s, %s)", pat, mdef_source);
 			ret = fnmatch(pat, mdef_source, 0);
 			if (ret == 0) break;
 			pat = strar_get(pat_list, i++);
-		}	
+		}
 		return ret;
 	}
 
@@ -1140,13 +1219,6 @@ private: // ================================================================
 		return ret;
 	}
 
-	int
-	local_lstat(char * name, struct stat * st) {
-		int ret;
-		ret = lstat(name, st);
-		return ret;
-	}
-	
 	int
 	local_open(char * name) {
 		int u_fd;
@@ -1939,6 +2011,7 @@ private: // ================================================================
 						SWEXTDEF_DEBUG("");
 						p+=2;
 						value = bound_value(p, &aux_value, 0);
+						skip_amount = 0; // this to avoid a warning
 						ret = parse_name_id_string(value, aux_value,
 								is_file_permissions_context, 
 								ownerE,
