@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 
@@ -25,6 +25,17 @@
 #include "swuser_config.h"
 #include "cplob.h"
 
+static
+void
+null_list(CPLOB * lob, int start, int stop)
+{
+	int i;
+	for (i = start; i < stop; i++) {
+		*(lob->list + i) = (char*)NULL;
+	}
+}
+
+
 /*-------- CPLOB public API routines ------------------------- */
 
 CPLOB *
@@ -45,6 +56,7 @@ cplob_open(int nobj)
 	if ((lob->list = (char **) malloc(nobj * sizeof(char*))) == NULL) {
 		 return (CPLOB *) (NULL);
 	}
+	null_list(lob, 0, nobj);
 	lob->width = sizeof(char*);
 	cplob_shallow_reset(lob);
 	return lob;
@@ -101,6 +113,8 @@ cplob_reopen(int new_length, CPLOB * lob)
 		lob->list = (char **)SWBIS_REALLOC((void*)lob->list,
 			(size_t)((new_length) * lob->width), lob->nlen);
 		if (!lob->list) return (CPLOB *) (NULL);
+		/* initialize new memory to NULL pointer values */
+		null_list(lob, lob->nlen, new_length);
 		lob->nlen = new_length;
 	}
 	return lob;
@@ -172,6 +186,7 @@ cplob_val(CPLOB * lob, int index)
 	if (index < 0) {
 		return NULL;
 	}
+	if (index > lob->nused) return NULL;
 	return *(lob->list + (index));
 }
 
@@ -195,3 +210,16 @@ cplob_backfill_and_nullterminate (CPLOB * lob)
 	}
 	return 0;
 }
+
+int
+cplob_remove_index(CPLOB * lob, int i)
+{
+	if (i < 0 || i >= lob->nused) return -1;
+	if (lob->nused == 0 || lob->nlen == 0) return 0;
+	memmove (lob->list+i, lob->list+i+1,
+		lob->width * (lob->nused - i - 1));
+	lob->nused--;
+	lob->nlen--;
+	return 0;
+}
+
