@@ -27,6 +27,7 @@
 #include "swutilname.h"
 #include "swlib.h"
 #include "strob.h"
+#include "vplob.h"
 #include "swevents.h"
 
 static char g_redir1[] = "";
@@ -40,14 +41,21 @@ static struct swEvents * g_evnt = eventsArray;
 
 static int s_verboseM  = SWC_VERBOSE_4;
 
+static VPLOB * s_arrM = NULL;
+static int s_arr_indexM = 0;
 
-#define SWEVENT_LV	1
+#define SWEVENT_LV1	1
+#define SWEVENT_LV2	2
+#define SWEVENT_LV3	3
+#define SWEVENT_LV4	4
+#define SWEVENT_LV5	5
 #define SWEVENT_LV6	6
+#define SWEVENT_LV	SWEVENT_LV3
 #define SWI_EVENT	1  /* Is an event that takes place in swi.c */
 
 				/* -SW_ERROR means event is always an error */
 static struct swEvents eventsArray[] = {
-{"", 0, 0, "", 0, SW_NOTE},
+{"", 0, 0, "", 0, SW_NOTICE},
 {"SW_ILLEGAL_STATE_TRANSITION",		1, 	SWEVENT_LV, "", 1, SW_ERROR},
 {"SW_BAD_SESSION_CONTEXT",		2, 	SWEVENT_LV, "", 1, SW_ERROR},
 {"SW_ILLEGAL_OPTION",			3, 	SWEVENT_LV, "", 1, SW_ERROR},
@@ -59,25 +67,25 @@ static struct swEvents eventsArray[] = {
 {"SW_AGENT_INITIALIZATION_FAILED", 	10,	SWEVENT_LV, "", 0, -SW_ERROR},
 {"SW_SERVICE_NOT_AVALIABLE", 		11,	SWEVENT_LV, "", 0, -SW_ERROR},
 {"SW_OTHER_SESSIONS_IN_PROGRESS", 	12,	SWEVENT_LV, "", 0, SW_WARNING},
-{"SW_SESSION_BEGINS", 			28, 	SWEVENT_LV, "", 0, SW_NOTE},
+{"SW_SESSION_BEGINS", 			28, 	SWEVENT_LV, "", 0, SW_NOTICE},
 {"SW_SESSION_ENDS", 			29, 	SWEVENT_LV, "", 0, SW_ERROR},
 {"SW_CONNECTION_LIMIT_EXCEEDED", 	30,	SWEVENT_LV, "", 0, -SW_ERROR}, 
 {"SW_SOC_DOES_NOT_EXIST", 		31, 	SWEVENT_LV, "", 0, -SW_ERROR},
 {"SW_SOC_IS_CORRUPT",	 		32,	SWEVENT_LV, "", 0, SW_ERROR},
-{"SW_SOC_CREATED", 			34, 	1,          "", 0, SW_NOTE},
-{"SW_CONFLICTING_SESSION_IN_PROGRESS", 	35, 	1,          "", 0, SW_ERROR},
-{"SW_SOC_LOCK_FAILURE",			36, 	1,          "", 0, SW_ERROR},
-{"SW_SOC_IS_READ_ONLY",			37, 	1,          "", 0, SW_ERROR},
-{"SW_SOC_IS_REMOTE", 			38, 	1,          "", 0, SW_NOTE},
-{"SW_SOC_IS_SERIAL", 			40, 	1,          "", 0, SW_NOTE},
-{"SW_SOC_INCORRECT_TYPE", 		41, 	1,          "", 0, SW_ERROR},
-{"SW_CANNOT_OPEN_LOGFILE", 		42, 	1,          "", 0, SW_ERROR},
-{"SW_SOC_AMBIGUOUS_TYPE",		49, 	SWEVENT_LV, "", 0, -SW_NOTE},
-{"SW_ANALYSIS_BEGINS", 			52, 	SWEVENT_LV, "", 1, SW_NOTE},
-{"SW_ANALYSIS_ENDS", 			53, 	SWEVENT_LV, "", 1, SW_NOTE},
-{"SW_EXREQUISITE_EXCLUDE",		56,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SW_CHECK_SCRIPT_EXCLUDE",		57,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SW_CONFIGURE_EXCLUDE", 		58,	SWEVENT_LV, "", 1, SW_NOTE},
+{"SW_SOC_CREATED", 			34, 	SWEVENT_LV, "", 0, SW_NOTICE},
+{"SW_CONFLICTING_SESSION_IN_PROGRESS", 	35, 	SWEVENT_LV, "", 0, SW_ERROR},
+{"SW_SOC_LOCK_FAILURE",			36, 	SWEVENT_LV, "", 0, SW_ERROR},
+{"SW_SOC_IS_READ_ONLY",			37, 	SWEVENT_LV, "", 0, SW_ERROR},
+{"SW_SOC_IS_REMOTE", 			38, 	SWEVENT_LV, "", 0, SW_NOTICE},
+{"SW_SOC_IS_SERIAL", 			40, 	SWEVENT_LV, "", 0, SW_NOTICE},
+{"SW_SOC_INCORRECT_TYPE", 		41, 	SWEVENT_LV, "", 0, SW_ERROR},
+{"SW_CANNOT_OPEN_LOGFILE", 		42, 	SWEVENT_LV, "", 0, SW_ERROR},
+{"SW_SOC_AMBIGUOUS_TYPE",		49, 	SWEVENT_LV, "", 0, -SW_NOTICE},
+{"SW_ANALYSIS_BEGINS", 			52, 	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SW_ANALYSIS_ENDS", 			53, 	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SW_EXREQUISITE_EXCLUDE",		56,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SW_CHECK_SCRIPT_EXCLUDE",		57,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SW_CONFIGURE_EXCLUDE", 		58,	SWEVENT_LV, "", 1, SW_NOTICE},
 {"SW_SELECTION_IS_CORRUPT", 		59,	SWEVENT_LV, "", 1, SW_ERROR},
 {"SW_SOURCE_ACCESS_ERROR", 		60, 	SWEVENT_LV, ": File not found.", 0, SW_ERROR},
 {"SW_SELECTION_NOT_FOUND", 		62, 	SWEVENT_LV, "", 0, SW_ERROR},
@@ -93,13 +101,14 @@ static struct swEvents eventsArray[] = {
 {"SW_CHECK_SCRIPT_WARNING",		72,	SWEVENT_LV, "", 1, SW_WARNING},
 {"SW_CHECK_SCRIPT_ERROR",		73,	SWEVENT_LV, "", 1, -SW_ERROR},
 {"SW_DSA_OVER_LIMIT",			75,	SWEVENT_LV, "", 1, SW_ERROR},
-{"SW_DSA_FAILED_TO_RUN",		76,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SW_SKIPPED_GLOBAL_ERROR", 		80,	SWEVENT_LV, "", 1, SW_NOTE},
+{"SW_DSA_FAILED_TO_RUN",		76,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SW_ALREADY_CONFIGURED",		78,	SWEVENT_LV1, "", 1, SW_NOTICE},
+{"SW_SKIPPED_GLOBAL_ERROR", 		80,	SWEVENT_LV, "", 1, SW_NOTICE},
 {"SW_FILE_WARNING",			84,	SWEVENT_LV, "", 1, SW_WARNING},
 {"SW_FILE_ERROR",			85,	SWEVENT_LV, "", 1, -SW_ERROR},
 {"SW_NOT_LOCATABLE",			86,	SWEVENT_LV, "", 1, SW_ERROR},
-{"SW_SAME_REVISION_SKIPPED", 		87,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SW_EXECUTION_BEGINS", 		88, 	SWEVENT_LV, "", 1, SW_NOTE},
+{"SW_SAME_REVISION_SKIPPED", 		87,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SW_EXECUTION_BEGINS", 		88, 	SWEVENT_LV, "", 1, SW_NOTICE},
 {"SW_EXECUTION_ENDS", 			89, 	SWEVENT_LV, "", 1, SW_ERROR},
 {"SW_PRE_SCRIPT_WARNING",		95,	SWEVENT_LV, "", 1, SW_WARNING},
 {"SW_PRE_SCRIPT_ERROR",			96,	SWEVENT_LV, "", 1, SW_ERROR},
@@ -110,51 +119,70 @@ static struct swEvents eventsArray[] = {
 {"SW_CONFIGURE_WARNING",		103,	SWEVENT_LV, "", 1, SW_WARNING},
 {"SW_CONFIGURE_ERROR",			104,	SWEVENT_LV, "", 1, SW_ERROR},
 {"SW_DATABASE_UPDATE_ERROR",		105,	SWEVENT_LV, "", 1, -SW_ERROR},
-{"SW_FILESET_BEGINS",			117,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SW_CONTROL_SCRIPT_BEGINS",		118,	SWEVENT_LV, "", 1, SW_NOTE},
+{"SW_FILESET_BEGINS",			117,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SW_CONTROL_SCRIPT_BEGINS",		118,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SW_CONTROL_SCRIPT_NOT_FOUND",		119,	SWEVENT_LV, "", 1, SW_NOTICE},
 
 	/*	
-	 * Events 0 - 255 are Posix or are reserved.
+	 * Events 0 - 255 are POSIX or are reserved.
 	 * Events >255 are specific to the swbis implementation.
 	 */ 
 
-{"SW_SOURCE_ACCESS_BEGINS", 		260, 	SWEVENT_LV, "", 0, SW_NOTE},
+{"SW_SOURCE_ACCESS_BEGINS", 		260, 	SWEVENT_LV, "", 0, SW_NOTICE},
 {"SW_SOURCE_ACCESS_ENDS", 		261, 	SWEVENT_LV, "", 0, SW_ERROR},
-{"SW_CONTROL_SCRIPT_ENDS",		262,	SWEVENT_LV, "", 1, SW_NOTE},
+{"SW_CONTROL_SCRIPT_ENDS",		262,	SWEVENT_LV2, "", 1, SW_NOTICE},
 {"SW_SOC_LOCK_CREATED",			263,	SWEVENT_LV, "", 1, SW_ERROR},
 {"SW_SOC_LOCK_REMOVED",			264,	SWEVENT_LV, "", 1, SW_ERROR},
-{"SW_SELECTION_EXECUTION_BEGINS",	265,	SWEVENT_LV, "", 0, SW_NOTE},
-{"SW_SELECTION_EXECUTION_ENDS",		266,	SWEVENT_LV, "", 0, SW_ERROR},
+{"SW_SELECTION_EXECUTION_BEGINS",	265,	SWEVENT_LV1, "", 0, SW_NOTICE},
+{"SW_SELECTION_EXECUTION_ENDS",		266,	SWEVENT_LV1, "", 0, SW_ERROR},
 {"SW_ISC_INTEGRITY_CONFIRMED",		267,	SWEVENT_LV, "", 0, SW_ERROR},
 {"SW_ISC_INTEGRITY_NOT_CONFIRMED",	268,	SWEVENT_LV, "", 0, SW_ERROR},
 {"SW_SPECIAL_MODE_BEGINS",		269,	SWEVENT_LV, "", 0, SW_ERROR},
 {"SW_SOC_INTEGRITY_CONFIRMED",		270,	SWEVENT_LV, "", 0, SW_ERROR},
 {"SW_SOC_INTEGRITY_NOT_CONFIRMED",	271,	SWEVENT_LV, "", 0, SW_ERROR},
+{"SW_ABORT_SIGNAL_RECEIVED",		272,	SWEVENT_LV1, "", 0, SW_ERROR},
 
-{"SWI_PRODUCT_SCRIPT_ENDS", 		280, 	SWEVENT_LV, "", 1, SW_NOTE},
-{"SWI_FILESET_SCRIPT_ENDS", 		281, 	SWEVENT_LV, "", 1, SW_NOTE},
+{"SWI_PRODUCT_SCRIPT_ENDS", 		280, 	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SWI_FILESET_SCRIPT_ENDS", 		281, 	SWEVENT_LV, "", 1, SW_NOTICE},
 
-{"SWI_CATALOG_UNPACK_BEGINS", 		302, 	SWEVENT_LV, "", 1, SW_NOTE},
+{"SWI_CATALOG_UNPACK_BEGINS", 		302, 	SWEVENT_LV, "", 1, SW_NOTICE},
 {"SWI_CATALOG_UNPACK_ENDS", 		303, 	SWEVENT_LV, "", 1, SW_ERROR},
-{"SWI_TASK_BEGINS", 			304, 	SWEVENT_LV6, "", 1, SW_NOTE},
+{"SWI_TASK_BEGINS", 			304, 	SWEVENT_LV6, "", 1, SW_NOTICE},
 {"SWI_TASK_ENDS", 			305, 	SWEVENT_LV6, "", 1, SW_ERROR},
 {"SWI_SWICOL_ERROR", 			306, 	SWEVENT_LV, "", 1, SW_ERROR},
-{"SWI_CATALOG_ANALYSIS_BEGINS",		307,	SWEVENT_LV, "", 1, SW_NOTE},
+{"SWI_CATALOG_ANALYSIS_BEGINS",		307,	SWEVENT_LV, "", 1, SW_NOTICE},
 {"SWI_CATALOG_ANALYSIS_ENDS",		308,	SWEVENT_LV, "", 1, SW_ERROR},
-{"SWBIS_TARGET_BEGINS",			309,	SWEVENT_LV, "", 1, SW_NOTE},
+{"SWBIS_TARGET_BEGINS",			309,	SWEVENT_LV, "", 1, SW_NOTICE},
 {"SWBIS_TARGET_ENDS",			310,	SWEVENT_LV, "", 1, SW_ERROR},
 {"SWI_NORMAL_EXIT",			311,	SWEVENT_LV, "", 0, SW_ERROR},
-{"SWI_SELECTION_BEGINS",		312,	SWEVENT_LV, "", 0, SW_NOTE},
+{"SWI_SELECTION_BEGINS",		312,	SWEVENT_LV, "", 0, SW_NOTICE},
 {"SWI_SELECTION_ENDS",			313,	SWEVENT_LV, "", 0, SW_ERROR},
-{"SWI_MSG",				314,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SWI_ATTRIBUTE",			315,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SWI_GROUP_BEGINS",			316,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SWI_GROUP_ENDS",			317,	SWEVENT_LV, "", 1, SW_NOTE},
-{"SWI_TASK_CTS",			318,	SWEVENT_LV6, "", 1, SW_NOTE},
-{"SWI_MAIN_SCRIPT_ENDS",		319,	SWEVENT_LV6, "", 1, SW_NOTE},
-{"<_nil_>", -1, 0, "", 0, SW_NOTE}
+{"SWI_MSG",				314,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SWI_ATTRIBUTE",			315,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SWI_GROUP_BEGINS",			316,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SWI_GROUP_ENDS",			317,	SWEVENT_LV, "", 1, SW_NOTICE},
+{"SWI_TASK_CTS",			318,	SWEVENT_LV6, "", 1, SW_NOTICE},
+{"SWI_MAIN_SCRIPT_ENDS",		319,	SWEVENT_LV6, "", 1, SW_NOTICE},
+{"<_nil_>", -1, 0, "", 0, SW_NOTICE}
 };
 
+static
+void *
+arr_get_p(void)
+{
+	void * s;
+	if (s_arrM == NULL) s_arrM = vplob_open();
+	s = vplob_val(s_arrM, s_arr_indexM);
+	if (s == NULL) {
+		s = (void*)strob_open(80);
+		vplob_add(s_arrM, s);	
+	} else {
+		strob_strcpy((STROB*)s, "");
+	}
+	s_arr_indexM++;
+	return s;
+}
+
 static
 struct swEvents *
 get_struct_by_message(char * line, struct swEvents * evnt)
@@ -231,7 +259,7 @@ get_event_status(char * line, int * statusp, char ** message)
 		strstr(line, ": SWI_") ||
 		strstr(line, ": SWBIS_")
 	) {
-		if (strstr(line, "status=")) {
+		if (strstr(line, SWEVENT_STATUS_PFX)) {
 			/*
 			* The event has a status
 			*/
@@ -311,21 +339,26 @@ swevent_shell_echo(int to_fd, int verbose_level,
 			char * loc, int value, char * host, char * msg)
 {
 	char * ret;
-	STROB * buf = strob_open(10);
-	STROB * sbuf = strob_open(10);
+	STROB * buf;
+	STROB * sbuf;
 	struct swEvents * eop;
 	char * redir = g_redirnull;
 	struct swEvents * evnt;
 
+	sbuf = strob_open(10);
+	buf = (STROB*)arr_get_p();
+
 	evnt = swevent_get_events_array();
 	eop = get_struct_by_value(value, evnt);
 	if (!eop) {
 		return strdup("echo \"\" 1>/dev/null");
 	}
 
-	if (eop->is_swi_eventM || /* need to always echo this internal event */
+	if (	1 || /* actually always do this, this test probably should be removed */
+		eop->is_swi_eventM || /* need to always echo this internal event */
 		verbose_level >= eop->verbose_threshholdM || 
-					verbose_level < 0) {
+		verbose_level < 0
+	) {
 		if (to_fd == 2 || eop->is_swi_eventM) {
 			/*
 			 * Internal swi protocol events are always on
@@ -345,12 +378,7 @@ swevent_shell_echo(int to_fd, int verbose_level,
 						msg, verbose_level));
 	strob_sprintf(buf, 1, "\" %s", redir);
 	strob_close(sbuf);
-	/*
-	 * Don't FIXME ??
-	 * New memory is allocated on each call, thus forming a memory leak.
-	 */
-	ret = strdup(strob_str(buf));
-	strob_close(buf);
+	ret = strob_str(buf);
 	return ret;
 }
 
@@ -376,8 +404,8 @@ swevent_is_error(char * line, int * statusp)
 	struct swEvents * evnts = eventsArray;
 	struct swEvents * ev;
 	char * s = (char*)NULL;
-	if ((s=strstr(line, "status="))) {
-		s += strlen("status=");
+	if ((s=strstr(line, SWEVENT_STATUS_PFX))) {
+		s += strlen(SWEVENT_STATUS_PFX);
 		n = /* swlib_status_ */ atoi(s);
 		if (n == INT_MAX || n == INT_MIN) {
 			/*
@@ -450,9 +478,6 @@ swevent_write_rpsh_event(int fd, char * event_line, int len)
 
 	get_event_status(event_line, &status, &msg);
 	strob_strcpy(tmp, "");
-	/*
-	fprintf(stderr, "JL msg=[%s]\n", msg); 
-	*/
 
 	/*
 	 * Form a simplified message in the form of
@@ -460,7 +485,7 @@ swevent_write_rpsh_event(int fd, char * event_line, int len)
 	 * where number is the event number
 	 */
 
-	if (msg && strstr(msg, "status=") != msg) {
+	if (msg && strstr(msg, SWEVENT_STATUS_PFX) != msg) {
 		/*
 		 * The message is not "status=X"
 		 */
@@ -476,9 +501,6 @@ swevent_write_rpsh_event(int fd, char * event_line, int len)
 		 */
 		strob_sprintf(tmp, 0, "%d:%d\n", event_value, status);
 	}
-	/*
-	fprintf(stderr, "JL writing [%s]\n", strob_str(tmp)); 
-	*/
 	ret = uxfio_unix_safe_write(fd, strob_str(tmp), strob_strlen(tmp));
 	if (ret < 0 || ret != (int)strob_strlen(tmp)) {
 		fprintf(stderr, "swevent_write_rpsh_event: error: write: ret=%d fd=%d %s\n",
@@ -528,4 +550,28 @@ swevent_get_events_array(void)
 	return eventsArray;
 }
 
+struct swEvents *
+swevents_get_struct_by_message(char * line, struct swEvents * evnt)
+{
+	return get_struct_by_message(line, evnt);
+}
 
+void
+swevent_s_arr_reset(void)
+{
+	/* This will force reuse of exiting string objects,
+	   however the caller needs to be sure that the reused
+	   objects have finished being used.  Aggressive use of
+	   this routine can unexpectedly cause a malfunction */
+	s_arr_indexM = 0;
+}
+
+void
+swevent_s_arr_delete(void)
+{
+	void * s;
+	int i;
+	i = 0;
+	while((s=vplob_val(s_arrM, i++)) != NULL) strob_close((STROB*)s);
+	vplob_close(s_arrM);
+}
