@@ -1,6 +1,12 @@
 #ifndef swevents_h_200403
 #define swevents_h_200403
 
+#define SWEVENT_VALUE_PREVIEW		"preview"
+#define SWEVENT_KW_STATUS		"status"
+#define SWEVENT_ATT_DELIM		"="
+#define SWEVENT_STATUS_PFX		SWEVENT_KW_STATUS SWEVENT_ATT_DELIM
+#define SWEVENT_ATT_STATUS_PREVIEW	SWEVENT_STATUS_PFX SWEVENT_VALUE_PREVIEW
+
 #define SW_ILLEGAL_STATE_TRANSITION 	1
 #define SW_BAD_SESSION_CONTEXT 		2
 #define SW_ILLEGAL_OPTION 		3
@@ -68,6 +74,7 @@
 #define SW_DATABASE_UPDATE_ERROR	105
 #define SW_FILESET_BEGINS		117
 #define SW_CONTROL_SCRIPT_BEGINS	118
+#define SW_CONTROL_SCRIPT_NOT_FOUND	119
 
 #define SW_SOURCE_ACCESS_BEGINS 	260 /* Not Posix */
 #define SW_SOURCE_ACCESS_ENDS 		261 /* Not Posix */
@@ -81,6 +88,7 @@
 #define SW_SPECIAL_MODE_BEGINS		269 /* Not Posix */
 #define SW_SOC_INTEGRITY_CONFIRMED 	270 /* Not Posix */
 #define SW_SOC_INTEGRITY_NOT_CONFIRMED	271 /* Not Posix */
+#define SW_ABORT_SIGNAL_RECEIVED	272 /* Not Posix */
 
 #define SWI_PRODUCT_SCRIPT_ENDS		280 /* Not Posix */
 #define SWI_FILESET_SCRIPT_ENDS		281 /* Not Posix */
@@ -104,15 +112,9 @@
 #define SWI_TASK_CTS			318 /* Not Posix  ClearToSend */
 #define SWI_MAIN_SCRIPT_ENDS		319 /* Not Posix */
 
-#define SW_SUCCESS	0
-#define SW_NOTE 	0
-#define SW_ERROR 	1
-#define SW_WARNING 	2
-#define SW_DESELECT	3
-
 struct swEvents {
-		char * codeM;	     /* Posix event code	*/
-		int valueM;	     /* Posix event value	*/
+		char * codeM;	     /* POSIX event code	*/
+		int valueM;	     /* POSIX event value	*/
 		int verbose_threshholdM;
 		char * msgM;	     /* Message text	   	*/
 		int is_swi_eventM;   /* May occur in a swicol_<*> task*/
@@ -138,4 +140,7 @@ int swevent_parse_message(char * line, char ** attribute, char ** value);
 struct swEvents * swevent_get_events_array(void);
 char * swevent_code(int value);
 void swevent_set_verbose(int v);
+struct swEvents * swevents_get_struct_by_message(char * line, struct swEvents * evnt);
+void swevent_s_arr_reset(void);
+void swevent_s_arr_delete(void);
 #endif
