@@ -66,6 +66,7 @@ i_is_value_false(char * s)
 	if (
 		s == NULL ||
 		(
+			strcasecmp(s, "f") == 0 ||
 			strcasecmp(s, "false") == 0 ||
 			strcasecmp(s, "no") == 0 ||
 			strcasecmp(s, "n") == 0 ||
@@ -84,6 +85,7 @@ i_is_value_true(char * s)
 	if (
 		s != NULL &&
 		(
+			strcasecmp(s, "t") == 0 ||
 			strcasecmp(s, "true") == 0 ||
 			strcasecmp(s, "yes") == 0 ||
 			strcasecmp(s, "y") == 0 ||
@@ -346,7 +348,7 @@ parseDefaultsFile(char * utility_name, char * defaults_filename,
 	int is_global_option;
 	int is_set;
 
-	uxfio_fd = uxfio_open("/dev/null", O_RDONLY, 0);
+	uxfio_fd = uxfio_open("", O_RDONLY, 0);
 	if (uxfio_fd < 0) {
 		return 1;
 	}
@@ -600,6 +602,8 @@ set_opta_boolean(struct extendedOptions * opta, enum eOpts nopt, char * value)
 	if (opta[nopt].is_boolM) {
 		if (
 			value != NULL &&
+			strcasecmp(value, "t") &&
+			strcasecmp(value, "f") &&
 			strcasecmp(value, SW_E_TRUE) &&
 			strcasecmp(value, SW_E_FALSE) &&
 			strcasecmp(value, "no") &&
@@ -646,13 +650,102 @@ swbisoption_get_opta(struct extendedOptions * opta, enum eOpts nopt)
 	return value;
 }
 
+char *
+get_opta_isc(struct extendedOptions * opta, enum eOpts nopt)
+{
+	char * value;
+	char * s;
+	char * host;
+	char * file;
+	E_DEBUG("");
+	if (nopt != SW_E_installed_software_catalog) {
+		return get_opta(opta, nopt); 
+	}
+	value = opta[nopt].valueM;
+	if (value == NULL) {
+		fprintf(stderr, "%s: warning: sw option %d [%s] is null, using default [%s]\n",
+			swlib_utilname_get(), (int)nopt, opta[nopt].optionNameM, opta[nopt].defaultValueM);
+	}
+	if (!value) value = opta[nopt].defaultValueM;
+
+	/* now put here support for an absolute path specified by URL syntax
+	    such as file://localhost/path  or
+                    file:///path
+
+	   if that value is not in URL file syntax even if it is an absolute path
+	   the leading slash will be stripped.  */
+
+	s = strstr(value, FILE_URL);
+	if (s == NULL) {
+		/* Here is the normal path when not using a file URL.
+		   In this case, all paths whether absolute or not are treated
+		   as a relative path [which becomes relative to the target
+		   path] */
+		E_DEBUG("processing ordinary path");
+		swlib_squash_all_leading_slash(value);
+		swlib_squash_all_dot_slash(value);
+		swlib_squash_leading_dot_slash(value);
+		return value;
+	} else {
+		/* Here is support for an absolute path specification
+		   for the installed software catalog location using the
+		   file URL syntax:
+			file://host/path
+			file:///path  */
+
+		E_DEBUG("processing file URL");
+		if (s != value) {
+			/* maybe a locahost pathname that has "file://" in it.
+			   issue a warning and return it */
+			fprintf(stderr, "%s: warning malformed file URL: %s\n",
+				swlib_utilname_get(), value);
+			return value;
+		}
+
+		/* If we are at here then the value must be one of two things
+			file:///path
+				or
+			file://host/path */
+	
+		s += strlen(FILE_URL);
+
+		if (*s == '/') {
+			/* Is the form file:///path */
+			host = NULL;
+			file = s;
+		} else {
+			/* Is the form file://host/path */
+			host = s;
+			file = strchr(s, '/');
+			if (file == NULL) {
+				/* malformed URL,
+					file://name
+				i.e. where name is either a host or single name in the '/' root directory */
+				fprintf(stderr, "%s: malformed file URL, assuming host part is file part\n",
+					swlib_utilname_get());
+				file = host;
+			} else {
+				if (strstr(host, "localhost/") != host) {
+					/* error, remote catalogs not supported at this time */
+					fprintf(stderr, "%s: error: remote catalog URL not supported: %s\n", swlib_utilname_get(), value);
+					fprintf(stderr, "%s: error: Possibly you intend URL: file:///absolute/path\n", swlib_utilname_get());
+					fprintf(stderr, "%s: error: exiting with error\n", swlib_utilname_get());
+					exit(1);
+				}
+			}
+		}
+	}
+	return file;
+}
+
 char *
 get_opta(struct extendedOptions * opta, enum eOpts nopt)
 {
 	char * value;
 	value = opta[nopt].valueM;
 	if (value == NULL) {
-		fprintf(stderr, "internal warning: sw option %d [%s] is null\n", (int)nopt, opta[nopt].optionNameM);
+		fprintf(stderr, "%s: warning: sw option %d [%s] is null, using default [%s]\n",
+			swlib_utilname_get(), (int)nopt, opta[nopt].optionNameM, opta[nopt].defaultValueM);
 	}
 	if (!value) value = opta[nopt].defaultValueM;
 	return value;
