@@ -65,6 +65,9 @@ struct extendedOptions {
 #define SWC_U_V  (1 << 8) 	/* applicability_flags   swverify  */
 #define SWC_U_X  (1 << 9) 	/* applicability_flags   not applicable  */
 
+#define FILE_URL	"file://"
+
+
 enum eOpts {	/* NOTE: the order must match the optionsArray 
 		  found in swsupplib/misc/swcommon_options.h */
 	SW_E_allow_downdate,
@@ -148,6 +151,7 @@ enum eOpts {	/* NOTE: the order must match the optionsArray
 	SW_E_swbis_remove_volatile,
 	SW_E_swbis_check_mtime,
 	SW_E_swbis_check_duplicates,
+	SW_E_swbis_check_owners,
 	SW_E_swbis_last_option
 	};  /* NOTE: the order must match the optionsArray
 		 found in swsupplib/misc/swcommon_options.h*/
@@ -170,6 +174,7 @@ void set_opta(struct extendedOptions * opta, enum eOpts nopt, char * value);
 void set_opta_initial(struct extendedOptions * opta, enum eOpts nopt, char * value);
 char * swbisoption_get_opta(struct extendedOptions * opta, enum eOpts nopt);
 char * get_opta(struct extendedOptions * opta, enum eOpts nopt);
+char * get_opta_isc(struct extendedOptions * opta, enum eOpts nopt);
 int parse_options_file(struct extendedOptions * opta, char * filename, char * util_name);
 char * initialize_options_files_list(char * usethis);
 int parse_options_files(struct extendedOptions * opta, char * option_files, char * util_name, int reqd, int show_only);
