@@ -53,8 +53,8 @@
 #include "swgpg.h"
 #include "fmgetpass.h"
 	
-static char * g_swgpg_dirname;
-static char * g_swgpg_fifopath;
+static char * g_swgpg_dirname = NULL;
+static char * g_swgpg_fifopath = NULL;
 static int swgpg_passfd;
 
 static
@@ -77,18 +77,38 @@ close_passfd(void)
 
 static
 char *
-make_dir(void)
+make_dir(char ** ma)
 {
 	int ret;
-	char *filename;
+	char * template;
+	char * filename;
+	char * sret;
 	mode_t um;
 	int try;
 	const int max_tries = 10;
-
-	E_DEBUG("");
-	try = 0;
+#undef  have__mkdtemp 
+#define have__mkdtemp
+#ifdef have__mkdtemp
+	E_DEBUG("begin using mkdtemp");
+	template = (char*)malloc(
+			strlen(SWGPG_FIFO_DIR) + 1 +
+			strlen(SWGPG_FIFO_PFX) + 8);
+	if (!template) return NULL;
+	*ma = template;
+	strcpy(template, SWGPG_FIFO_DIR);
+	strcat(template, "/");
+	strcat(template, SWGPG_FIFO_PFX);	
+	strcat(template, "XXXXXX");	
+	filename = mkdtemp(template); 	
+	if (filename == NULL) {
+		fprintf(stderr, "%s: mkdtemp error for template [%s]: %s\n", swlib_utilname_get(), template, strerror(errno));
+	}
+	E_DEBUG2("end using mkdtemp: filename=[%s]", filename);
+#else
+	E_DEBUG("begin using tempnam");
 	um = swlib_get_umask();
 	umask(0002);
+	try = 0;
 	E_DEBUG("");
 	do {
 		E_DEBUG("");
@@ -97,17 +117,19 @@ make_dir(void)
 		umask(um);
 		ret = mkdir(filename, (mode_t)(0700));
 		if (ret < 0) {
-			fprintf(stderr, "%s: %s\n", swlib_utilname_get(), strerror(errno));
+			fprintf(stderr, "%s: mkdir failed with error: %s\n", swlib_utilname_get(), strerror(errno));
 			free (filename);
 			filename = NULL;
 		}
 	} while (ret < 0 && try < max_tries);
 	E_DEBUG("");
-	umask(um);
-	if (try >= max_tries && ret < 0)
+	if (try >= max_tries && ret < 0) {
+		fprintf(stderr, "%s: failed to make temp dir in %d tries\n", swlib_utilname_get(), max_tries);
 		return NULL;
-
-	E_DEBUG("");
+	}
+	umask(um);
+	E_DEBUG2("end using tempnam: filename=[%s]", filename);
+#endif
 	return filename;
 }
 
@@ -396,16 +418,17 @@ swgpg_create_fifo(STROB * buf)
 	int ret;
 	char * filename;
 	char * dirname;
+	char * freeit = NULL;
 
 	E_DEBUG("");
 	if (!buf) return NULL;
 	if (g_swgpg_dirname != NULL) return NULL;
 	if (g_swgpg_fifopath != NULL) return NULL;
 	strob_strcpy(buf, "");
-	dirname = make_dir();
+	dirname = make_dir(&freeit);
 	if (!dirname) return NULL;
 	strob_strcpy(buf, dirname);
-	g_swgpg_dirname = dirname;
+	g_swgpg_dirname = strdup(dirname);
 	swlib_unix_dircat(buf, SWGPG_FIFO_NAME);
 	g_swgpg_fifopath = strdup(strob_str(buf));
 
@@ -418,6 +441,7 @@ swgpg_create_fifo(STROB * buf)
 	}
 
 	E_DEBUG("");
+	if (freeit) free(freeit);
 	return strob_str(buf);
 }
 
@@ -430,9 +454,11 @@ swgpg_remove_fifo(void)
 	if (g_swgpg_fifopath == NULL) return -1;
 	ret = unlink(g_swgpg_fifopath);
 	if (ret == 0) {
+		free(g_swgpg_fifopath);
 		g_swgpg_fifopath = NULL;
 		ret = rmdir(g_swgpg_dirname);
 		if (ret == 0) {
+			free(g_swgpg_dirname);
 			g_swgpg_dirname = NULL;
 		}
 	}
