@@ -228,6 +228,31 @@ restore_state(SWHEADER * header, SWHEADER_STATE * state)
 /* -------------------------------------------------------------------*/
 /* ------------------ Public Functions ------------------------------*/
 
+SWHEADER_STATE * 
+swheader_state_create(void)
+{
+	SWHEADER_STATE * x;
+	x = malloc (sizeof(SWHEADER_STATE));
+  	x->save_current_offset_pM = NULL;
+	x->save_current_offsetM = -1; 
+	x->save_current_offset_vM = -1; 
+	return x;
+}
+
+void
+swheader_state_delete(SWHEADER_STATE * x)
+{
+	free(x);
+}
+
+void
+swheader_state_copy(SWHEADER_STATE * to, SWHEADER_STATE * from)
+{
+  	to->save_current_offset_pM = from->save_current_offset_pM;
+	to->save_current_offsetM = from->save_current_offsetM;
+	to->save_current_offset_vM = from->save_current_offset_vM; 
+}
+
 void
 swheader_store_state(SWHEADER * header, SWHEADER_STATE * fp_state)
 {
@@ -289,6 +314,7 @@ swheader_open(char* (*f_goto_next)(void * , int* , int), void * image_object)
 void
 swheader_close(SWHEADER * swheader)
 {
+	if (swheader->image_head_) free(swheader->image_head_);
 	if (swheader != NULL) swbis_free(swheader);
 }
 
@@ -411,6 +437,7 @@ void
 swheader_set_image_head(SWHEADER * swheader, void * image)
 {
 	SWHEADER_E_DEBUG("");
+	if (swheader->image_head_) free(swheader->image_head_);
 	swheader->image_head_=image;
 }
 
@@ -866,7 +893,6 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 
 	taru_init_header(file_hdr);
 	taru_init_header_digs(file_hdr);
-
 	
 	swheader_store_state(fileheader, &state);
 
@@ -874,7 +900,6 @@ swheader_fileobject2filehdr(SWHEADER * fileheader, struct new_cpio_header * file
 
 		keyword = swheaderline_get_keyword(attr);
 		value = swheaderline_get_value(attr, &len);
-		/*JL fprintf(stderr, "%s = [%s]\n", keyword, value); */
 
 		if (strcmp(keyword, SW_A_mode) == 0) {
 			file_hdr->c_mode = 0;
@@ -1048,24 +1073,26 @@ swheader_getTarTypeFromTypeAttribute(char ch) {
 void
 swheader_print_header(SWHEADER * swheader)
 {
-            char * next_attr;
-            char * next_line;
-            swheader_reset(swheader);
-            swheader_set_current_offset_p_value(swheader, 0);
+	char * next_attr;
+	char * next_line;
+	SWHEADER_STATE state;
 
-            next_line = swheader_get_next_object(swheader, 
+	swheader_store_state(swheader, &state);
+        swheader_reset(swheader);
+        swheader_set_current_offset_p_value(swheader, 0);
+
+	next_line = swheader_get_next_object(swheader, 
 				(int)UCHAR_MAX, (int)UCHAR_MAX);
-            while (next_line){
-                    swheaderline_write_debug(next_line, STDERR_FILENO);
+	while (next_line) {
+		swheaderline_write_debug(next_line, STDERR_FILENO);
 
-                    swheader_goto_next_line((void *)swheader, 
-				swheader_get_current_offset_p(swheader), 
-				SWHEADER_PEEK_NEXT);
+		swheader_goto_next_line((void *)swheader,
+			swheader_get_current_offset_p(swheader), SWHEADER_PEEK_NEXT);
 
-                    while((next_attr=swheader_get_next_attribute(swheader)))
+		while((next_attr=swheader_get_next_attribute(swheader)))
                             swheaderline_write_debug(next_attr, STDERR_FILENO);
-                    next_line = swheader_get_next_object(swheader, 
-					(int)UCHAR_MAX, (int)UCHAR_MAX);
-            }
-	
+			next_line = swheader_get_next_object(swheader, 
+				(int)UCHAR_MAX, (int)UCHAR_MAX);
+	}
+	swheader_restore_state(swheader, &state);
 }
