@@ -96,6 +96,9 @@ void swheader_print_header(SWHEADER * swheader);
 */
 int swheader_get_object_offset_by_control_directory(SWHEADER * swheader,  STRAR * pairs);
 char * swheader_get_single_attribute_value(SWHEADER * swheader, char *keyword);
+SWHEADER_STATE * swheader_state_create(void);
+void swheader_state_delete(SWHEADER_STATE *);
+void swheader_state_copy(SWHEADER_STATE * to, SWHEADER_STATE * from);
 
 /* ---------------- internal debugging function ----------------------*/
 
