@@ -261,9 +261,9 @@ swheaderline_strdup(char * outputline) {
 	int value_length;
 	char * value = swheaderline_get_value(outputline, &value_length);
 	int len = (int)(value - outputline) + value_length + 1;
-	char * dup = (char *)malloc(len);
+	char * dup = (char *)malloc(len+1);
 	*(dup+len) = '\0';
-	memcpy(dup, outputline, len);
+	memcpy(dup, outputline, len+1);
 	return dup;
 }
 
