@@ -1,6 +1,6 @@
 /* swicat.c -- installed software catalog routines
    
-   Copyright (C) 2004,2006,2007 Jim Lowe
+   Copyright (C) 2004,2006,2007,2010 Jim Lowe
    All Rights Reserved.
   
    COPYING TERMS AND CONDITIONS:
@@ -201,6 +201,7 @@ evaluate_query(GB * G, SWICAT_REQ * req, char * spec, STROB * text, char * first
 
 	line = strob_strtok(text, NULL, "\r\n");
 	while(line) {
+		E_DEBUG2("line=[%s]", line);
 		if (swicat_sl) {
 			SWICAT_SR * sr = swicat_sr_create();
 			swicat_sr_parse(sr, line);	
@@ -209,6 +210,7 @@ evaluate_query(GB * G, SWICAT_REQ * req, char * spec, STROB * text, char * first
 		resp = check_requisite_response(line);
 		/* for exrequistes NULL is good,
 		   for prerequistes NULL is bad */
+		E_DEBUG2("resp=[%s]", resp);
 		switch (*type) {
 			case 'P':  /* prerequisites */
 				if (resp == NULL) {
@@ -632,38 +634,21 @@ swicat_isf_control_script(STROB * buf, SWI_CONTROL_SCRIPT * xx, int do_if_active
 {
 	/* if (do_if_active && !(xx->is_activeM)) return 0; */
 	strob_sprintf(buf, STROB_DO_APPEND, SW_A_control_file "\n");
-	strob_sprintf(buf, STROB_DO_APPEND, "\t" SW_A_tag " %s\n", xx->baseM.tagM);
+	strob_sprintf(buf, STROB_DO_APPEND, "\t" SW_A_tag " %s\n", xx->baseM.b_tagM);
 	strob_sprintf(buf, STROB_DO_APPEND, "\t" SW_A_result " %s\n", swi_control_script_posix_result(xx));
 	if (xx->resultM >= 0)
-		strob_sprintf(buf, STROB_DO_APPEND, "\t" "return_code" " %d\n", xx->resultM);
+		strob_sprintf(buf, STROB_DO_APPEND, "\t" SW_A_return_code " %d\n", xx->resultM);
 	return 0;
 }
 
 int
 swicat_isf_fileset(SWI * swi, STROB * buf, SWI_XFILE * xfile, int do_if_active)
 {
-	/* if (do_if_active && !(xfile->is_activeM)) return 0; */
-
 	strob_sprintf(buf, STROB_DO_APPEND, SW_A_fileset "\n");
-
+	strob_sprintf(buf, STROB_DO_APPEND, "    " SW_A_location " %s\n", swi->swi_pkgM->locationM);
 	write_attributes(buf, SWI_INDEX_HEADER(swi), xfile->baseM.header_indexM);
 	write_state(buf, xfile, do_if_active);
 	write_times(buf, (SWI_BASE *)&(xfile->baseM), 1 /* do_if_active */);
-
-	/*
-	{
-		char * line;
-		SWHEADER * swheader;
-		xfile->INDEX_header_indexM;
-		swheader = SWI_INDEX_HEADER(swi);
-		swheader_store_state(swheader, NULL);
-		swheader_set_current_offset(swheader, xfile->INDEX_header_indexM);
-		while((line=swheader_get_next_attribute(swheader)))
-			swheaderline_write_debug(line, STDERR_FILENO);
-		swheader_restore_state(swheader, NULL);
-	}
-	*/
-
 	swicat_isf_all_scripts(buf, xfile->swi_scM, do_if_active);
 	strob_sprintf(buf, STROB_DO_APPEND, "\n");
 	return 0;
@@ -680,9 +665,9 @@ swicat_isf_product(SWI * swi, STROB * buf, SWI_PRODUCT * prod, int do_if_active)
 	strob_sprintf(buf, STROB_DO_APPEND, SW_A_product "\n");
 	strob_sprintf(buf, STROB_DO_APPEND, SW_A_location " %s\n", swi->swi_pkgM->locationM);
 		
-	write_attributes(buf, SWI_INDEX_HEADER(swi), prod->baseM.header_indexM);
+	write_attributes(buf, SWI_INDEX_HEADER(swi), prod->p_baseM.header_indexM);
 
-	write_times(buf, (SWI_BASE *)&(prod->baseM), 1 /* do_if_active */);
+	write_times(buf, (SWI_BASE *)&(prod->p_baseM), 1 /* do_if_active */);
 	/* strob_sprintf(buf, STROB_DO_APPEND, SW_A_all_filesets "%s\n", "" ); */
 
 	/* ------------------------------------------------------ */
@@ -691,7 +676,7 @@ swicat_isf_product(SWI * swi, STROB * buf, SWI_PRODUCT * prod, int do_if_active)
 	/*
 	{
 		swheader_store_state(swheader, NULL);
-		swheader_set_current_offset(swheader, prod->baseM.header_indexM);
+		swheader_set_current_offset(swheader, prod->p_baseM.header_indexM);
 		while((line=swheader_get_next_attribute(swheader)))
 			swheaderline_write_debug(line, STDERR_FILENO);
 		swheader_restore_state(swheader, NULL);
@@ -701,7 +686,7 @@ swicat_isf_product(SWI * swi, STROB * buf, SWI_PRODUCT * prod, int do_if_active)
 	/* ------------------------------------------------------ */
 	/* ------------------------------------------------------ */
 
-	swicat_isf_all_scripts(buf, prod->pfilesM->swi_scM, do_if_active);
+	swicat_isf_all_scripts(buf, prod->xfileM->swi_scM, do_if_active);
 	while(i < SWI_MAX_OBJ && prod->swi_coM[i]) {
 		swicat_isf_fileset(swi, buf, prod->swi_coM[i], do_if_active);
 		i++;
@@ -712,34 +697,10 @@ swicat_isf_product(SWI * swi, STROB * buf, SWI_PRODUCT * prod, int do_if_active)
 int
 swicat_isf_installed_software(STROB * buf, SWI * swi)
 {
-	STROB * tmp;
-	char * s;
 	SWI_PRODUCT * prod;
 
-	tmp = strob_open(10);
-
 	prod = swi->swi_pkgM->swi_coM[0];
 	strob_sprintf(buf, STROB_DO_APPEND, SW_A_installed_software "\n");
-
-	s = strstr(swi->swi_pkgM->catalog_entryM, swi->swi_pkgM->installed_software_catalogM);
-	if (0) {
-		/* don't add the path and catalog attributes to the INSTALLED file */
-		/* eventually delete this code */
-	if (s) {
-		/* adjust the installed_software.path and .catalog attributes */
-		s = s + strlen(swi->swi_pkgM->installed_software_catalogM);
-		s = swlib_return_relative_path(s);
-		strob_strcpy(tmp, swi->swi_pkgM->target_pathM);
-		swlib_unix_dircat(tmp, swi->swi_pkgM->installed_software_catalogM);
-		strob_sprintf(buf, STROB_DO_APPEND, "\t" SW_A_path " %s\n", strob_str(tmp));
-		strob_sprintf(buf, STROB_DO_APPEND, "\t" SW_A_catalog " %s\n", s);
-	} else {
-		/* original prior to Feb2007 */
-		strob_sprintf(buf, STROB_DO_APPEND, "\t" SW_A_path " %s\n", swi->swi_pkgM->target_pathM);
-		strob_sprintf(buf, STROB_DO_APPEND, "\t" SW_A_catalog " %s\n", swi->swi_pkgM->catalog_entryM);
-	}
-	}
-
 	swicat_isf_product(swi, buf, prod, 1);
 	if (swi->swi_pkgM->swi_coM[1]) {
 		fprintf(stderr, "multiple products not supported\n");
@@ -783,6 +744,7 @@ swicat_env(STROB * buf, SWI * swi, char * control_script_pkg_dir, char * tag)
 
 	SWLIB_ASSERT(strlen(swi->swi_pkgM->installed_software_catalogM));
 	tmp_s = strstr(swi->swi_pkgM->catalog_entryM, swi->swi_pkgM->installed_software_catalogM);
+
 	SWLIB_ASSERT(tmp_s != NULL);
 	tmp_s += strlen(swi->swi_pkgM->installed_software_catalogM);
 	while(*tmp_s == '/') tmp_s++;
@@ -794,12 +756,29 @@ swicat_env(STROB * buf, SWI * swi, char * control_script_pkg_dir, char * tag)
 	
 	add_path_code(buf);
 
-	add_env_entry("SW_SESSION_OPTIONS",
-		"${SW_ROOT_DIRECTORY}/${SW_CATALOG}/${SWBIS_CATALOG_ENTRY}/" SW_A_session_options , buf);
+	if (*(swi->swi_pkgM->installed_software_catalogM) == '/') {
+		/* The installed_software_catalog was specified as 
+		   an absolute path */
+		add_env_entry("SW_SESSION_OPTIONS",
+			"${SW_CATALOG}/${SWBIS_CATALOG_ENTRY}/" SW_A_session_options , buf);
+	} else {
+		add_env_entry("SW_SESSION_OPTIONS",
+			"${SW_ROOT_DIRECTORY}/${SW_CATALOG}/${SWBIS_CATALOG_ENTRY}/" SW_A_session_options , buf);
+	}
+
 
 	/* add SW_SOFTWARE_SPEC */
-	/* FIXME , add support for sw selections */
-	add_env_entry("SW_SOFTWARE_SPEC", "*", buf);
+
+	/* FIXME, don't assume first product */
+	/* FIXME , add tag for fileset when writing the fileset */
+	
+	/* This is a provisional value for  SW_SOFTWARE_SPEC at the top
+	   of the file.  It is defined again before each script invocation
+	   so as to reflect the tag of the fileset */
+
+	strob_strcpy(tmp, "");
+	swverid_print(SWI_GET_PRODUCT(swi, 0)->p_baseM.swveridM, tmp);
+	add_env_entry("SW_SOFTWARE_SPEC", strob_str(tmp), buf);
 
 	strob_strcat(buf, ". \"${SW_SESSION_OPTIONS}\"\n");	
 
@@ -845,18 +824,18 @@ swicat_construct_controlsh_taglist(SWI * swi, char * sw_selections, STROB * list
 	for(i=0; i<SWI_MAX_OBJ; i++) {
 		product = package->swi_coM[i];
 		if (product) {
-			SWLIB_ASSERT(product->baseM.tagM != NULL);
-			strob_strcpy(tmp, product->baseM.tagM);
-			if (swlib_check_clean_path(product->baseM.tagM)) SWLIB_ASSERT(0);
+			SWLIB_ASSERT(product->p_baseM.b_tagM != NULL);
+			strob_strcpy(tmp, product->p_baseM.b_tagM);
+			if (swlib_check_clean_path(product->p_baseM.b_tagM)) SWLIB_ASSERT(0);
 			strob_sprintf(list, STROB_DO_APPEND, "%s", strob_str(tmp));
 			for(j=0; j<SWI_MAX_OBJ; j++) {
 				fileset = product->swi_coM[j];
 				if (fileset) {
-					SWLIB_ASSERT(fileset->baseM.tagM != NULL);
-					if (swlib_check_clean_path(fileset->baseM.tagM)) SWLIB_ASSERT(0);
-					strob_strcpy(tmp, product->baseM.tagM);
+					SWLIB_ASSERT(fileset->baseM.b_tagM != NULL);
+					if (swlib_check_clean_path(fileset->baseM.b_tagM)) SWLIB_ASSERT(0);
+					strob_strcpy(tmp, product->p_baseM.b_tagM);
 					strob_strcat(tmp, ".");
-					strob_strcat(tmp, fileset->baseM.tagM);
+					strob_strcat(tmp, fileset->baseM.b_tagM);
 					strob_sprintf(list, STROB_DO_APPEND, " %s", strob_str(tmp));
 				} else break;
 			}
@@ -872,51 +851,73 @@ swicat_write_script_cases(SWI * swi, STROB * buf, char * sw_selection)
 	int ret;
 	char * product_tag;
 	char * fileset_tag;
+	SWVERID * swverid_spec;
 	SWVERID * swverid;
 	CPLOB * taglist;
 	SWI_PRODUCT * product;
 	SWI_XFILE * xfile;
 	SWI_SCRIPTS * scripts;
+	STROB * tmp;
 
+
+	tmp = strob_open(32);
 	/*
 	 * the first step is too find the object implied by
 	 * sw_selection.  sw_selection has the form:
 	 *     <prod_tag>.<fileset_tag>
 	 */
 
+	E_DEBUG2("sw_selection=[%s]", sw_selection);
+	/* E_DEBUG2("[%s]", swi_dump_string_s(swi, "AAAAA")); */
+
 	swverid = swverid_open(SW_A_product, NULL);
 	taglist = swverid_u_parse_swspec(swverid, sw_selection);
+	E_DEBUG2("swverid_print = [%s]", swverid_show_object_debug(swverid, NULL, ""));
 
-	/*
+	/* NOT
 	 * do a sanity check,
 	 * only 2 dot delimited fields are allowed.
-	 */	
-
+	 *	
 	if (cplob_get_nused(taglist) > 3) {
-		/* i.e.
-		 *    <bundle>.<product>.<fileset><NUL>
-		 */
-		SWLIB_ASSERT(0);   /* fatal */
+		SWLIB_ASSERT(0);
 	}		
+	*/
 
 	product_tag = cplob_val(taglist, 0);
 	fileset_tag = cplob_val(taglist, 1);  /* may be NUL */
 
-	product = swi_find_product_by_swsel(swi->swi_pkgM, product_tag);
+	product = swi_find_product_by_swsel(swi->swi_pkgM, product_tag, NULL, (int*)NULL);
 	SWLIB_ASSERT(product != NULL);   /* fatal */
 
 	if (fileset_tag) {
-		xfile = swi_find_fileset_by_swsel(product, fileset_tag);
+		xfile = swi_find_fileset_by_swsel(product, fileset_tag, NULL);
 	} else {
 		/*
 		 * the scripts reside in the pfiles object
 		 */
-		xfile = product->pfilesM;
+		xfile = product->xfileM;
 	}
 	SWLIB_ASSERT(xfile != NULL);   /* fatal */
 	scripts = xfile->swi_scM;
 	SWLIB_ASSERT(scripts != NULL);   /* fatal */
 
+	/*
+	swverid_print(SWI_GET_PRODUCT(swi, 0)->p_baseM.swveridM, tmp);
+	*/
+
+	/* This swverid will become the value of SW_SOFTWARE_SPEC */
+
+	/* make a copy of the swverid to avoid harming the original */
+
+	swverid_spec = swverid_copy(SWI_GET_PRODUCT(swi, 0)->p_baseM.swveridM);
+
+	if (fileset_tag) {
+		/* add fileset_tag to the '.' deimited list */
+		cplob_add_nta(swverid_spec->taglistM, strdup(fileset_tag));
+	}
+	
+	swverid_print(swverid_spec, tmp);
+
 	/*
 	 * OK, we now have the scripts for the product or fileset
 	 * Loop over them and write the shell code fragment for each
@@ -926,13 +927,17 @@ swicat_write_script_cases(SWI * swi, STROB * buf, char * sw_selection)
 		"\t#\n"
 		"\t# write script cases here\n"
 		"\t#\n");
-		
+	
+	add_env_entry("SW_SOFTWARE_SPEC", strob_str(tmp), buf);
+	
 	ret = swi_afile_write_script_cases(scripts,  buf);
 	ret = 0;
 	SWLIB_ASSERT(ret == 0);   /* fatal */
 
 	swverid_close(swverid);
+	swverid_close(swverid_spec);
 	cplob_close(taglist);
+	strob_close(tmp);
 	return 0;
 }
 
@@ -1016,11 +1021,6 @@ swicat_write_isc_script(STROB * script_buf,
 		shlib_get_function_text_by_name("shls_get_verid_list", tmp, NULL)
 		);
 
-		/*
-		target_path,
-		get_opta(G->optaM, SW_E_installed_software_catalog)			
-		*/
-
 	/* put shell functions here needed for writing the tar format here */
 	if (strcmp(list_vform, SWICAT_FORM_TAR1) == 0) {
 		strob_sprintf(script_buf, 1,
@@ -1257,6 +1257,8 @@ swicat_req_analyze(GB * G, SWICAT_REQ * req, char * query_text, SWICAT_SL ** swi
 	char * spec;
 	SWICAT_SL * swicat_sl;
 
+	E_DEBUG2("query text=[%s]", query_text);
+
 	text = strob_open(10);
 	if (swicat_sl_p) {
 		*swicat_sl_p = NULL;
