@@ -59,6 +59,18 @@ swprogs/swlist -c - bz\*  @ /  | tar tf -
 #include "fnmatch_u.h"
 #include "swgpg.h"
 
+static
+int
+show_nopen(void)
+{
+	int ret;
+	ret = /*--*/open("/dev/null",O_RDWR);
+	if (ret<0)
+		fprintf(stderr, "fcntl error: %s\n", strerror(errno));
+	close(ret);
+	return ret;
+}
+
 static
 int
 fix_offset(SWICAT_E * e)
@@ -186,18 +198,34 @@ swicat_e_create(void)
 	swicat_e->swiM = NULL;
 	swicat_e->tmp_fdM = swlib_open_memfd();
 	swicat_e->restore_offsetM = -1;
+	swicat_e->xformat_close_on_e_deleteM = 0;
+	swicat_e->taru_close_on_e_deleteM = 0;
+	swicat_e->swi_close_on_e_deleteM = 0;
 	return swicat_e;
 }
 
 void
 swicat_e_delete(SWICAT_E * swicat_e)
 {
-	if (swicat_e->entry_prefixM) free(swicat_e->entry_prefixM);
-	if (swicat_e->xformatM) xformat_close(swicat_e->xformatM);
-	if (swicat_e->swiM) swi_delete(swicat_e->swiM);
-	if (swicat_e->taruM) taru_delete(swicat_e->taruM);
-	if (swicat_e->entry_fdM >= 0) uxfio_close(swicat_e->entry_fdM);
-	if (swicat_e->tmp_fdM >= 0) uxfio_close(swicat_e->tmp_fdM);
+	if (swicat_e->entry_prefixM)
+		free(swicat_e->entry_prefixM);
+
+	if (swicat_e->xformat_close_on_e_deleteM)
+		if (swicat_e->xformatM)
+			xformat_close(swicat_e->xformatM);
+	
+	if (swicat_e->swi_close_on_e_deleteM)
+		if (swicat_e->swiM)
+			swi_delete(swicat_e->swiM);
+
+	if (swicat_e->taru_close_on_e_deleteM)
+		if (swicat_e->taruM)
+			taru_delete(swicat_e->taruM);
+
+	if (swicat_e->entry_fdM >= 0)
+		uxfio_close(swicat_e->entry_fdM);
+	if (swicat_e->tmp_fdM >= 0)
+		uxfio_close(swicat_e->tmp_fdM);
 	free(swicat_e);
 }
 
@@ -224,7 +252,11 @@ swicat_e_open_entry_tarball(SWICAT_E * swicat_e, int ifd)
 	int ret;
 	intmax_t len;
 	int mem_fd;
+	int nullarchive;
+	char * b;
 	TARU * taru;
+	char * zerobuf;
+	char * readbuf;
 	XFORMAT * xformat;
 
 	E_DEBUG("");
@@ -245,24 +277,47 @@ swicat_e_open_entry_tarball(SWICAT_E * swicat_e, int ifd)
 	
 	ret = uxfio_lseek(mem_fd, (off_t)0, SEEK_SET);
 	if (ret != 0) {
-		taru_delete(taru);
-		uxfio_close(mem_fd);	
-		return -2;
+		taru_delete(taru); uxfio_close(mem_fd);	return -2;
+	}
+
+	/* Check for empty i.e. end-of-archive */
+	nullarchive = 0;
+	zerobuf = (char*)malloc(TARRECORDSIZE);
+	readbuf = (char*)malloc(TARRECORDSIZE);
+	memset(zerobuf, '\0', TARRECORDSIZE);
+	uxfio_read(mem_fd, readbuf, TARRECORDSIZE);
+	if (memcmp(zerobuf, readbuf, TARRECORDSIZE) == 0) {
+		uxfio_read(mem_fd, readbuf, TARRECORDSIZE);
+		if (memcmp(zerobuf, readbuf, TARRECORDSIZE) == 0) {
+			/* empty archive, handle this as a special
+			   error case */
+			nullarchive = 1;	
+		}
+	}
+	free(zerobuf);
+	free(readbuf);
+
+	ret = uxfio_lseek(mem_fd, (off_t)0, SEEK_SET);
+	if (ret != 0) {
+		taru_delete(taru); uxfio_close(mem_fd);	return -2;
 	}
 
 	xformat = xformat_open(-1, -1, arf_ustar);
 	SWLIB_ASSERT(xformat != NULL);
 
-	ret = xformat_open_archive_by_fd(xformat, mem_fd, UINFILE_DETECT_NATIVE, 0);
-	if (ret != 0) {
-		uxfio_close(mem_fd);
-		taru_delete(taru);
-		xformat_close(xformat);	
-		return -3;
+	if (nullarchive == 0) {
+		ret = xformat_open_archive_by_fd(xformat, mem_fd, UINFILE_DETECT_NATIVE, 0);
+		if (ret != 0) {
+			uxfio_close(mem_fd);
+			taru_delete(taru);
+			xformat_close(xformat);	
+			return -3;
+		}
 	}
 
 	swicat_e->xformatM = xformat;
 	swicat_e->taruM = taru;
+	swicat_e->taru_close_on_e_deleteM = 1;
 	swicat_e->start_offsetM = 0;
 	swicat_e->end_offsetM = (int)len;
 	swicat_e->entry_fdM = mem_fd;
@@ -278,17 +333,22 @@ swicat_e_open_entry_tarball(SWICAT_E * swicat_e, int ifd)
 		SWLIB_ERROR3("sanity check failed ret=%d, len=%d", ret, (int)len);
 		return -5;
 	}
-
-	uxfio_lseek(mem_fd, (off_t)0, SEEK_SET);
-	ret = init_catalog_entry(swicat_e, swicat_e->xformatM->swvarfsM, (char*)NULL);
-	if (ret != 0) {
-		SWLIB_ERROR("init_catalog_entry failed");
-		return -6;
-	}
-
+	
 	uxfio_lseek(mem_fd, (off_t)0, SEEK_SET);
 
-	return 0;
+	if (nullarchive == 0) {
+		ret = init_catalog_entry(swicat_e, swicat_e->xformatM->swvarfsM, (char*)NULL);
+		if (ret != 0) {
+			SWLIB_ERROR("init_catalog_entry failed");
+			return -6;
+		}
+		retval = 0;
+	} else {
+		retval = SWICAT_RETVAL_NULLARCHIVE;
+	}	
+	uxfio_lseek(mem_fd, (off_t)0, SEEK_SET);
+
+	return retval;
 }
 
 /**
@@ -332,7 +392,7 @@ swicat_e_open_catalog_tarball(int cfd)
 }
 
 int
-swicat_e_find_attribute_file(SWICAT_E * swicat_e, char * attribute, STROB * pbuf)
+swicat_e_find_attribute_file(SWICAT_E * swicat_e, char * attribute, STROB * pbuf, AHS * ahs)
 {
 	int retval;
 	int ret;
@@ -369,6 +429,9 @@ swicat_e_find_attribute_file(SWICAT_E * swicat_e, char * attribute, STROB * pbuf
 				retval = -1;
 				break;
 			}
+			if (ahs) {
+				ahs_copy(ahs, swvarfs_get_ahs(xformat->swvarfsM));
+			}
 			ret = swlib_ascii_text_fd_to_buf(pbuf, ufd);
 			E_DEBUG2("swlib_ascii_text_fd_to_buf returned %d", ret);
 			if (ret != 0) {
@@ -395,6 +458,54 @@ swicat_e_find_attribute_file(SWICAT_E * swicat_e, char * attribute, STROB * pbuf
 	return retval;
 }
 
+SWHEADER *
+swicat_e_isf_parse(SWICAT_E * e, int *pstatus, AHS * ahs)
+{
+	STROB * tmp;
+	SWHEADER * swheader;
+	int ifd = swlib_open_memfd();
+	int ofd = swlib_open_memfd();
+	char * img;
+	char * img2;
+	int datalen;
+	int ret;
+
+	tmp = strob_open(12);
+	if (pstatus) *pstatus = 0;
+	ret = swicat_e_find_attribute_file(e, SW_A_INSTALLED, tmp, ahs);
+	if (ret != 0) {
+		/* fatal error */
+		SWLIB_ERROR("");
+		if (pstatus) *pstatus = 1;
+		return NULL;
+	} 
+
+	uxfio_write(ifd, strob_str(tmp), strob_strlen(tmp));
+	uxfio_write(ifd, "\n\n\n", 2);     /* the _e_find_attribute_file()
+					      routine strips the trailing newline,
+					      we have to put it back */
+	uxfio_lseek(ifd, (off_t)0, SEEK_SET);
+
+	ret = sw_yyparse(ifd, ofd, SW_A_INSTALLED, 0, SWPARSE_FORM_MKUP_LEN);
+	if (ret) {
+		fprintf(stderr,
+		"%s: error parsing %s\n", swlib_utilname_get(), SW_A_INSTALLED);
+		if (pstatus) *pstatus = 2;
+		return NULL;
+	}
+	uxfio_lseek(ofd, (off_t)0, SEEK_SET);
+	img = swi_com_get_fd_mem(ofd, &datalen);
+	img2 = malloc((size_t)datalen+1);
+	memcpy(img2, img, datalen);
+	img2[datalen] = '\0';	
+	swheader = swheader_open((char *(*)(void *, int *, int))(NULL), NULL);
+	swheader_set_image_head(swheader, img2);
+	uxfio_close(ifd);
+	uxfio_close(ofd);
+	strob_close(tmp);
+	return swheader;
+}
+
 SWI_FILELIST *
 swicat_e_make_file_list(SWICAT_E * e, SWUTS * swuts, SWI * swi)
 {
@@ -420,7 +531,7 @@ swicat_e_make_file_list(SWICAT_E * e, SWUTS * swuts, SWI * swi)
 	}
 	
 	E_DEBUG("");
-	ret = swicat_e_find_attribute_file(e, SW_A_location, tmp);
+	ret = swicat_e_find_attribute_file(e, SW_A_location, tmp, NULL);
 	if (ret < 0) {
 		/* fatal error */
 		SWLIB_ERROR("");
@@ -565,11 +676,13 @@ swicat_e_open_swi(SWICAT_E * e)
 	UINFORMAT * uinformat;
 
 	E_DEBUG("BEGIN");
+	E_DEBUG2("LOWEST ENTERING AA swicat_e_open_swi FD=%d", show_nopen());
 	ret = fix_offset(e);
 	if (ret < 0) {
-		SWLIB_ERROR("");	
+		SWLIB_ERROR("");
 		return NULL;
 	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	newfd = swlib_open_memfd();
 
 	ifd = e->entry_fdM;
@@ -592,18 +705,23 @@ swicat_e_open_swi(SWICAT_E * e)
 
 	E_DEBUG2("swlib_pipe_pump returned %d", ret);
 	
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	uxfio_lseek(newfd, (off_t)(0), SEEK_SET);
+	E_DEBUG2("CALLING swvarfs_opendup(%d", newfd);
        	swvarfs = swvarfs_opendup(newfd, UINFILE_DETECT_NATIVE, (mode_t)0);
+	E_DEBUG2("LOWEST AFTER swvarfs_opendup FD=%d", show_nopen());
 	if (swvarfs == NULL) {
 		E_DEBUG("");
 		return NULL;
 	}
+	swvarfs->did_dupM = 1; /* This causes swvarfs_close to close the fd */
 	E_DEBUG("searching: START");
 
 	/*
 	 * Now find the ../export/catalog.tar file and return a file descriptor
 	 */	
 
+	E_DEBUG2("LOWEST BEFORE swvarfs_get_next_dirent FD=%d", show_nopen());
 	catalog_tar_fd = -1;
         while ((name=swvarfs_get_next_dirent(swvarfs, &st)) != NULL && strlen(name)) {
 		E_DEBUG2("searching: name=%s", name);
@@ -613,6 +731,7 @@ swicat_e_open_swi(SWICAT_E * e)
 			break;
 		}
 	}
+	E_DEBUG2("LOWEST AFTER swvarfs_get_next_dirent FD=%d", show_nopen());
 	E_DEBUG("searching: END");
 	E_DEBUG2("catalog_tar_fd=%d", catalog_tar_fd);
 	if (catalog_tar_fd < 0) {
@@ -625,14 +744,29 @@ swicat_e_open_swi(SWICAT_E * e)
 	 * Now transfer this file to its own dedicated file descriptor
 	 */
 
+	E_DEBUG2("UXFIO fd: %s",(uxfio_show_all_open_fd(stderr), ""));	
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
+	E_DEBUG2("UXFIO fd: %s",(uxfio_show_all_open_fd(stderr), ""));	
 	newfd2 = swlib_open_memfd();
+	E_DEBUG("");
+	E_DEBUG2("UXFIO fd: %s",(uxfio_show_all_open_fd(stderr), ""));	
 	ret = swlib_pipe_pump(newfd2, catalog_tar_fd);
+	E_DEBUG("");
+	E_DEBUG2("UXFIO fd: %s",(uxfio_show_all_open_fd(stderr), ""));	
 	uxfio_lseek(newfd2, (off_t)(0), SEEK_SET);
+	E_DEBUG2("catalog_tar_fd is %d", catalog_tar_fd);
+	E_DEBUG2("UXFIO fd: %s",(uxfio_show_all_open_fd(stderr), ""));	
 	swvarfs_u_close(swvarfs, catalog_tar_fd);
+	E_DEBUG("");
+	E_DEBUG("");
+	E_DEBUG2("UXFIO fd: %s",(uxfio_show_all_open_fd(stderr), ""));	
 	swvarfs_close(swvarfs);
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
+	E_DEBUG2("UXFIO fd: %s",(uxfio_show_all_open_fd(stderr), ""));	
 	
-	uxfio_close(newfd);
+	/* uxfio_close(newfd); */
 	ifd = newfd2;
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 
 	/*
 	 * Now create the SWI object which models the in-memory structures
@@ -640,8 +774,10 @@ swicat_e_open_swi(SWICAT_E * e)
 	 */
 	
 	E_DEBUG("");
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	swi = swi_create();
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	/*
 	 * open the package catalog
 	 */
@@ -657,15 +793,24 @@ swicat_e_open_swi(SWICAT_E * e)
 		return NULL;
 	}
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	/*
 	 * fill in the (SWI*) structure
 	 */
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	swi->xformatM = xformat;
+	swi->xformat_close_on_deleteM = 1;
+	if (e->xformatM)
+		xformat_close(e->xformatM);
 	e->xformatM = xformat;
 	swi->swvarfsM = xformat->swvarfsM;
+	xformat->swvarfs_is_externalM = 1;
+	swi->swvarfs_close_on_deleteM = 1;
 	swi->uinformatM = swvarfs_get_uinformat(swi->swvarfsM);
 	uinformat = swvarfs_get_uinformat(swi->swvarfsM);
+	swi->swvarfsM->uinformat_close_on_deleteM = 0;
+	swi->uinformat_close_on_deleteM = 1;
 	swpath = uinfile_get_swpath(uinformat);
 	swpath_reset(swpath);
 	xformat_set_tarheader_flag(xformat, TARU_TAR_FRAGILE_FORMAT, 1);
@@ -676,22 +821,27 @@ swicat_e_open_swi(SWICAT_E * e)
 	xformat_set_ifd(swi->xformatM, ifd);
 	E_DEBUG("");
 
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	/* FIXME, also check the GPG signature here */
 
 	/*
          * Decode the package catalog
 	 */
 	
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	ret = swi_decode_catalog(swi);
 	if (ret < 0) {
 		E_DEBUG("error");
 		unfix_offset(e);
 		return NULL;
 	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	uxfio_close(ifd);  /* newfd2 */
 	unfix_offset(e);
 	set_entry_fd(e);
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
 	E_DEBUG2("swi=%p", (void*)swi);
+	E_DEBUG2("LOWEST LEAVING AA swicat_e_open_swi FD=%d", show_nopen());
 	return swi;
 }
 
