@@ -2,7 +2,7 @@
  */
 
 /*
-  Copyright (C) 2004,2005,2006,2007 Jim Lowe
+  Copyright (C) 2004,2005,2006,2007,2010 Jim Lowe
   All Rights Reserved.
   
   COPYING TERMS AND CONDITIONS:
@@ -183,7 +183,9 @@ task_wait_for(SWICOL * swicol, STROB * retbuf,
 	int readReturn;
 	STROB * buf = strob_open(180);
 	char * s;
+	int alarm_monitor;
 	time_t start = time(NULL);
+	time_t alarm_start = time(NULL);
 	time_t now = start;
 	struct timespec req;
               
@@ -193,17 +195,38 @@ task_wait_for(SWICOL * swicol, STROB * retbuf,
 	E_DEBUG2("Entering: %s", stop_string);	
 	if (retbuf) strob_strcpy(retbuf, "");
 
+	alarm_start = (time_t)(0);
+	alarm_monitor = 0;
 	cret = 0;
 	do_stop = 0;
 	E_DEBUG("");	
 	while(
-		(cret == 0) || 
-		(do_stop == 0 && ((int)(now - start) < timelimit))
+		(alarm_monitor == 0) &&
+		(
+			(cret == 0) || 
+			(do_stop == 0 && ((int)(now - start) < timelimit))
+		)
 	     )
 	{
 		nanosleep(&req, (struct timespec *)(NULL));
 		now = time(NULL);
+
+		if (swicol->master_alarmM) {
+			/* allow a several second delay before terminating
+			   this loop to allow the signals to perculate */
+			E_DEBUG("Got Alarm");
+			if (alarm_start == (time_t)(0)) {
+				E_DEBUG("Set Alarm Start");
+				alarm_start = time(NULL);
+			}
+			if ((now - alarm_start) > 1 /*seconds*/) {
+				E_DEBUG("Alarm Expired setting alarm_monitor");
+				alarm_monitor = 1;
+			}
+		}
+
 		ret = swgpReadLine(buf, event_fd, &readReturn);
+
 		if (cret == 0) {
 			cret = ret;
 		}
@@ -263,7 +286,7 @@ task_wait_for(SWICOL * swicol, STROB * retbuf,
 		if (ret && retbuf) 
 			strob_strcat(retbuf, strob_str(buf));
 	}
-	if (do_stop == 0) {
+	if (do_stop == 0 && swicol->master_alarmM == 0) {
 		/*
 		 * time limit exceeded.
 		 */
@@ -274,7 +297,13 @@ task_wait_for(SWICOL * swicol, STROB * retbuf,
 				"SW_RESOURCE_ERROR: time limit of %d seconds exceeded\n",
 							timelimit);
 
+	} else if (alarm_monitor || swicol->master_alarmM) {
+		E_DEBUG("ALARM");	
+		retval = 2;
+		swutil_doif_writef(swicol->verbose_levelM, 1, swicol->logspecM,
+				STDERR_FILENO, "SW_ABORT_SIGNAL_RECEIVED: user interrupt\n");
 	}
+
 	strob_close(buf);
 	E_DEBUG2("Leaving: %d", retval);	
 	return retval;
@@ -391,7 +420,7 @@ print_task_script(SWICOL * swicol, STROB * command, uintmax_t size, char * dir,
 		tonullstring = "2>/dev/null";
 	}
 
-	strob_sprintf(enddesc, 0, "%s: status=$sw_retval", task_desc);
+	strob_sprintf(enddesc, 0, "%s: " SWEVENT_STATUS_PFX "$sw_retval", task_desc);
 
 	ablocks = swlib_umaxtostr(blocks, ubuf);
 
@@ -399,6 +428,7 @@ print_task_script(SWICOL * swicol, STROB * command, uintmax_t size, char * dir,
 		"{\n"
 		"dd bs=512 count=%s %s | (\n"
 		"# trap true 1 2 13 14 15\n"
+		CSHID
 		"%s\n"			/* SWI_TASK_BEGINS */
 		"swxdir=\"%s\"\n"
 		"cd \"$swxdir\"\n"
@@ -711,6 +741,7 @@ swicol_rpsh_get_event_message(SWICOL * swicol, int event_value,
 	}
 	strob_close(tmp);
 	if (p_index) *p_index = --ix;
+	E_DEBUG2("returning message: <<<<<<<< [%s]", retval);
 	return retval;
 }
 
@@ -728,16 +759,22 @@ swicol_rpsh_get_event_status(SWICOL * swicol, char * msg,
 	 * the message is the status
 	 */
 	if (!msg) return -1;
+	E_DEBUG2("Message [%s]", msg);	
+	
 	if (isdigit((int)(*msg))) {
 		retval = swlib_atoi(msg, &atoi_ret);
 		if (atoi_ret) {
-			retval = -127;
+			retval = SW_INTERNAL_ERROR_127;
 		}
+	} else if (strcmp(msg, SWEVENT_STATUS_PFX SWEVENT_VALUE_PREVIEW) == 0) {
+		/* status=preview */
+		/* special status for preview mode in swconfig */
+		retval = 0;
 	} else {
 		fprintf(stderr,
-			"%s: event %d does not return a status\n",
+			"%s: event %d did not return a valid status\n",
 			swlib_utilname_get(), event_value);
-		retval = -128;
+		retval = SW_INTERNAL_ERROR_128;
 	}
 	return retval;
 }
@@ -763,7 +800,9 @@ swicol_rpsh_task_expect(SWICOL * swicol, int event_fd, int timelimit)
 	 * wait for the SWI_TASK_ENDS event
 	 */
 	ret = swicol_rpsh_task_wait(swicol, buf, event_fd, timelimit);
-	retval = ret;
+	/* fprintf(stderr, "swicol_rpsh_task_wait returned [%d]\n", ret); */
+	if (ret)
+		retval = -4;
 	
 	E_DEBUG2("expect buffer=[%s]", strob_str(buf));
 
@@ -782,7 +821,7 @@ swicol_rpsh_task_expect(SWICOL * swicol, int event_fd, int timelimit)
 		if (isdigit((int)(*ev)) == 0) {
 			fprintf(stderr, "%s: error: END event has invalid status: %s status=%s\n",
 				swlib_utilname_get(), strob_str(swicol->id_stringM), strob_str(buf));
-			retval = -1;
+			retval = -2;
 		} else {
 			retval = swlib_atoi(ev, NULL);
 		}
