@@ -43,7 +43,7 @@
 #include "swheaderline.h"
 
 #define SWICOL_TRAILER 	CPIO_INBAND_EOA_FILENAME
-#define SWICOL_ALARM_EVENT "Commanded abort"
+#define SWICOL_ALARM_EVENT "Intentional abort"
 #define SWICOL_STATUS_NOMINAL_ERROR	129
 #define SWICOL_ABORT_STEP_1 1
 #define SWICOL_ABORT_STEP_2 2
@@ -90,7 +90,10 @@
 #define SWICOL_TL_80	80
 #define SWICOL_TL_100	100
 #define SWICOL_TL_3600	3600
-#define SWICOL_TL_UL	90000000
+#define SWICOL_TL_6h	(SWICOL_TL_3600 * 6)
+#define SWICOL_TL_12h	(SWICOL_TL_3600 * 12)
+#define SWICOL_TL_1d	(SWICOL_TL_3600 * 24)
+#define SWICOL_TL_UL	(SWICOL_TL_1d * 10000)
 
 typedef struct {
 	struct sw_logspec * logspecM;
