@@ -68,7 +68,7 @@ static struct timespec * io_req = (struct timespec*)NULL;
 static int g_burst_adjust = 65000;
 
 #define TEE_DB_FILE "/tmp/synct_buf.bin"
-#define DO_TEE 0
+#define DO_TEE 0	/* this is for debugging */
 	
 static
 ssize_t
@@ -544,12 +544,12 @@ struct timespec ** swlib_get_io_req_p(void)
 }
 
 int
-swlib_test_verbose(int verbose_level, int swbis_event,
+swlib_test_verbose(struct swEvents * ev, int verbose_level, int swbis_event,
 		int is_swi_event, int event_status, int is_posix_event)
 {
 	if (
-		(verbose_level >= SWC_VERBOSE_1 && 
-				event_status != 0) ||
+		(ev && (verbose_level >= ev->verbose_threshholdM)) ||
+		(verbose_level >= SWC_VERBOSE_1 && event_status != 0) ||
 		(verbose_level >= SWC_VERBOSE_3 && is_posix_event) ||
 		(verbose_level >= verbose_swbis_event_thresholdG && swbis_event) ||
 		(verbose_level >= SWC_VERBOSE_6 && is_swi_event) ||
@@ -1995,7 +1995,7 @@ swlib_write_signing_files(XFORMAT * package, int ofd,
 	if (!swpath) return -21;
 	if (ifd < 0) return -32;
 
-	nullfd = open("/dev/null", O_RDWR, 0);
+	nullfd = swbis_devnull_open("/dev/null", O_RDWR, 0);
 	if (nullfd < 0) return -2;
 
 	while ((ret = xformat_read_header(package)) > 0) {
@@ -2087,7 +2087,7 @@ swlib_write_signing_files(XFORMAT * package, int ofd,
 				xformat_get_tarheader_flags(package));
 	}
 
-	close(nullfd);
+	swbis_devnull_close(nullfd);
 	strob_close(namebuf);
 	swpath_close(swpath);
 	return retval;
@@ -2222,7 +2222,7 @@ swlib_digests(int ifd, char * md5, char * sha1, char * size, char * sha512)
 	if (sha1) strcpy(sha1, digs->sha1);
 	if (sha512) strcpy(sha512, digs->sha512);
 	if (size) strcpy(size, digs->size);
-	free(digs);
+	taru_digs_delete(digs);
 	if (im >= 0) return 0;
 	return -1;
 }
@@ -2235,7 +2235,7 @@ swlib_md5_copy(int ifd, intmax_t count, char * md5, int ofd)
 	digs = taru_digs_create();
 	im = swlib_digs_copy(ofd, ifd, count, digs, -1);
 	strcpy(md5, digs->md5);
-	free(digs);
+	taru_digs_delete(digs);
 	return im;
 }
 
@@ -2262,8 +2262,8 @@ swlib_shcmd_output_fd(SHCMD ** cmdvec)
 	if (vc == cmdvec) return -1;
 	vc--;
 	lastcmd = *vc;
-
-	fd = uxfio_open("/dev/null", O_RDONLY, 0);
+	
+	fd = uxfio_open("", O_RDONLY, 0);
 	if (fd < 0) return fd;
 	uxfio_fcntl(fd, UXFIO_F_SET_BUFACTIVE, UXFIO_ON);
 	uxfio_fcntl(fd, UXFIO_F_SET_BUFTYPE, UXFIO_BUFTYPE_DYNAMIC_MEM);
@@ -2328,27 +2328,12 @@ swlib_apply_mode_umask(char type, mode_t umask, mode_t mode) {
 	return mode;	
 }
 
-int
-swlib_open_nullfd(void)
-{
-	int fd = open("/dev/null", O_RDWR, 0);
-	if (fd < 0) {
-		SWLIB_ERROR2("/dev/null: %s", strerror(errno));
-		SWLIB_FATAL("error opening /dev/null");
-	}
-	return fd;
-}
-
-int
-swlib_close_nullfd(int fd)
-{
-	return close(fd);
-}
 
 int
 swlib_open_memfd(void)
 {
-	int fd = uxfio_open("/dev/null", O_RDONLY, 0);
+	int fd;
+	fd = uxfio_open("", O_RDWR, 0); 
 	if (fd < 0) return fd;
 	uxfio_fcntl(fd, UXFIO_F_SET_BUFACTIVE, UXFIO_ON);
 	uxfio_fcntl(fd, UXFIO_F_SET_BUFTYPE, UXFIO_BUFTYPE_DYNAMIC_MEM);
@@ -2768,19 +2753,8 @@ swlib_attribute_check_default(char * object, char * keyword, char * value)
 int
 swlib_is_option_true(char * s)
 {
-	char * CHARTRUE = "true";
-	if (
-		s == (char*)NULL ||
-		(
-			strcasecmp(s, CHARTRUE) != 0 &&
-			strcasecmp(s, "y") != 0 &&
-			1
-		) ||
-		0
-	) {
-		return 0; /* not true */
-	}
-	return 1; /* 1 == yes, true */
+	if (!s) return 0;
+	return swextopt_is_value_true(s);
 }
 
 void
