@@ -97,12 +97,13 @@
 #define SWLIB_ERROR3(A,B,C) swutil_cpp_doif_writef(1,1, (void*)NULL, STDERR_FILENO, __FILE__, __LINE__, __FUNCTION__, "ERROR: " A "\n", B, C)
 #define SWLIB_ERROR4(A,B,C,D) swutil_cpp_doif_writef(1,1, (void*)NULL, STDERR_FILENO, __FILE__, __LINE__, __FUNCTION__, "ERROR: " A "\n", B, C, D)
 
+#define SWBIS_STDIO_FNAME	"-"
 
+#define SWBIS_PGM_SH		"sh"
+#define SWBIS_PGM_BIN_SH	"/bin/sh"
 #define SWBIS_LOGGER_SIGTERM		SIGUSR2
-
 #define SWBIS_TAINTED_CHARS		"'\"|*?;&<>`$[]"
 #define SWBIS_WS_TAINTED_CHARS		"\n\r\t "
-
 #define SWBIS_DEFAULT_CATALOG_USER		"root"
 #define SWBIS_DEFAULT_CATALOG_GROUP		"root"
 #define SWBIS_DEFAULT_DISTRIBUTION_USER		"root"
@@ -134,8 +135,6 @@
 
 #define SWBIS_PROGS_LAYOUT_VERSION 		"1.0"
 
-#define SWBIS_NULL_FD 	UXFIO_NULL_FD		/* Provides optimized writing to /dev/null */
-
 #define SW_EXIT_SUCCESS	0
 #define SW_EXIT_ONE	1	/* Medium not modified. */
 #define SW_EXIT_TWO	2	/* Medium modified. */
@@ -156,6 +155,7 @@
 /* #include "swutillib.h" */
 #include	<termios.h>
 #include	<sys/ioctl.h>	/* 44BSD requires this too */
+#include "swevents.h"
 
 
 typedef struct {
@@ -257,8 +257,6 @@ int 		swlib_audit_distribution(XFORMAT * xformat, int do_re_encode, int ofd, uin
 			int * deadman, void (*alarm_handler)(int));
 mode_t		swlib_apply_mode_umask(char type, mode_t umask, mode_t mode);
 int 		swlib_open_memfd(void);
-int 		swlib_open_nullfd(void);
-int 		swlib_close_nullfd(int fd);
 int 		swlib_get_verbose_level(void);
 void 		swlib_set_verbose_level(int n);
 size_t		swlib__strlcpy(char * dst, const char * src, size_t siz);
@@ -266,13 +264,13 @@ int 		swlib_pad_amount(int fd, int amount);
 int 		swlib_tr(char * src, int to, int from);
 int 		swlib_process_hex_escapes(char * s1);
 int 		swlib_compare_8859(char * s1, char * s2);
-int 		swlib_test_verbose(int verbose_level, int swbis_event,
+int 		swlib_test_verbose(struct swEvents *, int verbose_level, int swbis_event,
 int		is_swi_event, int event_status, int is_posix_event);
 int 		swlib_drop_root_privilege(void);
 struct timespec ** swlib_get_io_req_p(void);
 struct timespec * swlib_get_io_req(void);
 int *		swlib_burst_adjust_p(void);
-uintmax_t ** swlib_pump_get_ppstatbytes(void);
+uintmax_t **	swlib_pump_get_ppstatbytes(void);
 int 		swlib_vrelpath_compare(char * s1, char * s2, char * cwd);
 void	 	swlib_add_trailing_slash(STROB *path);
 int 		swlib_check_clean_relative_path(char * s);
@@ -280,32 +278,31 @@ int 		swlib_check_clean_path(char * s);
 int 		swlib_check_clean_absolute_path(char * s);
 int 		swlib_check_safe_path(char * s);
 void		swlib_internal_error(char * reason, char * file, int line, char * function);
-int swlib_expand_escapes(char **pa, int *newlen, char *src, STROB * ustore);
-int swlib_unexpand_escapes(STROB * dest, char *src);
-int swlib_unexpand_escapes2(char *src);
-int swlib_rpmvercmp(const char * a, const char * b);
-int swlib_check_legal_tag_value(char * s);
-int swlib_altfnmatch(char * s1, char * s2);
-int swlib_doif_writeap(int fd, STROB * buffer, char * format, va_list * pap);
-int swlib_tee_to_file(char * filename, int source_fd, char * source_buf, int len, int do_append);
-int swlib_synct_read(SYNCT *, int fd, void * buf);
-int swlib_synct_suck(int ofd, int ifd);
-void swlib_synct_delete(SYNCT * synct);
-SYNCT * swlib_synct_create(void);
-char * swlib_umaxtostr(uintmax_t i, STROB * buf);
-char * swlib_imaxtostr(intmax_t i, STROB * buf);
-mode_t swlib_get_umask(void);
-int swlib_ascii_text_fd_to_buf(STROB * pbuf, int ifd);
-void swlib_apply_location(STROB * relocated_path, char * path, char * location, char * directory);
-char * swlib_attribute_check_default(char * object, char * keyword, char * value);
-void std_quicksort (void *const pbase, size_t total_elems, size_t size, int (*f)(const void *, const void *));
-int swlib_is_option_true(char * s);
-void swlib_set_swbis_verbose_threshold(int s);
-int swlib_squash_trailing_char(char *path, char ch);
-int swlib_unix_dirtrunc(STROB * buf);
-int swlib_unix_dirtrunc_n(STROB * buf, int n);
-void swlib_squash_illegal_tag_chars(char * s);
-intmax_t swlib_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int);
-void std_quicksort (void *const pbase, size_t total_elems, size_t size, int  (*)(const void *, const void *));
-mode_t swlib_filestring_to_mode(char * str, int do_include_type);
+int 		swlib_expand_escapes(char **pa, int *newlen, char *src, STROB * ustore);
+int 		swlib_unexpand_escapes(STROB * dest, char *src);
+int 		swlib_unexpand_escapes2(char *src);
+int 		swlib_rpmvercmp(const char * a, const char * b);
+int 		swlib_check_legal_tag_value(char * s);
+int 		swlib_altfnmatch(char * s1, char * s2);
+int 		swlib_doif_writeap(int fd, STROB * buffer, char * format, va_list * pap);
+int 		swlib_tee_to_file(char * filename, int source_fd, char * source_buf, int len, int do_append);
+int 		swlib_synct_read(SYNCT *, int fd, void * buf);
+int 		swlib_synct_suck(int ofd, int ifd);
+void 		swlib_synct_delete(SYNCT * synct);
+SYNCT * 	swlib_synct_create(void);
+char * 		swlib_umaxtostr(uintmax_t i, STROB * buf);
+char *		swlib_imaxtostr(intmax_t i, STROB * buf);
+mode_t 		swlib_get_umask(void);
+int 		swlib_ascii_text_fd_to_buf(STROB * pbuf, int ifd);
+void 		swlib_apply_location(STROB * relocated_path, char * path, char * location, char * directory);
+char * 		swlib_attribute_check_default(char * object, char * keyword, char * value);
+int 		swlib_is_option_true(char * s);
+void 		swlib_set_swbis_verbose_threshold(int s);
+int 		swlib_squash_trailing_char(char *path, char ch);
+int 		swlib_unix_dirtrunc(STROB * buf);
+int 		swlib_unix_dirtrunc_n(STROB * buf, int n);
+void 		swlib_squash_illegal_tag_chars(char * s);
+intmax_t 	swlib_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int);
+void 		std_quicksort (void *const pbase, size_t total_elems, size_t size, int  (*)(const void *, const void *));
+mode_t		 swlib_filestring_to_mode(char * str, int do_include_type);
 #endif
