@@ -18,6 +18,7 @@
 
 #define FILENEEDDEBUG 1
 #undef FILENEEDDEBUG
+
 #include "swuser_config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -161,23 +162,13 @@ swutil_setup_xformat(SWLOG * swutil,
 		int uinfile_open_flags
 	)
 {
-	/* XFORMAT * xformat; */
-	/* int format; */
-
 	XFORMAT * ret;
 	int open_error = 0;
 	int flags;
 
-	/* format = arf_ustar; */
-	/* xformat = xformat_open(STDIN_FILENO, target_fd1, format); */
-
-	/*
-	flags = UINFILE_DETECT_FORCEUXFIOFD |
-		UINFILE_DETECT_UNCPIO |
-		UINFILE_DETECT_IEEE;
-	*/
 	flags = uinfile_open_flags;
 	
+	E_DEBUG("");
 	if (is_seekable) {
 		flags |= UINFILE_UXFIO_BUFTYPE_DYNAMIC_MEM;
 		swlib_doif_writef(g_verboseG, SWC_VERBOSE_7, 
@@ -203,21 +194,42 @@ swutil_setup_xformat(SWLOG * swutil,
 	 * Open the source archive.
 	 *
 	 */
-	if (source_fd0 >= 0 && open_error == 0) {
+	E_DEBUG2("source_path is [%s]", source_path);
+	if (
+		source_fd0 >= 0 &&
+		(source_path == NULL || uinfile_open_flags & UINFILE_DETECT_IEEE) && 
+		open_error == 0)
+	{
 		/*
 		 * Open the archive on fd
 		 */
+		E_DEBUG("open by fd");
+		E_DEBUG2("source_path is [%s]", source_path);
 		if (xformat_open_archive_by_fd(xformat, 
 						source_fd0, flags, 0)) {
 			swlib_doif_writef(g_verboseG, g_logspec->fail_loudlyM, 
 				g_logspec, swutil->swu_efdM, "archive open failed\n");
 			xformat_close(xformat);
+			E_DEBUG("open error");
+			ret = (XFORMAT*)NULL;
+		}
+		E_DEBUG("after xformat_open_archive_by_fd");
+
+	} else if (source_fd0 >= 0 && source_path && open_error == 0) {
+		E_DEBUG("~~~~~~~~~~~~~~~ open by fd and name");
+		/* fprintf(stderr, "JL you're a SLACKER: %s:%s at line %d\n", __FILE__, __FUNCTION__, __LINE__); */
+		if (xformat_open_archive_by_fd_and_name(xformat, source_fd0, flags, 0, source_path)) {
+			swlib_doif_writef(g_verboseG, g_logspec->fail_loudlyM, 
+				g_logspec, swutil->swu_efdM, "archive open failed\n");
+			xformat_close(xformat);
+			E_DEBUG("open error");
 			ret = (XFORMAT*)NULL;
 		}
 	} else if (source_fd0 < 0 && open_error == 0) {
 		/*
 		 * Open the archive in the directory.
 		 */
+		E_DEBUG("open file or directory");
 		if (xformat_open_archive_dirfile(xformat, 
 					source_path, flags, 0)) {
 			swlib_doif_writef(g_verboseG, g_logspec->fail_loudlyM, 
@@ -230,11 +242,11 @@ swutil_setup_xformat(SWLOG * swutil,
 		/*
 		 * Open error.
 		 */
+		E_DEBUG("open error");
 		swlib_doif_writef(g_verboseG, g_logspec->fail_loudlyM, g_logspec, swutil->swu_efdM,
 			"%s not found\n", source_path);
 		ret = (XFORMAT*)NULL;
 	}
-
 	return ret;
 }
 
