@@ -58,6 +58,17 @@ NOTE:
 
 #define u_fd_array_len(a)	((int)(sizeof(a->u_fd_setM)/sizeof(SWVARFS_U_FD_DESC)))
 
+static
+int
+show_nopen(void)
+{
+	int ret;
+	ret = /*--*/open("/dev/null",O_RDWR);
+	if (ret<0)
+		fprintf(stderr, "fcntl error: %s\n", strerror(errno));
+	close(ret);
+	return ret;
+}
 
 static void
 ud_init(SWVARFS * swvarfs)
@@ -702,6 +713,7 @@ swvarfs_get_next_dirent_fs(SWVARFS * swvarfs, struct stat *st)
 	}
 
 	if (swvarfs->stackixM) {  /* The 0th index is the not-used index. */
+		SWVARFS_E_DEBUG(""); 
 		get_dir_context(swvarfs, &dirx);
  		dirp = dirx.dirp;
 		dp = dirx.dp;
@@ -718,6 +730,7 @@ swvarfs_get_next_dirent_fs(SWVARFS * swvarfs, struct stat *st)
 	} else {
 		/* Open the root (given) directory.
 		 */
+		SWVARFS_E_DEBUG(""); 
 
 		swvarfs_setdir(swvarfs, fullpath);
 
@@ -730,15 +743,18 @@ swvarfs_get_next_dirent_fs(SWVARFS * swvarfs, struct stat *st)
 
 	do {	
 		while ( (dirp = readdir(dp)) != NULL) {
+			SWVARFS_E_DEBUG(""); 
 			if (strcmp(dirp->d_name, ".") == 0  ||
 				strcmp(dirp->d_name, "..") == 0) {
 				continue;
 			}
+			SWVARFS_E_DEBUG(""); 
 			strob_strcat(swvarfs->direntpathM, "/");
 			strob_strcat(swvarfs->direntpathM, dirp->d_name);
 			fullpath = strob_str(swvarfs->direntpathM);
 			stat_node(swvarfs, fullpath, statbuf);
 
+			SWVARFS_E_DEBUG(""); 
 			if (S_ISDIR(statbuf->st_mode)) {
 						/* is a directory */
 				olddp = dp;
@@ -762,12 +778,14 @@ swvarfs_get_next_dirent_fs(SWVARFS * swvarfs, struct stat *st)
 			return fullpath;	
 			break;
 		}
+		SWVARFS_E_DEBUG(""); 
 		if (dirp == NULL) {  /* reach last entry */
 			raise_dir_context(swvarfs, &dirx);
 			delete_last_component(swvarfs, fullpath);
 			dp = dirx.dp;
 		}
 	} while (dp);
+	SWVARFS_E_DEBUG("LEAVING"); 
 	swvarfs->derrM = 0;
 	return (char*)(NULL);
 }
@@ -823,6 +841,8 @@ swvarfs_i_init(SWVARFS * swvarfs)
 			Cannot set this NULL here!!  
 			swvarfs->format_descM = (UINFORMAT*)NULL; 
 	*/
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
+	swvarfs->uinformat_close_on_deleteM = 1;
 	swvarfs->opencwdpathM = strob_open(32);
 	swvarfs->openpathM = strob_open(32);
 	swvarfs->dirscopeM = strob_open(32);
@@ -844,6 +864,7 @@ swvarfs_i_init(SWVARFS * swvarfs)
 	swvarfs->direntpathM = strob_open(16);
 	swvarfs->stackixM = 0;
 	swvarfs->derrM = 0;
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
 	swvarfs->have_read_filesM = 0;
 	swvarfs->current_filechainM = NULL;
 	swvarfs->uxfio_buftype = UXFIO_BUFTYPE_FILE;
@@ -857,9 +878,12 @@ swvarfs_i_init(SWVARFS * swvarfs)
 	swvarfs->did_u_openM = -1;
 	ud_init(swvarfs);
 	swvarfs->f_statM = (int (*)(char *, struct stat *))(lstat);
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
 	swvarfs->link_recordM = hllist_open();
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
 	swvarfs->taruM = taru_create();
 	swvarfs->taruM->taru_tarheaderflagsM = 0;
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
 }
 
 static
@@ -1877,19 +1901,20 @@ swvarfs_open(char * name, int oflags, mode_t p_mode)
 }
 
 SWVARFS *
-swvarfs_opendup(int ifd, int oflags, mode_t mode)
+swvarfs_opendup_with_name(int ifd, int oflags, mode_t mode, char * name)
 {
 	SWVARFS *swvarfs=(SWVARFS*)malloc(sizeof(SWVARFS));
 
 	SWVARFS_E_DEBUG2("ENTERING mode=%d", oflags); 
 	if (swvarfs == NULL) return NULL;
 
-	SWVARFS_E_DEBUG(""); 
-	swvarfs->fdM = uinfile_opendup(ifd, (mode_t)(0),
-				&(swvarfs->format_descM), oflags);
-	SWVARFS_E_DEBUG(""); 
+	if (name == NULL)
+		return swvarfs_opendup(ifd, oflags, mode);
+
+	/* fprintf(stderr, "JL you're a SLACKER: %s:%s at line %d\n", __FILE__, __FUNCTION__, __LINE__); */
+	swvarfs->fdM = uinfile_opendup_with_name(ifd, (mode_t)(0),
+				&(swvarfs->format_descM), oflags, name);
 	if (swvarfs->fdM < 0) {
-		SWVARFS_E_DEBUG(""); 
 		swbis_free(swvarfs);
 		return (SWVARFS*)(NULL);
 	} else if (swvarfs->fdM == ifd) {
@@ -1898,15 +1923,51 @@ swvarfs_opendup(int ifd, int oflags, mode_t mode)
 		swvarfs->did_dupM=1;
 	}
 
-	SWVARFS_E_DEBUG(""); 
 	swvarfs_i_init(swvarfs);
-	SWVARFS_E_DEBUG(""); 
 	if (swvarfs_i_iopen(swvarfs, oflags))
 		return NULL;
 	swvarfs->formatM=uinfile_get_type(swvarfs->format_descM);
 	return swvarfs;
 }
 
+SWVARFS *
+swvarfs_opendup(int ifd, int oflags, mode_t mode)
+{
+	SWVARFS *swvarfs=(SWVARFS*)malloc(sizeof(SWVARFS));
+
+	SWVARFS_E_DEBUG2("ENTERING mode=%d", oflags); 
+	if (swvarfs == NULL) return NULL;
+
+	SWVARFS_E_DEBUG(""); 
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
+	swvarfs->fdM = uinfile_opendup(ifd, (mode_t)(0),
+				&(swvarfs->format_descM), oflags);
+	SWVARFS_E_DEBUG(""); 
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
+	if (swvarfs->fdM < 0) {
+		SWVARFS_E_DEBUG(""); 
+		swbis_free(swvarfs);
+		return (SWVARFS*)(NULL);
+	} else if (swvarfs->fdM == ifd) {
+		SWVARFS_E_DEBUG("did_dup = 0"); 
+		swvarfs->did_dupM=0;
+	} else {
+		SWVARFS_E_DEBUG("did_dup = 1"); 
+		swvarfs->did_dupM=1;
+	}
+
+	SWVARFS_E_DEBUG(""); 
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
+	swvarfs_i_init(swvarfs);
+	SWVARFS_E_DEBUG(""); 
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
+	if (swvarfs_i_iopen(swvarfs, oflags))
+		return NULL;
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
+	swvarfs->formatM=uinfile_get_type(swvarfs->format_descM);
+	return swvarfs;
+}
+
 int
 swvarfs_u_readlink(SWVARFS * swvarfs, char * path, char * buf, size_t bufsize)
 {
@@ -2184,8 +2245,14 @@ swvarfs_close(SWVARFS * swvarfs)
 {
 	int ret = 0;
 	SWVARFS_E_DEBUG("ENTERING"); 
-	if (swvarfs->did_dupM)
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
+	if (swvarfs->did_dupM) {
+		SWVARFS_E_DEBUG("did_dupM: CLOSING swvarfs->fdM"); 
 		uxfio_close(swvarfs->fdM);
+	} else {
+		SWVARFS_E_DEBUG("did_dupM: NOT CLOSING swvarfs->fdM"); 
+		;
+	}
 	strob_close(swvarfs->opencwdpathM);
 	strob_close(swvarfs->openpathM);
 	strob_close(swvarfs->u_nameM);
@@ -2197,12 +2264,17 @@ swvarfs_close(SWVARFS * swvarfs)
 		strob_close(swvarfs->direntpathM);
 	if (swvarfs->do_close_ahsM)
 		ahs_close(swvarfs->ahsM);
-	if (swvarfs->format_descM) 
-		ret = uinfile_close(swvarfs->format_descM);
+	if (swvarfs->uinformat_close_on_deleteM) {
+		if (swvarfs->format_descM) 
+			ret = uinfile_close(swvarfs->format_descM);
+	} else {
+		ret = 0;
+	}
 	swvarfs_i_delete_filechain(swvarfs);
 	hllist_close(swvarfs->link_recordM);
 	taru_delete(swvarfs->taruM);
 	swbis_free(swvarfs);
+	SWVARFS_E_DEBUG2("LOWEST FD=%d", show_nopen()); 
 	return ret;
 }
 
@@ -2416,34 +2488,43 @@ swvarfs_get_next_dirent(SWVARFS * swvarfs, struct stat *st)
 	int tartype;
 	int is_at_end;
 	char * name;
+	char *s;
 	SWVARFS_E_DEBUG("ENTERING"); 
 	if (swvarfs->formatM==UINFILE_FILESYSTEM) {
+		SWVARFS_E_DEBUG(""); 
 		name = swvarfs_get_next_dirent_fs(swvarfs, st);
 		is_at_end = 0;
+		SWVARFS_E_DEBUG(""); 
 	} else {
 		int * peoa = &(swvarfs->eoaM);
+		SWVARFS_E_DEBUG(""); 
 		name = swvarfs_get_next_dirent_archive_NEW(swvarfs, st, peoa);
 		is_at_end = *peoa;
+		SWVARFS_E_DEBUG(""); 
 	}
 
+	SWVARFS_E_DEBUG(""); 
 	if (is_at_end == 0) {
+		SWVARFS_E_DEBUG(""); 
 		tartype = ahs_get_tar_typeflag(swvarfs->ahsM);
 		if (tartype < 0) {
 			fprintf(stderr, "%s: unrecognized file type [%d] for file: %s\n",
 				swlib_utilname_get(), (int)tartype, name?name:"<null>");
 			set_vcwd(swvarfs->vcwdM, name?name:"");
 		} else if (tartype == DIRTYPE) {
+			SWVARFS_E_DEBUG(""); 
 			set_vcwd(swvarfs->vcwdM, name?name:"");
 		} else {
 			/*
 			* strip off last component.
 			*/
-			char *s;
+			SWVARFS_E_DEBUG(""); 
 			set_vcwd(swvarfs->vcwdM, name?name:"");
 			if ((s=strrchr(strob_str(swvarfs->vcwdM), '/'))) {
 				*s = '\0';
 			}
 		}
+		SWVARFS_E_DEBUG(""); 
 	} else {
 		/*
 		* end of archive.
