@@ -362,7 +362,6 @@ create_version_id(char * version_id_string)
 	if (*(ver_id->idM) == SWVERID_VERID_REVISION) {
 		/*
 		 * all rel_ops supported except '='
-		fprintf(stderr, "JL [%s] %d\n", ver_id->rel_opM, ver_id->rel_op_codeM);
 		 */
 		if (ver_id->rel_op_codeM == SWVERID_CMP_EQ2) {
 			fprintf(stderr, "%s: error: invalid use of '=' for revision specs\n",
@@ -641,8 +640,15 @@ parse_swspec_string (SWVERID * swverid)
 	 * which are dot separated tags
 	 */
 	tags = strob_strtok(tmp, source, ",");
+
+	if (tags == NULL)
+		return -1;
+
 	tag = strob_strtok(tag_tmp2, tags, ".");
 
+	if (tag == NULL)
+		return -1;
+
 	if (strpbrk(tag, SWVERID_RELOP_NEQ SWVERID_RELOP_LT
 			SWVERID_RELOP_LTE SWVERID_RELOP_EQ2
 			SWVERID_RELOP_EQ SWVERID_RELOP_GTE
@@ -699,15 +705,15 @@ classify_namespace (char * object_kw)
 	if (!object_kw)  return SWVERID_NS_NA;
 	if (!strncmp("fileset", object_kw, 7)) {
 		return SWVERID_NS_MID;
-	} else if (!strncmp("controlfile", object_kw, 11)) {
+	} else if (!strncmp(SW_A_control_file, object_kw, 11)) {
 		return SWVERID_NS_MID;
-	} else if (!strncmp("subproduct", object_kw, 10)) {
+	} else if (!strncmp(SW_A_subproduct, object_kw, 10)) {
 		return SWVERID_NS_MID;
-	} else if (!strncmp("product", object_kw, 7)) {
+	} else if (!strncmp(SW_A_product, object_kw, 7)) {
 		return SWVERID_NS_TOP;
-	} else if (!strncmp("bundle", object_kw, 6)) {
+	} else if (!strncmp(SW_A_bundle, object_kw, 6)) {
 		return SWVERID_NS_TOP;
-	} else if (!strncmp("file", object_kw, 4)) {
+	} else if (!strncmp(SW_A_file, object_kw, 4)) {
 		return SWVERID_NS_LOW;
 	} else {
 		return SWVERID_NS_NA;
@@ -765,20 +771,26 @@ swverid_copy(SWVERID * src)
 
 	dst = swverid_open(NULL, NULL);
 
-	if (src->object_nameM)
+	if (src->object_nameM) {
+		if (dst->object_nameM) free(dst->object_nameM);
 		dst->object_nameM = strdup(src->object_nameM);
-	else
+	} else {
 		dst->object_nameM = NULL;
+	}
 
-	if (src->source_copyM)
+	if (src->source_copyM) {
+		if (dst->source_copyM) free(dst->source_copyM);
 		dst->source_copyM = strdup(src->source_copyM);
-	else
+	} else {
 		dst->source_copyM = NULL;
+	}
 
-	if (src->catalogM) 
+	if (src->catalogM) {
+		if (dst->catalogM) free(dst->catalogM);
 		dst->catalogM = strdup(src->catalogM);
-	else
+	} else {
 		dst->catalogM = NULL;
+	}
 
 	dst->use_path_compareM = src->use_path_compareM;
 	dst->namespaceM = src->namespaceM;
@@ -1394,11 +1406,22 @@ swverid_debug_print(SWVERID * swverid, STROB * buf)
 }
 
 char *
-swverid_print(SWVERID * swverid, STROB * buf)
+swverid_print(SWVERID * swverid, STROB * fp_buf)
 {
 	char * ret;
+	STROB * buf;
+	static STROB * nb;
 	SWVERID * next;
 
+	if (fp_buf) {
+		buf = fp_buf;
+	} else {
+		if (nb == NULL) {
+			nb = strob_open(32);
+		}
+		buf = nb;		
+	}
+
 	strob_strcpy(buf, "");
 	swverid_i_print(swverid, buf);
 	next = swverid;	
@@ -1411,31 +1434,27 @@ swverid_print(SWVERID * swverid, STROB * buf)
 }
 
 char *
-swverid_show_object_debug(SWVERID * swverid, STROB * buf, char * prefix)
+swverid_show_object_debug(SWVERID * swverid, STROB * fp_buf, char * prefix)
 {
 	SWVERID * xx;
+	static STROB * buf;
 	STROB * buf1;
 	char * tag;
 	int in;
 
+	if (fp_buf == NULL) {
+		if (buf == NULL)
+			buf = strob_open(32);
+	} else {
+		buf = fp_buf;
+	}
+
 	xx = swverid;
 	buf1 = strob_open(10);
 
 	strob_strcpy(buf, "");
 	swverid_print(swverid, buf1);
 
-/*	
-	char * object_nameM;
-	char * source_copyM;
-	char * catalogM;
-	int use_path_compareM;
-	CPLOB * taglistM;
-	int namespaceM;	
-	Swverid_Cmp_Code comparison_codeM;
-	struct VER_ID * ver_id_listM;	
-	SWUTS * swutsM;		
-*/
-
 	strob_sprintf(buf, 1, "%s%p (SWVERID*)\n", prefix,  (void*)xx);
 	strob_sprintf(buf, 1, "%s%p swverid_print() = [%s]\n", prefix, (void*)xx, swverid_print(swverid, buf1));
 	strob_sprintf(buf, 1, "%s%p->object_nameM = [%s]\n", prefix, (void*)xx,  xx->object_nameM);
@@ -1466,6 +1485,7 @@ swverid_show_object_debug(SWVERID * swverid, STROB * buf, char * prefix)
 			swverid->swutsM->versionM,
 			swverid->swutsM->releaseM,
 			swverid->swutsM->machineM);
+
 	return strob_str(buf);
 }
 
