@@ -81,7 +81,18 @@ is_name(char * s)
 	return 1;
 }
 
-static
+void
+topsf_add_excludes(TOPSF * topsf, STROB * buf)
+{
+	char * s;
+	int i;
+	i = 0;
+	if (topsf->exclude_listM == NULL) return;
+	while ((s=strar_get(topsf->exclude_listM, i++)) != NULL) {
+		strob_sprintf(buf, STROB_DO_APPEND, SW_A_exclude " %s\n", s);
+	}
+}
+
 int
 parse_slack_desc(STROB * desc, STROB * title, char * src)
 {
@@ -124,14 +135,17 @@ parse_slack_desc(STROB * desc, STROB * title, char * src)
 			;
 		}
 	}
+
+	strob_strcpy(tmp, strob_str(desc));
+	swlib_expand_escapes(NULL, NULL, strob_str(tmp), desc);
+
 	strob_close(tmp);
 	uxfio_close(fd);
 	return 0;
 }
 
-static
 int
-parse_slack_pkg_name(char * pkgfilename, STRAR * st)
+topsf_parse_slack_pkg_name(char * pkgfilename, STRAR * st)
 {
 	char * n;
 	char * s;
@@ -140,9 +154,12 @@ parse_slack_pkg_name(char * pkgfilename, STRAR * st)
 	char * version = NULL;
 	char * arch = NULL;
 	char * build_num = NULL;
+	int ret;
 
 	/* example: gzip-1.3.12-i486-1.tgz */
 
+	/* fprintf(stderr, "JL you're a SLACKER: %s:%s at line %d\n", __FILE__, __FUNCTION__, __LINE__); */
+	ret = 0;
 	tmp = strob_open(32);
 	strob_strcpy(tmp, pkgfilename);
 
@@ -151,6 +168,12 @@ parse_slack_pkg_name(char * pkgfilename, STRAR * st)
 	if (!s) return -1;
 	*s = '\0';   /* squash the ``.tgz'' */
 
+	s = strrchr(n, ':');
+	if (s) {
+		s++;
+		n = s;
+	}
+
 	s = strrchr(n, '/');
 	if (s) {
 		s++;
@@ -177,13 +200,17 @@ parse_slack_pkg_name(char * pkgfilename, STRAR * st)
 
 	name = n;
 
+	if (strpbrk(name, "/:")) {
+		/* bad, call this an error */
+		ret = -2;
+	}
 	strar_add(st, name);
 	strar_add(st, version);
 	strar_add(st, arch);
 	strar_add(st, build_num);
 
 	strob_close(tmp);
-	return 0;
+	return ret;
 }
 
 
@@ -370,6 +397,8 @@ plain_source_tarball_write_psf_buf(TOPSF * topsf, STROB * psf)
 	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " PSF\n");
 	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " catalog\n");
 	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " catalog/*\n");
+	
+	topsf_add_excludes(topsf, psf);
 
 	strar_close(name_fields);
 	strob_close(xtmp);
@@ -418,7 +447,7 @@ slack_write_psf_buf(TOPSF * topsf, STROB * psf)
 		return -1;
 	}
 
-	ret = parse_slack_pkg_name(topsf->pkgfilenameM, name_fields);
+	ret = topsf_parse_slack_pkg_name(topsf->pkgfilenameM, name_fields);
 	if (ret < 0) {
 		E_DEBUG("error");
 		return -2;
@@ -565,10 +594,13 @@ slack_write_psf_buf(TOPSF * topsf, STROB * psf)
 	/* Recursive spec after the leading ./ */
 
 	strob_sprintf(psf, STROB_DO_APPEND, SW_A_file " *\n");
-	strob_sprintf(psf, STROB_DO_APPEND, "%s\n", strob_str(vola));
+
+	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " PSF\n");
+	topsf_add_excludes(topsf, psf);
 	strob_sprintf(psf, STROB_DO_APPEND, "#" SW_A_exclude " control\n");
 	strob_sprintf(psf, STROB_DO_APPEND, "#" SW_A_exclude " control/*\n");
-	strob_sprintf(psf, STROB_DO_APPEND, SW_A_exclude " PSF\n");
+
+	strob_sprintf(psf, STROB_DO_APPEND, "%s\n", strob_str(vola));
 
 	strar_close(name_fields);
 	strob_close(xtmp);
@@ -644,7 +676,7 @@ find_link(TOPSF * topsf, char * keyname)
 
 void topsf_set_taru(TOPSF * topsf, TARU * taru) { topsf->taruM = taru; }
 
-TOPSF * topsf_open(char * package_name, int oflags)
+TOPSF * topsf_open(char * filename, int oflags, char * package_name)
 {
 	TOPSF * topsf=(TOPSF*)malloc(sizeof(TOPSF));
 	
@@ -654,7 +686,14 @@ TOPSF * topsf_open(char * package_name, int oflags)
 	E_DEBUG("");
 	topsf->swacfl_=swacfl_open();
 	E_DEBUG("");
-	topsf->fd_ = uinfile_open(package_name, (mode_t)(NULL), &(topsf->format_desc_), oflags); 
+
+	
+	if (strcmp(filename, "-") == 0 && package_name != NULL) {
+		/* fprintf(stderr, "JL you're a SLACKER: %s:%s at line %d\n", __FILE__, __FUNCTION__, __LINE__); */
+		topsf->fd_ = uinfile_open_with_name(filename, (mode_t)(NULL), &(topsf->format_desc_), oflags, package_name); 
+	} else {
+		topsf->fd_ = uinfile_open(filename, (mode_t)(NULL), &(topsf->format_desc_), oflags); 
+	}
 	E_DEBUG("");
 	
 	if (topsf->fd_ < 0) {
@@ -686,13 +725,14 @@ TOPSF * topsf_open(char * package_name, int oflags)
 	time(&(topsf->mtimeM));
 	topsf->debpsfM = NULL;
 	topsf->rpmpsfM = NULL;
-	if (strcmp(package_name, "-") == 0)
+	if (strcmp(filename, "-") == 0)
 		topsf->pkgfilenameM = NULL;
 	else
-		topsf->pkgfilenameM = strdup(package_name);
+		topsf->pkgfilenameM = strdup(filename);
 	topsf->checkdigestnameM = NULL;
 	topsf->ownerM = NULL;
 	topsf->groupM = NULL;
+	topsf->exclude_listM = NULL;
 	return topsf_i_package_open(topsf, oflags);
 }
 
