@@ -366,7 +366,6 @@ shls_apply_socspec() {
 		#	vendor_tag is the value of vendor_tag, nil if no vendor_tag	
 		#
 		candidate=$0
-		# dmsg("JL " candidate)
 		split($0, C, "/");
 		got_match="no"
 		if ( C[1] ~  ep_pat1 || C[2] ~  ep_pat1 ) {
@@ -536,7 +535,11 @@ shls_bashin2() {
 			"")
 				ihnb=$(($ihnb+1))
 				;;
+			\#*)
+				break
+				;;
 			*)
+				echo "$swutilname: warning: unexpected bytes in shls_bashin2" 1>&2
 				break
 				;;
 		esac
@@ -544,7 +547,7 @@ shls_bashin2() {
 	case "$ihnb" in 0|1) ;; *) ihpl="s";; esac
 	case "$ihnb" in
 		0) ;;
-		*) echo "$swutilname: Warning: ${ihnb} NUL block${ihpl} at start of input for task ${ih_taskid}" 1>&2 0</dev/null
+		*) echo "$swutilname: Warning: ${ihnb} NUL octet${ihpl} at start of input for task ${ih_taskid}" 1>&2 0</dev/null
 		;;
 	esac
 
@@ -554,8 +557,14 @@ shls_bashin2() {
 	do
 		ih_line=`dd count=1 bs=1 2>/dev/null`
 		case "$ih_line" in
-			S*) break ;;
-			*) ;;
+			# OLD  S*) break ;;
+			# OLD  *) ;;
+			S) break ;;
+			\#) ;;
+			" ") ;;
+			"") ;;
+			_) ;;
+			*) echo "$swutilname: garbage in shls_bashin2, FATAL" 1>&2; dd of=/dev/null 2>/dev/null; return 37; ;;
 		esac
 	done
 
