@@ -697,6 +697,30 @@ strob_vsprintf(STROB * sb, int do_append, char * format, va_list ap)
 	return ret;
 }
 
+int
+strob_snprintf(STROB * sb, int do_append, size_t len, char * format, ...)
+{
+	int ret;
+	va_list ap;
+	
+	int old_len;
+	char * s;
+
+	if (len <= 0) return len;
+
+	if (do_append)
+		old_len = strob_strlen(sb);
+	else
+		old_len = 0;
+
+	va_start(ap, format);
+	ret = strob_vsprintf(sb, do_append, format, ap); 
+	va_end(ap);
+
+	strob_chr_index(sb, old_len + len - 1, 0 /*i.e. NUL, '\x0' */);
+	return ret;
+}
+
 int
 strob_sprintf(STROB * sb, int do_append, char * format, ...)
 {
