@@ -41,6 +41,20 @@
 /* #include "swevents_array.h" */
 #include "debug_config.h"
 
+
+static
+int
+show_nopen(void)
+{
+	int ret;
+	ret = /*--*/open("/dev/null",O_RDWR);
+	if (ret<0)
+		fprintf(stderr, "fcntl error: %s\n", strerror(errno));
+	close(ret);
+	return ret;
+}
+
+
 static 
 SWPATH_EX *
 swi_get_last_swpath_ex(SWI * swi)
@@ -63,17 +77,19 @@ swi_get_last_swpath_ex(SWI * swi)
 /**
 *  Find a product or fileset by the 'tag'
 */
+static
 SWI_BASE *
-find_object_by_swsel(void * parent, char * swsel)
+find_object_by_swsel(void * parent, char * swsel, char * number, int * p_index)
 {
 	int i;
 	SWI_PACKAGE * package;
 	SWI_BASE * object;
 	char * verid_delim;
 
+	if (p_index) *p_index = -1;
 	package = (SWI_PACKAGE *)(parent);
 	for(i=0; i<SWI_MAX_OBJ; i++) {
-		object = (SWI_BASE*)(&(package->swi_coM[i]->baseM));
+		object = (SWI_BASE*)(&(package->swi_coM[i]->p_baseM));
 		if (object) {
 			/*
 			 * FIXME, crudely handle version ids by ignoring them
@@ -83,7 +99,13 @@ find_object_by_swsel(void * parent, char * swsel)
 				verid_delim = swsel + strlen(swsel);
 			}
 			
-			if (strncmp(swsel, object->tagM, (size_t)(verid_delim - swsel)) == 0) {
+			if (strncmp(swsel, object->b_tagM, (size_t)(verid_delim - swsel)) == 0) {
+				if (number && object->numberM) {
+					if (strcmp(number, object->numberM) != 0) {
+						continue;
+					} 
+				}
+				if (p_index) *p_index = i;
 				return object;
 			}
 		} else {
@@ -175,8 +197,8 @@ swi_xfile_add_control_script(SWI * swi, SWI_XFILE * xfile, char * name,
 	sc = swi_control_script_create();
 	s = swi_xfile_construct_file_member(xfile, name, fd,
 					swi->xformatM->swvarfsM);
-	sc->baseM.tagM = strdup(tag);
-	swi_com_check_clean_relative_path(sc->baseM.tagM);
+	sc->baseM.b_tagM = strdup(tag);
+	swi_com_check_clean_relative_path(sc->baseM.b_tagM);
 	sc->afileM = s;
 	swi_add_script(xfile->swi_scM, sc);
 	return;
@@ -233,6 +255,7 @@ swi_package_delete(SWI_PACKAGE * s)
 	}
 	free((void*)(cplob_release(s->exlistM)));	
 
+	/* if (s->current_xfileM) swi_xfile_delete(s->current_xfileM); */
 	if (s->catalog_entryM != (char*)NULL) free(s->catalog_entryM);
 	if (s->target_pathM != (char*)NULL) free(s->target_pathM);
 	if (s->target_hostM != (char*)NULL) free(s->target_hostM);
@@ -242,6 +265,7 @@ swi_package_delete(SWI_PACKAGE * s)
 	if (s->dfilesM) swi_xfile_delete(s->dfilesM);
 	if (s->baseM.global_headerM) swheader_close(s->baseM.global_headerM);
 	if (s->swdef_fdM > 0) uxfio_close(s->swdef_fdM);
+	if (s->installer_sigM) free(s->installer_sigM);
 	s->swdef_fdM = -1;
 	swi_scripts_delete(s->swi_scM);
 	for(i=0; i<SWI_MAX_OBJ; i++) {
@@ -287,10 +311,30 @@ swi_package_create(void)
 void
 swi_delete(SWI * s)
 {
-	close(s->nullfdM);
+	E_DEBUG2("Entering swi_delete LOWEST FD=%d", show_nopen());
+	swbis_devnull_close(s->nullfdM);
 	swi_package_delete(s->swi_pkgM);
 	swicol_delete(s->swicolM);
-	if (s->distdataM) swi_distdata_delete(s->distdataM);
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
+	if (s->distdataM){
+		 swi_distdata_delete(s->distdataM);
+	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
+	if (s->xformat_close_on_deleteM) {
+		E_DEBUG("calling xformat_close");
+		xformat_close(s->xformatM);
+	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
+	if (s->swvarfs_close_on_deleteM) {
+		E_DEBUG("calling swvarfs_close");
+		swvarfs_close(s->swvarfsM);
+	}
+	E_DEBUG2("LOWEST FD=%d", show_nopen());
+	if (s->uinformat_close_on_deleteM) {
+		E_DEBUG("calling uinfile_close");
+		uinfile_close(s->uinformatM);
+	}
+	E_DEBUG2("LEAVING swi_delete: LOWEST FD=%d", show_nopen());
 	free(s);
 }
 
@@ -306,7 +350,7 @@ SWI * swi_create(void)
 	swi->uinformatM = NULL;
 	swi->swpathM = NULL;
 	swi->swi_pkgM = swi_package_create();
-	swi->nullfdM = open("/dev/null", O_RDWR);
+	swi->nullfdM = swbis_devnull_open("/dev/null", O_RDWR, 0);
 	memset(swi->tarbufM, '\0', sizeof(swi->tarbufM));
 	swi->distdataM = NULL;
 	swi->opt_alt_catalog_rootM = 0;
@@ -316,6 +360,9 @@ SWI * swi_create(void)
 	swi->debug_eventsM = 0;
 	swi->target_version_idM = swverid_open("HOST", NULL);
 	swi->does_have_payloadM = -1; /* unset */
+	swi->xformat_close_on_deleteM = 0;
+	swi->swvarfs_close_on_deleteM = 0;
+	swi->uinformat_close_on_deleteM = 0;
 	return swi;
 }
 
@@ -323,13 +370,13 @@ void
 swi_product_delete(SWI_PRODUCT * s)
 {
 	int i;
-	if (s->baseM.tagM) free(s->baseM.tagM);
+	if (s->p_baseM.b_tagM) free(s->p_baseM.b_tagM);
 	if (s->package_pathM) free(s->package_pathM);
 	if (s->control_dirM) free(s->control_dirM);
 
 	if (s->filesetsM) strar_close(s->filesetsM);
 
-	if (s->pfilesM) swi_xfile_delete(s->pfilesM);
+	if (s->xfileM) swi_xfile_delete(s->xfileM);
 	
 	for(i=0; i<SWI_MAX_OBJ; i++) {
 		if (s->swi_coM[i])
@@ -360,7 +407,7 @@ swi_product_create(SWHEADER * global_index_header, SWPATH_EX * current)
 	s->control_dirM = (char*)NULL;
 	s->filesetsM = (STRAR*)NULL;
 	swiInitListOfObjects((void**)(s->swi_coM));
-	s->pfilesM = NULL;
+	s->xfileM = NULL;
 	s->is_selectedM = 1;
 	s->is_compatibleM = 0;
 	return s;
@@ -473,8 +520,8 @@ swi_expand_shared_file_control_sripts(SWI * swi, SWI_XFILE * xfile)
 				SWLIB_ASSERT(afile != NULL);
 					
 				new_script = swi_control_script_create();
-				new_script->baseM.tagM = strdup(tag_value);
-				swi_com_check_clean_relative_path(new_script->baseM.tagM);
+				new_script->baseM.b_tagM = strdup(tag_value);
+				swi_com_check_clean_relative_path(new_script->baseM.b_tagM);
 				new_script->afileM = afile;
 				(afile->refcountM)++;
 				swi_add_script(xfile->swi_scM, new_script);
@@ -695,7 +742,7 @@ swi_handle_control_transition(SWI * swi)
 			 */
 			pfiles = swi_xfile_create(SWI_XFILE_TYPE_FILESET, SWI_INDEX_HEADER(swi), current);
 			pfiles->package_pathM = strdup(current->pkgpathname);
-			package->current_productM->pfilesM = pfiles;
+			package->current_productM->xfileM = pfiles;
 			package->current_xfileM = pfiles;
 		} else if (tret < 0) {
 			SWI_internal_error();
@@ -745,7 +792,7 @@ swi_handle_control_transition(SWI * swi)
 			package->current_xfileM = NULL;
 		}
 		pfiles->package_pathM = strdup(current->pkgpathname);
-		package->current_productM->pfilesM = pfiles;
+		package->current_productM->xfileM = pfiles;
 		package->current_xfileM = pfiles;
 		E_DEBUG2("   Current Product [%p]",
 				(void*)(package->current_productM));
@@ -945,7 +992,7 @@ swi_decode_catalog(SWI * swi)
 
 	/* Now fix-up the control scripts that share
 	   a common control file.  */
-	ret = swi_expand_shared_file_control_sripts(swi, swi->swi_pkgM->swi_coM[0]->pfilesM);
+	ret = swi_expand_shared_file_control_sripts(swi, swi->swi_pkgM->swi_coM[0]->xfileM);
 	swi_com_assert_value(ret == 0, __FILE__, __LINE__);
 
 	E_DEBUG("");
@@ -982,7 +1029,7 @@ int
 swi_product_has_control_file(SWI_PRODUCT * swi_prod, char * control_file_name)
 {
 	int ret;
-	ret = swi_xfile_has_posix_control_file(swi_prod->pfilesM,
+	ret = swi_xfile_has_posix_control_file(swi_prod->xfileM,
 		control_file_name);
 	return ret;
 }
@@ -991,7 +1038,7 @@ SWI_CONTROL_SCRIPT *
 swi_product_get_control_script_by_tag(SWI_PRODUCT * prod, char * tag)
 {
 	SWI_CONTROL_SCRIPT * ret; 
-	ret = swi_xfile_get_control_script_by_tag(prod->pfilesM, tag);
+	ret = swi_xfile_get_control_script_by_tag(prod->xfileM, tag);
 	return ret;
 }
 
@@ -1041,10 +1088,10 @@ swi_recurse_swi_tree(SWI * swi, char * sw_selections, int (*payload)(void*, void
 }
 
 SWI_PRODUCT *
-swi_find_product_by_swsel(SWI_PACKAGE * parent, char * swsel)
+swi_find_product_by_swsel(SWI_PACKAGE * parent, char * swsel, char * number, int * p_index)
 {
 	SWI_BASE * base;
-	base = find_object_by_swsel(parent, swsel);
+	base = find_object_by_swsel(parent, swsel, number, p_index);
 	if (!base) return (SWI_PRODUCT*)base;
 	swi_base_assert(base);
 	SWLIB_ASSERT(base->type_idM == SWI_I_TYPE_PROD); 	
@@ -1052,14 +1099,17 @@ swi_find_product_by_swsel(SWI_PACKAGE * parent, char * swsel)
 }
 
 SWI_XFILE *
-swi_find_fileset_by_swsel(SWI_PRODUCT *  parent, char * swsel)
+swi_find_fileset_by_swsel(SWI_PRODUCT *  parent, char * swsel, int * p_index)
 {
 	SWI_BASE * base;
-	base = find_object_by_swsel(parent, swsel);
+	base = find_object_by_swsel(parent, swsel, NULL, p_index);
 	if (!base) return (SWI_XFILE*)base;
 	swi_base_assert(base);
 	SWLIB_ASSERT(base->type_idM == SWI_I_TYPE_XFILE); 	
-	SWLIB_ASSERT(((SWI_XFILE*)base)->typeM == SWI_XFILE_TYPE_FILESET); 	
+	SWLIB_ASSERT(
+		((SWI_XFILE*)base)->typeM == SWI_XFILE_TYPE_FILESET ||
+		((SWI_XFILE*)base)->typeM == SWI_XFILE_TYPE_PFILES 
+		); 	
 	return (SWI_XFILE*)(base);
 }
 
@@ -1258,7 +1308,7 @@ swi_do_decode(SWI * swi, SWLOG * swutil,
 }
 
 int
-swi_find_compatible_product(SWI * swi, SWUTS * target_uts, int *pn_products)
+swi_find_compatible_product(SWI * swi, SWUTS * target_uts, int *pn_products, int * p_swi_index)
 {
 	SWI_PRODUCT * product;
 	SWHEADER * global_index;
@@ -1282,7 +1332,7 @@ swi_find_compatible_product(SWI * swi, SWUTS * target_uts, int *pn_products)
 		   which is the start of the product definition */
 		if (pn_products) ++(*pn_products);
 		swheader_reset(global_index);
-		swheader_set_current_offset(global_index, product->baseM.header_indexM);
+		swheader_set_current_offset(global_index, product->p_baseM.header_indexM);
 
 		product_uts = swuts_create();
 		swi_get_uts_attributes_from_current(swi, product_uts, global_index);
@@ -1290,6 +1340,7 @@ swi_find_compatible_product(SWI * swi, SWUTS * target_uts, int *pn_products)
 		if (ret == 0) {
 			n_matches++;
 			product->is_compatibleM = SW_TRUE;
+			if (p_swi_index) *p_swi_index = product_number-1;
 		}
 		swuts_delete(product_uts);
 	}
@@ -1363,7 +1414,7 @@ swi_make_fileset_file_list(SWI * swi, int product_num, int fileset_num, char * l
 	}
 
 	global_index = swi_get_global_index_header(swi);
-	swheader_set_current_offset(global_index, product->baseM.header_indexM);
+	swheader_set_current_offset(global_index, product->p_baseM.header_indexM);
 	directory = swheader_get_single_attribute_value(global_index, SW_A_directory);
 	directory = swlib_attribute_check_default(SW_A_product, SW_A_directory, directory);
 	
@@ -1456,7 +1507,6 @@ swi_make_file_list(SWI * swi, SWUTS * swuts, char * location, int disallow_incom
 	char * is_volatile;
 	int tar_filetype;
 	int n_products;
-	STROB * tmp;
 	STROB * relocated_path;
 	SWI_FILELIST * flist;
 	SWI_PRODUCT * product;
@@ -1466,7 +1516,6 @@ swi_make_file_list(SWI * swi, SWUTS * swuts, char * location, int disallow_incom
 
 	E_DEBUG("");
 	flist = swi_fl_create();
-	tmp = strob_open(12);
 	relocated_path = strob_open(12);
 
 	if (swi == NULL) {
@@ -1475,8 +1524,7 @@ swi_make_file_list(SWI * swi, SWUTS * swuts, char * location, int disallow_incom
 	}
 
 	E_DEBUG("");
-	/*  swi_find_compatible_product() marks product->is_compatibleM = SW_TRUE */
-	n_compat = swi_find_compatible_product(swi, swuts, &n_products);
+	n_compat = swi_find_compatible_product(swi, swuts, &n_products, NULL);
 
 	/* Now check for non-root "/" target and for packages that only have
 	   one product, claim it is compatible */
