@@ -85,12 +85,11 @@ typedef struct {		/* the Bundle structure			*/
 } SWI_BUNDLE;
 
 typedef struct {		/* the Product structure		*/
-	SWI_BASE baseM;
-
+	SWI_BASE p_baseM;
+	SWI_XFILE * xfileM;	/* Product pfiles			*/
 	SWI_XFILE * swi_coM[SWI_MAX_OBJ];  /* list of filesets  	*/
 	char * package_pathM;	/* The package path for this object	*/
 	char * control_dirM;	/* the product control dir		*/
-	SWI_XFILE * pfilesM;
 	STRAR * filesetsM;	/* list of installed filesets		*/
 	int is_selectedM;	/* is selected				*/
 	int is_compatibleM;	/* is compatible			*/
@@ -126,7 +125,6 @@ typedef struct {			/* The global package structure	*/
 	char * installed_catalog_ownerM; /* owner of installed_software_catalog */
 	char * installed_catalog_groupM; /* group of installed_software_catalog */
 	mode_t installed_catalog_modeM;	 /* mode of installed_software_catalog */
-
 } SWI_PACKAGE;
 
 typedef struct {
@@ -150,7 +148,6 @@ typedef struct {	/* The Global package control structure */
 	int verboseM;		/* Controls debugging messages 		*/
 	char tarbufM[1024];
 	SWI_DISTDATA * distdataM;
-	/* char * sh_dash_sM; */ /* the shell command			*/
 	int opt_alt_catalog_rootM; /* -r option				*/
 	char * exported_catalog_prefixM; /* i.e.  <path>/catalog/ from  */
 				/* the distribution 			*/
@@ -161,6 +158,9 @@ typedef struct {	/* The Global package control structure */
 	int debug_eventsM;      /*                                      */
 	SWVERID * target_version_idM;  /* Version Id of the target host        */
 	int does_have_payloadM; /* -1, 0 or 1, is there a payload? */
+	int xformat_close_on_deleteM;
+	int swvarfs_close_on_deleteM;
+	int uinformat_close_on_deleteM;
 } SWI;
 
 void swi_base_set_is_active(SWI_BASE * base, int n);
@@ -205,12 +205,12 @@ char * swi_base_dump_string_s(SWI_BASE * xx, char * prefix);
 int swi_examine_signature_blocks(SWI_XFILE * dfiles, int * sig_block_start, int * sig_block_end);
 SWI_PRODUCT * swi_package_get_product(SWI_PACKAGE * swi_pkg, int arr_index);
 SWI_XFILE * swi_package_get_fileset(SWI_PRODUCT * swi_prod, int arr_index);
-int swi_product_has_control_file(SWI_PRODUCT * swi_prod, char * control_file_name);
 SWI_XFILE * swi_product_get_fileset(SWI_PRODUCT * swi_prod, int arr_index);
+int swi_product_has_control_file(SWI_PRODUCT * swi_prod, char * control_file_name);
 SWI_CONTROL_SCRIPT * swi_product_get_control_script_by_tag(SWI_PRODUCT * prod, char * tag);
 int swi_recurse_swi_tree(SWI * swi, char * sw_selections, int (*payload)(void*, void * uptr), void * uptr);
-SWI_PRODUCT * swi_find_product_by_swsel(SWI_PACKAGE * parent, char * swsel);
-SWI_XFILE * swi_find_fileset_by_swsel(SWI_PRODUCT * parent, char * swsel);
+SWI_PRODUCT * swi_find_product_by_swsel(SWI_PACKAGE * parent, char * swsel, char * number, int *);
+SWI_XFILE * swi_find_fileset_by_swsel(SWI_PRODUCT * parent, char * swsel, int *);
 void swi_get_uts_attributes_from_current(SWI * swi, SWUTS * uts, SWHEADER * swheader);
 void swi_set_utility_id(SWI * swi, int swc_u_id);
 
@@ -228,7 +228,7 @@ int swi_do_decode(SWI * swi, SWLOG * swutil,
 	struct sw_logspec * g_logspec,
 	int uinfile_open_flags);
 
-int swi_find_compatible_product(SWI * swi, SWUTS * target_uts, int * num_products);
+int swi_find_compatible_product(SWI * swi, SWUTS * target_uts, int * num_products, int * swi_index);
 SWHEADER * swi_get_global_index_header(SWI * swi);
 SWHEADER * swi_get_fileset_info_header(SWI * swi, int product_index, int fileset_index);
 SWI_FILELIST * swi_make_fileset_file_list(SWI * swi, int product_num, int fileset_num, char * location);
