@@ -127,7 +127,7 @@ swi_xfile_script_dump_string_s(SWI_CONTROL_SCRIPT * xx, char * prefix)
 
 	strob_sprintf(buf, 0, "%s%p (SWI_CONTROL_SCRIPT*)\n", prefix,  (void*)xx);
 	strob_sprintf(buf, 1, "%s%p->baseM.is_activeM  = [%d]\n", prefix, (void*)xx,  xx->baseM.is_activeM);
-	strob_sprintf(buf, 1, "%s%p->tagM        = [%s]\n", prefix, (void*)xx,  xx->baseM.tagM);
+	strob_sprintf(buf, 1, "%s%p->b_tagM        = [%s]\n", prefix, (void*)xx,  xx->baseM.b_tagM);
 	strob_sprintf(buf, 1, "%s%p->resultM     = [%d]\n", prefix, (void*)xx,  xx->resultM);
 	strob_sprintf(buf, 1, "%s%p->afileM      = [%p]\n", prefix, (void*)xx,  xx->afileM);
 	tmps = swi_xfile_member_dump_string_s(xx->afileM, prefix);
@@ -154,7 +154,7 @@ swi_base_dump_string_s(SWI_BASE * xx, char * prefix)
 	strob_sprintf(buf, 1, "%s%p->id_startM       = [%d]\n", prefix, (void*)xx,     (int)(xx->id_startM));
 	strob_sprintf(buf, 1, "%s%p->type_idM        = [%d]\n", prefix, (void*)xx,     (int)(xx->type_idM));
 	strob_sprintf(buf, 1, "%s%p->is_activeM      = [%d]\n", prefix, (void*)xx,     xx->is_activeM);
-	strob_sprintf(buf, 1, "%s%p->tagM            = [%s]\n", prefix, (void*)xx,     xx->tagM);
+	strob_sprintf(buf, 1, "%s%p->b_tagM          = [%s]\n", prefix, (void*)xx,     xx->b_tagM);
 	strob_sprintf(buf, 1, "%s%p->create_timeM    = [%lu]\n", prefix, (void*)xx,    (unsigned long)(xx->create_timeM));
 	strob_sprintf(buf, 1, "%s%p->mod_timeM       = [%lu]\n", prefix, (void*)xx,    (unsigned long)(xx->mod_timeM));
 	strob_sprintf(buf, 1, "%s%p->header_indexM   = [%d]\n", prefix, (void*)xx,     xx->header_indexM);
@@ -179,7 +179,6 @@ swi_base_dump_string_s(SWI_BASE * xx, char * prefix)
 		int memfd;
 		int len;
 		char * image;
-/* JL fprintf(stderr, "JL [%d]\n", xx->header_indexM); */
 		current_offset = swheader_get_current_offset(xx->global_headerM);
 		swheader_set_current_offset(xx->global_headerM, xx->header_indexM);
 
@@ -265,7 +264,7 @@ swi_product_dump_string_s(SWI_PRODUCT * xx, char * prefix)
 	* char * control_dirM;
 	* char * tagM;		
 	* SWI_SCRIPTS * swi_scM;	
-	* SWI_XFILE * pfilesM;
+	* SWI_XFILE * xfileM;
 	* SWI_XFILE * swi_coM[SWI_MAX_OBJ]; 
 	*/
 	int i = 0;
@@ -279,20 +278,20 @@ swi_product_dump_string_s(SWI_PRODUCT * xx, char * prefix)
 
 	strob_sprintf(buf, 0, "%s%p (SWI_PRODUCT*)\n", prefix,  (void*)xx);
 
-	strob_sprintf(buf, 1, "%s%p->baseM  = [%p]\n", prefix, (void*)xx,  (void*)(&(xx->baseM)));
-	strob_sprintf(tmp, 0, "%s%p->baseM:", prefix, (void*)(&(xx->baseM)));
-	strob_sprintf(buf, 1, "%s", swi_base_dump_string_s(&(xx->baseM), strob_str(tmp)));
+	strob_sprintf(buf, 1, "%s%p->p_baseM  = [%p]\n", prefix, (void*)xx,  (void*)(&(xx->p_baseM)));
+	strob_sprintf(tmp, 0, "%s%p->baseM:", prefix, (void*)(&(xx->p_baseM)));
+	strob_sprintf(buf, 1, "%s", swi_base_dump_string_s(&(xx->p_baseM), strob_str(tmp)));
 
 	strob_sprintf(buf, 1, "%s%p->package_pathM   = [%s]\n", prefix, (void*)xx,  xx->package_pathM);
 	strob_sprintf(buf, 1, "%s%p->control_dirM    = [%s]\n", prefix, (void*)xx,  xx->control_dirM);
-	strob_sprintf(buf, 1, "%s%p->pfilesM         = [%p]\n", prefix, (void*)xx,  (void*)(xx->pfilesM));
+	strob_sprintf(buf, 1, "%s%p->xfileM         = [%p]\n", prefix, (void*)xx,  (void*)(xx->xfileM));
 	
-	strob_sprintf(buf, 1, "%s%p->pfilesM         = [%p] (BEGIN PFILES)\n",
-						prefix, (void*)xx,  (void*)(xx->pfilesM));
-	tmp_s = swi_xfile_dump_string_s(xx->pfilesM, prefix);
+	strob_sprintf(buf, 1, "%s%p->xfileM         = [%p] (BEGIN PFILES)\n",
+						prefix, (void*)xx,  (void*)(xx->xfileM));
+	tmp_s = swi_xfile_dump_string_s(xx->xfileM, prefix);
 	strob_sprintf(buf, 1, "%s", tmp_s);
-	strob_sprintf(buf, 1, "%s%p->pfilesM         = [%p] (END PFILES)\n",
-						prefix, (void*)xx,  (void*)(xx->pfilesM));
+	strob_sprintf(buf, 1, "%s%p->xfileM         = [%p] (END PFILES)\n",
+						prefix, (void*)xx,  (void*)(xx->xfileM));
 	
 	strob_sprintf(buf, 1, "%s%p->filesetsM  = [%p]\n",
 					prefix, (void*)xx,  (void*)(xx->filesetsM));
