@@ -37,6 +37,7 @@
 
 typedef struct {		/* The fileset/pfiles/dfiles  object	*/
 	SWI_BASE baseM;
+	void * xfileM;		/* (SWI_XFILE*) points to this		*/
 	char * package_pathM;	/* The package path for this object	*/
 	char * control_dirM;	/* the fileset control dir		*/
 	int typeM;		/* type: 0 fileset; 1 dfiles; 2 pfiles  */
@@ -64,6 +65,7 @@ SWI_FILE_MEMBER * swi_xfile_get_control_file_by_path(SWI_XFILE * xfile, char * p
 SWI_CONTROL_SCRIPT * swi_xfile_get_control_script_by_path(SWI_XFILE * xfile, char * path);
 
 SWI_CONTROL_SCRIPT * swi_xfile_get_control_script_by_tag(SWI_XFILE * xfile, char * tag);
+SWI_CONTROL_SCRIPT * swi_xfile_get_control_script_by_id(SWI_XFILE * xfile, int id);
 
 SWI_FILE_MEMBER * swi_xfile_contstruct_file_member(SWI_XFILE * xfile, char * name, int fd);
 
