@@ -224,7 +224,7 @@ ahsStaticDeleteFilehdr(struct new_cpio_header * file_hdr)
 	ahsStaticSetTarFilename(file_hdr, NULL);
 	ahsStaticSetTarUsername(file_hdr, NULL);
 	ahsStaticSetTarGroupname(file_hdr, NULL);
-	swbis_free(file_hdr);
+	taru_free_header(file_hdr);
 }
 
 void
