@@ -372,6 +372,7 @@ taru_write_archive_member(TARU * taru, char *filename,
 	struct new_cpio_header * file_hdr = ahsStaticCreateFilehdr();
 	int fd = -1;
 	intmax_t ret = 0;
+	int nullfd = -1;
 
 	*linkname='\0';;
 	if (statbuf) {
@@ -406,7 +407,8 @@ taru_write_archive_member(TARU * taru, char *filename,
 			if (file_hdr->c_filesize > 0) {
 				fd = open(filename, O_RDONLY, 0);
 			} else {
-				fd = open("/dev/null", O_RDONLY, 0);
+				fd = swbis_devnull_open("/dev/null", O_RDWR, 0);
+				nullfd = fd;
 			}
 			if (fd < 0) {
 				fprintf(stderr, "swbis: %s: %s\n",
@@ -424,9 +426,13 @@ taru_write_archive_member(TARU * taru, char *filename,
 	ret = tarui_write_archive_member_(taru, statbuf, file_hdr,
 					linkrec, defer, porinode, ofd,
 					fd, format, tarheaderflags);
-	if (fd >= 0) {
+
+	if (fd >= 0 && nullfd != fd) {
 		close(fd);
+	} else {
+		swbis_devnull_close(fd);
 	}
+
 	ahsStaticDeleteFilehdr(file_hdr);
 	return ret;
 }
@@ -464,7 +470,7 @@ taru_write_archive_file_data(TARU * taru,
 	int nullfd;
 
 	if (!taru) 
-		nullfd = swlib_open_nullfd();
+		nullfd = swbis_devnull_open("/dev/null", O_RDWR, 0);
 	else
 		nullfd = taru->nullfdM;
 
@@ -481,7 +487,7 @@ taru_write_archive_file_data(TARU * taru,
 			  	archive_format_out,
 				-1/* adjuct_fd */,
 				file_hdr0->digsM);
-	if (!taru) swlib_close_nullfd(nullfd);	
+	if (!taru) swbis_devnull_close(nullfd);
 	return ret;
 }
 
@@ -663,7 +669,6 @@ taru_write_archive_member_header(TARU * taru,
 					1,
 					&nfound);
 				if (nfound || file_hdr->c_is_tar_lnktype == 1) {
-					/* fprintf(stderr, "JLDX here here [%s] \n", link_record_buf->path_); */
 					/* hllist_show_to_file(linkrecord, stderr); */
 					E_DEBUG("");
 					
@@ -709,10 +714,6 @@ taru_write_archive_member_header(TARU * taru,
 			E_DEBUG("");
 			if (linkrecord_active) {
 				if (file_hdr->c_nlink > 1) {
-					/*
-					fprintf(stderr, "JLDX hllist_add_record %s %d :%s\n",
-						 __FILE__, __LINE__, ahsStaticGetTarFilename(file_hdr));
-					*/
 					E_DEBUG("");
 					hllist_add_record(linkrecord,
 					ahsStaticGetTarFilename(file_hdr), 
@@ -802,9 +803,6 @@ taru_write_archive_member_header(TARU * taru,
 			E_DEBUG("");
 			if (linkrecord_active) {
 				if (file_hdr->c_nlink > 1) {
-					/*
-					fprintf(stderr, "JLDX hllist_add_record %s %d\n", __FILE__, __LINE__);
-					*/
 					hllist_add_record(linkrecord,
 						ahsStaticGetTarFilename(file_hdr), 
 						(dev_t)makedev(file_hdr->c_dev_maj,\
@@ -1150,6 +1148,7 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 	int header_length;
 	char intbuf[UINTMAX_STRSIZE_BOUND];
 
+	E_DEBUG("");
 	file_hdr0 = ahsStaticCreateFilehdr();
 	memset(zeros_512, 0x00, 512);
 	hllist = hllist_open();
@@ -1157,16 +1156,20 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 	hllist_disable_find(hllist);
 	retvalx = 0;
 
-	file_hdr0->digsM = digs;	
+	E_DEBUG("");
+	taru_digs_delete(file_hdr0->digsM);
+	file_hdr0->digsM = digs;
 
+	E_DEBUG("");
 	if (ls_fd >= 0) {
 		ls_tmp = strob_open(120);
 	} else {
 		ls_tmp = NULL;
 	}
 
+	E_DEBUG("");
 	while (uxfio_sfa_read(input_fd, buf, 512) == 512) {
-
+		E_DEBUG("");
 		if (memcmp(buf, zeros_512, 512) == 0) {
 			if (uxfio_sfa_read(input_fd, buf, 512) == 512) {
 				if (!memcmp(buf, zeros_512, 512)) {
@@ -1182,6 +1185,7 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 				return -2;
 			}
 		}
+		E_DEBUG("");
 		if (taru_tarheader_check(buf)) {
 			fprintf(stderr, "tarheader error ifd=%d ofd=%d\n", input_fd, out_file_des);
 			ahsStaticDeleteFilehdr(file_hdr0);
@@ -1209,6 +1213,7 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 					buf, (int*)NULL, 0, header_length);
 
 
+		E_DEBUG("");
 		retx = tarui_write_archive_member_
 			(taru, (struct stat*)NULL, file_hdr0,
 				hllist, defer, porinode,
@@ -1231,13 +1236,16 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 		}
 
 
+		E_DEBUG("");
 	}	/* while read 512-byte blocks */
 
+	E_DEBUG("");
 	/* The collection is complete; append the trailer.  */
 	ret = taru_write_archive_trailer(taru,
 				archive_format_out,
 				out_file_des, 512, 0, 0);
 
+	E_DEBUG("");
 	if (ret <= 0) return -10;
 	retvalx += (intmax_t)ret;
 	if (nb) *nb = (intmax_t)retvalx;
@@ -1245,5 +1253,6 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 	file_hdr0->digsM = NULL; /* Prevents deleting passed-in object */
 	ahsStaticDeleteFilehdr(file_hdr0);
 	hllist_close(hllist);
+	E_DEBUG("");
 	return 0;
 }
