@@ -117,6 +117,7 @@ struct old_cpio_header
 */
 
 typedef struct {
+	int do_poisonM;
 	char md5[33];  /* ascii md5 */
 	short int do_md5;
 	char sha1[41];  /* ascii sha1 */
@@ -129,7 +130,9 @@ typedef struct {
 
 #define DIGS_ENABLE_OFF 0
 #define DIGS_ENABLE_ON 1
+#define DIGS_ENABLE_POISON 2
 #define DIGS_ENABLE_CLEAR -1
+#define DIGS_DO_POISON 1
 
 #define TARU_C_ID_UNSET ULONG_MAX
 
