@@ -131,6 +131,82 @@ static void setst (unsigned short bits, char *chars);
 	otherwise.
 	'T' if the file is sticky but not executable.  */
 
+static
+mode_t
+set_perms(char * fps, int set, int * p_error)
+{
+	mode_t ret_mode;
+	char * s;
+	char r_b;
+	char w_b;
+	char x_b;	
+	unsigned int m_r;
+	unsigned int m_w;
+	unsigned int m_x;
+	unsigned int m_s;
+	unsigned int m_t;
+
+	s = fps;
+	/* set == 0   is user */
+	/* set == 1   is group */
+	/* set == 2   is other */
+	
+	if (set == 0) {
+		m_r = S_IRUSR;
+		m_w = S_IWUSR;
+		m_x = S_IXUSR;
+		m_s = S_ISUID;
+		m_t = 0;
+	} else if (set == 1) {
+		m_r = S_IRGRP;
+		m_w = S_IWGRP;
+		m_x = S_IXGRP;
+		m_s = S_ISGID;
+		m_t = 0;
+	} else if (set == 2) {
+		m_r = S_IROTH;
+		m_w = S_IWOTH;
+		m_x = S_IXOTH;
+		m_s = 0;
+		m_t = S_ISVTX;
+	}
+	
+	*p_error = 0;
+	if (*s == '\0') { *p_error = 1; return 0; }
+	r_b = *s;
+	
+	s++; if (*s == '\0') { *p_error = 1; return 0; }
+	w_b = *s;
+	
+	s++; if (*s == '\0') { *p_error = 1; return 0; }
+	x_b = *s;
+
+	ret_mode = 0;
+
+	if (r_b == 'r') {
+		ret_mode |= m_r;
+	}
+
+	if (w_b == 'w') {
+		 ret_mode |= m_w;
+	}
+
+	if (x_b == 'x') {
+		ret_mode |= m_x;
+	}
+
+	if (x_b == 's') {
+		ret_mode |= m_x;
+		ret_mode |= m_s;
+	}
+
+	if (x_b == 't') {
+		ret_mode |= m_x;
+		 ret_mode |= m_t;
+	}
+	return ret_mode;
+}
+
 void
 swlib_filemodestring (mode_t  mode, char * str)
 {
@@ -264,3 +340,54 @@ setst (unsigned short bits, char * chars)
     }
 #endif
 }
+
+mode_t
+swlib_filestring_to_mode(char * str, int do_include_type)
+{
+	mode_t ret;
+	mode_t mode_ret;
+	int error;
+	char * s;
+	int type;
+
+	mode_ret = (mode_t)0;
+	s = str;
+
+	mode_ret = (mode_t)0;
+	type = *s;	
+	if (
+		type == 'd' ||
+		type == '-' ||
+		type == 'l' ||
+		type == 's' 
+	) {
+		if (do_include_type) {
+			/* FIXME, set type */
+		}
+		;
+	} else {
+		/* error */
+		return 0;
+	}
+	
+	s++; if (*s == '\0') return 0;
+	ret = set_perms(s, 0, &error);  /* user */
+	if (error) return 0;
+	mode_ret |= ret;
+
+	s++; if (*s == '\0') return 0;
+	s++; if (*s == '\0') return 0;
+	s++; if (*s == '\0') return 0;
+	ret = set_perms(s, 1, &error);  /* group */
+	if (error) return 0;
+	mode_ret |= ret;
+	
+	s++; if (*s == '\0') return 0;
+	s++; if (*s == '\0') return 0;
+	s++; if (*s == '\0') return 0;
+	ret = set_perms(s, 2, &error);  /* other */
+	if (error) return 0;
+	mode_ret |= ret;
+
+	return mode_ret;
+}
