@@ -4,7 +4,7 @@
 /*
    Copyright (C) 1985, 1992, 1993, 1994, 1996, 1997, 1999, 2000, 2001, 
    2003, 2004 Free Software Foundation, Inc.
-   Copyright (C) 2003,2004,2005 Jim Lowe
+   Copyright (C) 2003,2004,2005,2010 Jim Lowe
 
    Portions of this code are derived from code 
    copyrighted by the Free Software Foundation.  Retention of their 
@@ -26,6 +26,8 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+#define FILENEEDDEBUG 1
+#undef FILENEEDDEBUG
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -84,6 +86,9 @@ static SYSDBPAIR userCacheM[CACHE_LEN];
 static SYSDBPAIR groupCacheM[CACHE_LEN];
 
 #include "debug_config.h"
+#define TARNEEDDEBUG 1
+#undef TARNEEDDEBUG
+
 #ifdef TARNEEDDEBUG
 #define TAR_E_DEBUG(format) SWBISERROR("TAR DEBUG: ", format)
 #define TAR_E_DEBUG2(format, arg) SWBISERROR2("TAR DEBUG: ", format, arg)
@@ -126,8 +131,6 @@ error_msg_control(char * name, int id, char * dbname)
 	}
 	if (strstr(strob_str(store), strob_str(tmp))) {
 		;
-		/* JL strob_strcat(store, strob_str(tmp));	 */
-		/* JL fprintf(stderr, "eraseme %s", strob_str(tmp)); */
 	} else {
 		strob_strcat(store, strob_str(tmp));	
 		fprintf(stderr, "%s", strob_str(tmp));
@@ -222,6 +225,17 @@ l_tar_idcache_getuser(long uid, char *name)
 	return name;
 }
 
+static 
+int 
+l_cache_getuidbyname(char *user, long *puid)
+{
+	int ret;
+	uid_t id;
+	ret = taru_get_uid_by_name(user, &id);
+	*puid = (long)id;
+	return ret;
+}
+
 static 
 int 
 l_getuidbyname(char *user, long *puid)
@@ -250,6 +264,17 @@ l_tar_idcache_getgroup(long gid, char *name)
 	return name;
 }
 
+static 
+int 
+l_cache_getgidbyname(char *user, long *pgid)
+{
+	int ret;
+	gid_t id;
+	ret = taru_get_gid_by_name(user, &id);
+	*pgid = (long)id;
+	return ret;
+}
+
 static 
 int 
 l_getgidbyname(char *group, long *pgid)
@@ -351,7 +376,9 @@ int
 taru_get_uid_by_name(char *username, uid_t *puid)
 {
 	long pid;
-	int ret = get_pwent(0, 0, username, NULL,
+	int ret;
+	TAR_E_DEBUG("");
+	ret = get_pwent(0, 0, username, NULL,
 			&pid, 
 			l_getuidbyname, 
 			l_tar_idcache_getuser,
@@ -365,7 +392,9 @@ taru_get_uid_by_name(char *username, uid_t *puid)
 int 
 taru_get_user_by_uid(uid_t uid, char * buf)
 {
-	int ret = get_pwent(0, uid, NULL, buf, NULL,
+	int ret;
+	TAR_E_DEBUG("");
+	ret = get_pwent(0, uid, NULL, buf, NULL,
 			l_getuidbyname, 
 			l_tar_idcache_getuser,
 			&dbpair_user_nset,
@@ -377,8 +406,10 @@ taru_get_user_by_uid(uid_t uid, char * buf)
 int 
 taru_get_gid_by_name(char *groupname, gid_t *guid)
 {
+	int ret;
 	long pid;
-	int ret =  get_pwent(1, 0, groupname, NULL,
+	TAR_E_DEBUG("");
+	ret =  get_pwent(1, 0, groupname, NULL,
 			&pid,
 			l_getgidbyname, 
 			l_tar_idcache_getgroup,
@@ -392,7 +423,9 @@ taru_get_gid_by_name(char *groupname, gid_t *guid)
 int 
 taru_get_group_by_gid(gid_t gid, char * buf)
 {
-	int ret = get_pwent(1, gid, NULL, buf, NULL,
+	int ret;
+	TAR_E_DEBUG("");
+	ret = get_pwent(1, gid, NULL, buf, NULL,
 			l_getgidbyname, 
 			l_tar_idcache_getgroup,
 			&dbpair_group_nset,
@@ -444,7 +477,6 @@ taru_set_filetype_from_tartype(char ch, mode_t * mode, char * filename)
 	case DIRTYPE:
 		(*mode) |= CP_IFDIR;
 		break;
-#ifndef __MSDOS__
 	case CHRTYPE:
 		(*mode) |= CP_IFCHR;
 		break;
@@ -465,7 +497,6 @@ taru_set_filetype_from_tartype(char ch, mode_t * mode, char * filename)
 	case LNKTYPE:
 		(*mode) |= CP_IFREG;
 		break;
-#endif				/* !__MSDOS__ */
 	case AREGTYPE:
 		/* Old tar format; if the last char in filename
 		   is '/' then it is a directory, otherwise it's a regular
@@ -583,10 +614,8 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	int tar_iflags_retain_header_id = 0;
 	int buffer_offset;
 	int tmpret;
-#ifndef __MSDOS__
 	long uidp;
 	long gidp;
-#endif
 
 	buffer_offset = 0;
 	if (taru) {
@@ -603,7 +632,8 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	if (do_record_header) {
 		taru->header_lengthM = 0;	
 	}
-	
+
+	E_DEBUG("");	
 	GNU_L_REREAD:	
 	
 	if (header_buffer == NULL) {
@@ -623,6 +653,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		memcpy((void *) (&tar_rec), header_buffer+buffer_offset, TARRECORDSIZE);
 	}
 
+	E_DEBUG("");	
 	if (do_record_header) {
 		strob_set_memlength(taru->headerM, taru->header_lengthM + 512);
 		memcpy((void*)(strob_str(taru->headerM) + taru->header_lengthM),
@@ -632,6 +663,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 
 	retval += 512;
 
+	E_DEBUG("");	
 	/* Check for a block of 0's.  */
 	if (null_block((long *) &tar_rec, TARRECORDSIZE)) {
 #if 0
@@ -659,6 +691,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 #endif
 	}
 
+	E_DEBUG("");	
 	if (*(((char*)tar_hdr) + 156) == 'L') {
 		if (header_buffer && buf_len == TARRECORDSIZE) {
 			/*
@@ -682,6 +715,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			long_link_bytes += 512;
 			retval += 512;
 
+			E_DEBUG("");	
 			/* Sanity check */
 			if (retval + long_link_bytes != buf_len) {
 				fprintf(stderr, "fatal error in taru_in_tar_header2: type L error\n");
@@ -706,6 +740,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			/* 
 			* read the data block which contains the filename.
 			*/
+			E_DEBUG("");	
 			taru_otoumax(tar_hdr->size, &file_hdr->c_filesize);
 
 			if (file_hdr->c_filesize > 511) {
@@ -746,7 +781,9 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		}
 	}
 
+	E_DEBUG("");	
 	while (1) {
+		E_DEBUG("");	
 		taru_otoul(tar_hdr->chksum, &file_hdr->c_chksum);
 
 		if (file_hdr->c_chksum != taru_tar_checksum(tar_hdr)) {
@@ -801,6 +838,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			free(long_link_buffer);
 			long_link_buffer = NULL;
 		}
+		E_DEBUG("");	
 		file_hdr->c_namesize = strlen(tmpname) + 1;
 
 		if ((char)(tar_hdr->typeflag) == LNKTYPE) {
@@ -812,19 +850,24 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		taru_otoul(tar_hdr->mode, &(file_hdr->c_mode));
 		file_hdr->c_mode = file_hdr->c_mode & 07777;
 		
+		E_DEBUG("");	
 		ahsStaticSetTarUsername(file_hdr, tar_hdr->uname);
 		if (
 			tar_iflags_retain_header_id == 0 &&
-			l_getuidbyname(tar_hdr->uname, &uidp) == 0
+			l_cache_getuidbyname(tar_hdr->uname, &uidp) == 0
 		) {
+			E_DEBUG("");	
 			file_hdr->c_uid = uidp;
 		} else {
+			E_DEBUG("");	
 			taru_otoul(tar_hdr->uid, &file_hdr->c_uid);
 		}
+		E_DEBUG("");	
 		taru_set_filetype_from_tartype((char)(tar_hdr->typeflag),
 			(modet=(mode_t)(file_hdr->c_mode), &modet),
 				tmpname);
 	
+		E_DEBUG("");	
 		switch ((int)(tar_hdr->typeflag)) {
 			case LNKTYPE:
 				file_hdr->c_is_tar_lnktype = 1;
@@ -833,10 +876,11 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 				file_hdr->c_is_tar_lnktype = 0;
 		}
 	
+		E_DEBUG("");	
 		ahsStaticSetTarGroupname(file_hdr, tar_hdr->gname);
 		if (
 			tar_iflags_retain_header_id == 0 &&
-			l_getgidbyname(tar_hdr->gname, &gidp) == 0
+			l_cache_getgidbyname(tar_hdr->gname, &gidp) == 0
 		) {
 			file_hdr->c_gid = gidp;
 		} else {
@@ -849,6 +893,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 				(unsigned long *) &file_hdr->c_rdev_maj);
 		taru_otoul(tar_hdr->devminor,
 				(unsigned long *) &file_hdr->c_rdev_min);
+		E_DEBUG("");	
 		ahsStaticSetTarLinkname(file_hdr, (char *)(NULL));
 
 		switch (tar_hdr->typeflag) {
@@ -860,7 +905,6 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		case DIRTYPE:
 			file_hdr->c_mode |= CP_IFDIR;
 			break;
-#ifndef __MSDOS__
 		case CHRTYPE:
 			file_hdr->c_mode |= CP_IFCHR;
 
@@ -906,7 +950,6 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			ahsStaticSetTarLinkname(file_hdr, tar_hdr->linkname);
 			file_hdr->c_filesize = 0;
 			break;
-#endif				/* !__MSDOS__ */
 		case AREGTYPE:
 			/* Old tar format; if the last char in 
 			   filename is '/' then it is
@@ -966,7 +1009,7 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	TAR_E_DEBUG2("ENTERING %d", out_des);
 	TAR_E_DEBUG3("ENTERING username = %s groupname = %s",\
 						username, groupname);
-	TAR_E_DEBUG2("file_hdr %s\n", taru_header_dump_string_s(file_hdr, ""));
+	/* TAR_E_DEBUG2("file_hdr %s\n", taru_header_dump_string_s(file_hdr, "")); */
 	bzero((char *) &tar_rec, TARRECORDSIZE);
 
 	tar_iflags_like_star = (tar_iflags & TARU_TAR_BE_LIKE_STAR);
@@ -1070,7 +1113,6 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	case CP_IFDIR:
 		tar_hdr->typeflag = DIRTYPE;
 		break;
-#ifndef __MSDOS__
 	case CP_IFCHR:
 		tar_hdr->typeflag = CHRTYPE;
 		break;
@@ -1100,7 +1142,6 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		SIZE_TO_CHARS(0,  tar_hdr->size, termch);
 		break;
 #endif				/* CP_IFLNK */
-#endif				/* !__MSDOS__ */
 
 	default:
 		if (file_hdr->c_mode == 0 &&
@@ -1144,11 +1185,6 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			{
 			TAR_E_DEBUG2("finding uid for username = %s", username);
 			if (taru_get_uid_by_name(username, &puid)) {
-				/* JL
-				fprintf(stderr,
-				"%s: error finding uname [%s], setting uid to %d\n",
-					swlib_utilname_get(), username, (int)puid);
-				*/
 				puid = NOBODY_ID;
 			}
 			swlib_strncpy(tar_hdr->uname, username, SNAME_LEN);
@@ -1184,11 +1220,6 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			TAR_E_DEBUG2("finding gid for groupname=%s", groupname);
 			swlib_strncpy(tar_hdr->gname, groupname, SNAME_LEN);
 			if (taru_get_gid_by_name(groupname, &pgid)) {
-				/* JL
-				fprintf(stderr,
-				"%s: error finding gname [%s], setting gid to %d\n",
-					swlib_utilname_get(), username, (int)pgid);
-				*/
 				pgid = NOBODY_ID;
 			}
 		} else if (
@@ -1213,8 +1244,6 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	}
 
 	if (tar_hdr->typeflag == CHRTYPE || tar_hdr->typeflag == BLKTYPE) {
-		/* fprintf(stderr, "JLJL major here %d\n", (int)(file_hdr->c_rdev_maj)); */
-		/* fprintf(stderr, "JLJL minor here %d\n", (int)(file_hdr->c_rdev_min)); */
 		MAJOR_TO_CHARS(file_hdr->c_rdev_maj, tar_hdr->devmajor, termch);
 		MINOR_TO_CHARS(file_hdr->c_rdev_min, tar_hdr->devminor, termch);
 	} else {
