@@ -22,6 +22,7 @@
 
 /* #include "system.h" */
 #include "swuser_config.h"
+#include "swutilname.h"
 
 #define STRINGIFY_BIGINT(i, b) \
   stringify_uintmax_t_backwards ((uintmax_t) (i), (b) + UINTMAX_STRSIZE_BOUND)
@@ -181,7 +182,7 @@ to_chars (int negative, uintmax_t value, size_t valsize,
 	    <= MAX_VAL_WITH_DIGITS (size - 1, LG_256))
 	   && base256_allowed)
     {
-      fprintf(stderr, "tar: to_chars: making old gnu field\n");
+      fprintf(stderr, "%s: to_chars: making old gnu field\n", swlib_utilname_get());
       where[0] = negative ? -1 : 1 << (LG_256 - 1);
       to_base256 (negative, value, where + 1, size - 1);
     }
@@ -195,7 +196,7 @@ to_chars (int negative, uintmax_t value, size_t valsize,
   else if (negative && valsize * CHAR_BIT <= (size - 1) * LG_8)
     {
       static int warned_once;
-      fprintf(stderr, "tar: to_chars: making non portable field\n");
+      fprintf(stderr, "%s: to_chars: making non portable field\n", swlib_utilname_get());
       if (! warned_once)
 	{
 	  warned_once = 1;
@@ -220,7 +221,7 @@ to_chars (int negative, uintmax_t value, size_t valsize,
       char const *maxval_string = STRINGIFY_BIGINT (maxval, maxbuf);
       char const *value_string;
 
-      fprintf(stderr, "tar: to_chars: making substitute value.\n");
+      fprintf(stderr, "%s: to_chars: making substitute value.\n", swlib_utilname_get());
       if (base256_allowed)
 	{
 	  uintmax_t m = maxval + 1 ? maxval + 1 : maxval / 2 + 1;
@@ -262,20 +263,21 @@ to_chars (int negative, uintmax_t value, size_t valsize,
 static uintmax_t
 gid_substitute (int *negative)
 {
-  gid_t r;
+  intmax_t r;
+  int eid = 0;
+  static gid_t gid_nobody;
 #ifdef GID_NOBODY
   r = GID_NOBODY;
 #else
-  static gid_t gid_nobody;
-
    if (!gid_nobody) {
 	if (taru_get_gid_by_name("nobody", &gid_nobody)) {
+		eid = -2;
     		gid_nobody = -2;
 	}
    }
    r = (gid_t)gid_nobody;
 #endif
-  *negative = r < 0;
+  *negative = eid < 0;
   return r;
 }
 
@@ -357,19 +359,22 @@ time_to_chars (time_t v, char *p, size_t s, int archive_format, int termch)
 static uintmax_t
 uid_substitute (int *negative)
 {
-  uid_t r;
+  intmax_t r;
+  int eid = 0;
+  static uid_t uid_nobody;
 #ifdef UID_NOBODY
   r = UID_NOBODY;
 #else
-  static uid_t uid_nobody;
   
    if (!uid_nobody) {
-        if (taru_get_uid_by_name("nobody", &uid_nobody))
+        if (taru_get_uid_by_name("nobody", &uid_nobody)) {
+		eid = -2;
     		uid_nobody = -2;
+	}
    }
   r = (uid_t)uid_nobody;
 #endif
-  *negative = r < 0;
+  *negative = eid < 0;
   return r;
 }
 
