@@ -24,7 +24,6 @@
 
 #define XFORMATNEEDDEBUG 1
 #include "debug_config.h"
-#define XFORMATNEEDDEBUG 1
 #undef XFORMATNEEDDEBUG
 
 #ifdef XFORMATNEEDDEBUG
@@ -286,13 +285,17 @@ u_open_file_common(XFORMAT * xux, int fd)
 {
 	char *s;
 	struct stat st;
+	XFORMAT_E_DEBUG("");
 	swvarfs_u_fstat(xux->swvarfsM, fd, &st);	
 	taru_statbuf2filehdr(xformat_vfile_hdr(xux), &st,
 				(char*)NULL, (char*)NULL, (char*)NULL);
+	XFORMAT_E_DEBUG("");
 	ahs_set_name(xux->ahsM,
 		(s = swvarfs_u_get_name(xux->swvarfsM, fd)) ? s : "");
+	XFORMAT_E_DEBUG("");
 	ahs_set_linkname(xux->ahsM,
 		(s = swvarfs_u_get_linkname(xux->swvarfsM, fd)) ? s : "");
+	XFORMAT_E_DEBUG("");
 	return fd;
 }
 
@@ -673,7 +676,7 @@ xformat_set_preview_level(XFORMAT * xux, int fd)
 { 
 	XFORMAT_E_DEBUG("");
 	if (!(xux->taruM)) {
-		fprintf(stderr, "JLJL AAARRRRGG at line %d\n", __LINE__);
+		fprintf(stderr, "AAARRRRGG xux->taruM is null at line %d\n", __LINE__);
 		return;
 	}
 	taru_set_preview_level(xux->taruM, fd);
@@ -684,7 +687,7 @@ xformat_set_preview_fd(XFORMAT * xux, int fd)
 { 
 	XFORMAT_E_DEBUG("");
 	if (!(xux->taruM)) {
-		fprintf(stderr, "JLJL AAARRRRGG at line %d\n", __LINE__);
+		fprintf(stderr, "AAARRRRGG xux->taruM at line %d\n", __LINE__);
 		return;
 	}
 	taru_set_preview_fd(xux->taruM, fd);
@@ -919,11 +922,14 @@ intmax_t
 xformat_write_by_name(XFORMAT * xux, char * name, struct stat *st)
 {
 	intmax_t ret;
+        struct  new_cpio_header * hdr0 = ahsStaticCreateFilehdr();
 	
 	XFORMAT_E_DEBUG("");
+
+	taru_statbuf2filehdr(hdr0, st, (char*)NULL, name, (char*)NULL);
 	ret = taru_write_archive_member(xux->taruM, name,
-					st,
-					(struct new_cpio_header *)(NULL),
+					(struct stat*)(NULL) /* st */,
+					hdr0,
 					xux->link_recordM, 
 					xux->deferM,
 					xux->use_false_inodesM,
@@ -935,6 +941,7 @@ xformat_write_by_name(XFORMAT * xux, char * name, struct stat *st)
 	
 	if (ret > 0) xux->bytes_writtenM += ret;
 	XFORMAT_E_DEBUG("LEAVING");
+	ahsStaticDeleteFilehdr(hdr0);
 	return ret;
 }
 
@@ -1194,6 +1201,22 @@ xformat_close_archive(XFORMAT * xux)
 	return ret;
 }
 
+int 
+xformat_open_archive_by_fd_and_name(XFORMAT * xux, int fd, int flags, mode_t mode, char * name)
+{
+	
+	XFORMAT_E_DEBUG("");
+
+	if (name == NULL)
+		return xformat_open_archive_by_fd(xux, fd, flags, mode);
+
+	/* fprintf(stderr, "JL you're a SLACKER: %s:%s at line %d\n", __FILE__, __FUNCTION__, __LINE__); */
+	xux->swvarfsM = swvarfs_opendup_with_name(fd, flags, mode, name);
+	if (xux->swvarfsM == NULL) return -1; 
+	open_archive_init(xux);
+	return 0;
+}
+
 int 
 xformat_open_archive_by_fd(XFORMAT * xux, int fd, int flags, mode_t mode)
 {
@@ -1312,7 +1335,7 @@ xformat_copy_pass_md5(XFORMAT * xux, int dst_fd, int src_fd, char * md5buf)
 		strcpy(md5buf, digs->md5);
 	}
 	if (retval > 0) xux->bytes_writtenM += retval;
-	free(digs);
+	taru_digs_delete(digs);
 	return retval;
 }
 
