@@ -60,6 +60,9 @@
 #define UINFILE_I_INIT_READ_SIZE 124
 #define UINFILE_I_TAR_BLOCK_SIZE 512
 
+#define MIN_PEEK_LEN	3
+#define LENGTH_OF_MAGIC	3  
+
 static
 int
 does_have_name_version(char * buf)
@@ -202,6 +205,8 @@ uinfile_detect_ieee(UINFORMAT * uinformat, int oflags)
 	int allow_generic_tar = oflags & UINFILE_DETECT_OTARALLOW;
 	struct new_cpio_header *file_hdr=uinformat->file_hdrM;
 
+	if (uinformat->swpathM)
+		swpath_close(uinformat->swpathM);
 	uinformat->swpathM = swpath_open("");
 
 	if (uinfile_check_ieee_fd(uinformat->fdM) < 0) {
@@ -388,7 +393,7 @@ uinfile_handle_return(UINFORMAT * uinformat, int oflags, int uxfio_buffer_type)
 
 static int
 uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
-				int oflags, int xdupfd, int uxfio_buffer_type)
+		int oflags, int xdupfd, int uxfio_buffer_type, char * slack_name)
 {
 	int ret;
 	int i;
@@ -403,6 +408,7 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 	char rpmmagic[] = UINFILE_MAGIC_rpm;
 	char bz2magic[] = UINFILE_MAGIC_bz2;
 	char lzmamagic[] = UINFILE_MAGIC_lzma;
+	char xzmamagic[] = UINFILE_MAGIC_xz;
 	char debmagic[] = UINFILE_MAGIC_deb;
 	char slackmagic[] = UINFILE_MAGIC_slack;
 	char *zcat_command = (char *) (NULL);
@@ -424,19 +430,26 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 	int val1;
 	int val2;
 
+	/*
+	if (slack_name)
+		*-*fprintf(stderr, "JL you're a SLACKER: %s:%s at line %d\n", __FILE__, __FUNCTION__, __LINE__);
+	*/
 	UINFILE_E_DEBUG("BEGIN ****************************** ");
 	UINFILE_E_DEBUG2("xdupfd = %d", xdupfd);
+	UINFILE_E_DEBUG2("slack_name = %s", slack_name);
+
 	dodebcontrol = oflags & UINFILE_DETECT_DEB_CONTROL;
 	dodebcontext = oflags & UINFILE_DETECT_DEB_CONTEXT;
 	dodebdata = oflags & UINFILE_DETECT_DEB_DATA;
+
 	UINFILE_E_DEBUG2("dodebdata=%d", dodebdata);
 	UINFILE_E_DEBUG2("dodebcontext=%d", dodebcontext);
 	UINFILE_E_DEBUG2("dodebcontrol=%d", dodebcontrol);
+
 	forcetar = oflags & UINFILE_DETECT_OTARFORCE;
 	doieee = oflags & UINFILE_DETECT_IEEE;
 	doarb = oflags & UINFILE_DETECT_ARBITRARY_DATA;
 	dounrpminstall = oflags & UINFILE_DETECT_UNRPM;
-/* JL test dounrpminstall = 0;  */
 	douncpio = oflags & UINFILE_DETECT_UNCPIO;
 	donative = oflags & UINFILE_DETECT_NATIVE;
 	
@@ -460,13 +473,17 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 		(*uinformat)->ztypeM = UINFILE_COMPRESSED_NA;
 		(*uinformat)->typeM = UNKNOWN_FILEFORMAT;
 		memset((*uinformat)->type_revisionM, '\0', sizeof((*uinformat)->type_revisionM));
-		(*uinformat)->swpathM = (SWPATH*)(NULL);
+		(*uinformat)->swpathM = swpath_open("");
 		(*uinformat)->taruM = taru_create();
 		(*uinformat)->slackheaderM = NULL;
 		UINFILE_E_DEBUG("seting deb_file_fd = -1");
 		(*uinformat)->deb_file_fd_ = -1;
 		(*uinformat)->n_deb_peeked_bytesM = 0;
 		(*uinformat)->pathname_prefixM = NULL;
+		if (slack_name)
+			(*uinformat)->slack_nameM = strdup(slack_name);
+		else
+			(*uinformat)->slack_nameM = NULL;
 		sigemptyset(&((*uinformat)->blockmask_));
 		sigemptyset(&((*uinformat)->defaultmask_));
 		sigaddset(&((*uinformat)->blockmask_), SIGALRM);
@@ -477,10 +494,10 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 				(*uinformat)->pidlistM[i] = 0;
 	}
 
-
 	/* ------------------------------------------------------------- */
 	/*            Open file or pipe                                  */
 	/* ------------------------------------------------------------- */
+
 	if (filename == (char *) (NULL) ||
 				(!strcmp(filename, "-"))) {	/* is a pipe */
 		UINFILE_E_DEBUG("HERE PIPE");
@@ -495,8 +512,11 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 		UINFILE_E_DEBUG2("HERE dupfd = %d", dupfd);
 		
 		val1 = dupfd < UXFIO_FD_MIN ;
-		val2 = lseek(dupfd, 0L, SEEK_CUR);
-
+		if (dupfd < UXFIO_FD_MIN)
+			val2 = lseek(dupfd, 0L, SEEK_CUR);
+		else
+			val2 = -1;
+			
 		if (val1 && val2 == -1) {
 			UINFILE_E_DEBUG3("HERE %d %s", errno, strerror(errno));
 			if (errno == ESPIPE){
@@ -520,7 +540,7 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 			}
 		} else {
 			UINFILE_E_DEBUG("HERE");
-			ret = lseek(dupfd, 0L, SEEK_CUR);
+			ret = val2; /*lseek(dupfd, 0L, SEEK_CUR); */
 			if (ret >= 0) {
 				UINFILE_E_DEBUG2("HERE current_pos_=%d", (int)((*uinformat)->current_pos_));
 				(*uinformat)->current_pos_ = ret;
@@ -570,21 +590,31 @@ LABEL_REREAD:
 		peeklen = UINFILE_I_INIT_READ_SIZE;
 	} else {
 		/*
-		 * this is used is DETECT_ABRITRARY_DATA is set
+		 * this is used if DETECT_ABRITRARY_DATA is set
 		 */
 		UINFILE_E_DEBUG("HERE");
-		peeklen = 3;
+		peeklen = MIN_PEEK_LEN;
 	}	
 
 	memset(xbuffer, '\0', sizeof(xbuffer));
 
 	UINFILE_E_DEBUG3("HERE refd=%d  peeklen=%d", refd, peeklen);
-	if ((ret=taru_tape_buffered_read(refd, xbuffer, peeklen)) != peeklen) {
-		UINFILE_E_DEBUG("ERROR HERE");
-		if (ret <= 0) fprintf(stderr,
-		"%s: error in initial read. request=%d return=%d\n",
-			swlib_utilname_get(), peeklen,  ret);
-		return -1;
+	if (doarb) {
+		/* arbitray data, make no requirement that it be longer than MIN_PEEK_LEN */
+		if ((ret=uxfio_read(refd, xbuffer, peeklen)) < 0) {
+			UINFILE_E_DEBUG("ERROR HERE");
+			fprintf(stderr, "%s: error in read of arbitrary data: request=%d return=%d\n",
+				swlib_utilname_get(), peeklen,  ret);
+			return -1;
+		}
+	}  else {
+		if ((ret=taru_tape_buffered_read(refd, xbuffer, peeklen)) != peeklen) {
+			UINFILE_E_DEBUG("ERROR HERE");
+			if (ret <= 0) fprintf(stderr,
+			"%s: error in initial read. request=%d return=%d\n",
+				swlib_utilname_get(), peeklen,  ret);
+			return -1;
+		}
 	}
 	UINFILE_E_DEBUG("HERE");
 
@@ -704,14 +734,18 @@ LABEL_REREAD:
 						/* gzipped compression */
 			UINFILE_E_DEBUG("type: COMPRESSED_GZ");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_GZ;
-		} else if (!memcmp(xbuffer, (void *) bz2magic, 3)) {
+		} else if (!memcmp(xbuffer, (void *) bz2magic, LENGTH_OF_MAGIC)) {
 							/* bz2 compression */
 			UINFILE_E_DEBUG("type: COMPRESSED_BZ2");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_BZ2;
-		} else if (!memcmp(xbuffer, (void *) lzmamagic, 3)) {
+		} else if (!memcmp(xbuffer, (void *) lzmamagic, LENGTH_OF_MAGIC)) {
 							/* LZMA compression */
 			UINFILE_E_DEBUG("type: COMPRESSED_LZMA");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_LZMA;
+		} else if (!memcmp(xbuffer, (void *) xzmamagic, LENGTH_OF_MAGIC)) {
+							/* XZ compression */
+			UINFILE_E_DEBUG("type: COMPRESSED_XZ");
+			(*uinformat)->ztypeM = UINFILE_COMPRESSED_XZ;
 		} else if (!memcmp(xbuffer, (void *) Zmagic, 2)) {
 							/* Unix compress */
 			UINFILE_E_DEBUG("type: COMPRESSED_Z");
@@ -758,11 +792,11 @@ LABEL_REREAD:
 			   the gz magic is there */
 
 			deb_gz_offset = SARMAG + SARHDR + val1 + SARHDR;  /* val1 must be 4 */
-			if (peeklen < deb_gz_offset + 3) {
+			if (peeklen < deb_gz_offset + LENGTH_OF_MAGIC) {
 				/* read further into the file */
 
-				ret = taru_tape_buffered_read(refd, xbuffer+peeklen, deb_gz_offset+3 - peeklen);
-				if (ret != deb_gz_offset+3 - peeklen) {
+				ret = taru_tape_buffered_read(refd, xbuffer+peeklen, deb_gz_offset+LENGTH_OF_MAGIC - peeklen);
+				if (ret != deb_gz_offset+LENGTH_OF_MAGIC - peeklen) {
 					fprintf(stderr,
 					"%s: error in second initial read. request=%d return=%d\n",
 						swlib_utilname_get(), peeklen,  ret);
@@ -772,7 +806,7 @@ LABEL_REREAD:
 			
 			/* look for gzmagic */
 
-			if (memcmp(((unsigned char*)xbuffer)+deb_gz_offset, (void *)gzmagic, 3) != 0) {
+			if (memcmp(((unsigned char*)xbuffer)+deb_gz_offset, (void *)gzmagic, LENGTH_OF_MAGIC) != 0) {
 				fprintf(stderr, "%s: gz magic not found for control.tar.gz\n", swlib_utilname_get());
 				return -1;
 			}
@@ -838,7 +872,6 @@ LABEL_REREAD:
 			   plain tarball */
 
 			taru_print_tar_ls_list(tmp, (*uinformat)->file_hdrM, LS_LIST_VERBOSE_L2);
-			/* fprintf(stderr, "JL [%s]\n", strob_str(tmp)); */
 
 			(*uinformat)->slackheaderM = malloc(TARRECORDSIZE);
 			if ((*uinformat)->slackheaderM == NULL) return -1;
@@ -920,6 +953,23 @@ LABEL_REREAD:
 			UINFILE_E_DEBUG2("returning ret=%d", ret);
 			strob_close(tmp);
 			return ret;
+
+		} else if (slack_name && doieee == 0 && donative == 0 && dodebcontext == 0 &&
+			dodebcontrol == 0 && dodebdata == 0 &&
+			dounrpminstall == 0 &&
+			memcmp(xbuffer, UINFILE_MAGIC_binary_tarball, strlen(UINFILE_MAGIC_binary_tarball)) == 0
+		) {
+			UINFILE_E_DEBUG2("type: binary_tarball, probably slackware: slack_name=%s", slack_name);
+
+			/* Here we know it could be slackware binary package
+			   this code path supports the --slackware-pkg-name=NAME option of swpackage
+			   and lxpsf and the swinstall case of ''swinstall --slackware -s:slackage-binary-tarball'' 
+			   No other cases take this code path, If we are here we punt to swpackage instead of
+			   returning in this process */
+
+			(*uinformat)->ztypeM = UINFILE_COMPRESSED_SLACK_WITH_NAME;
+			/* fprintf(stderr, "JL you're a SLACKER: %s:%s at line %d\n", __FILE__, __FUNCTION__, __LINE__); */
+
 		} else if (doieee == 0 && donative == 0 && dodebcontext == 0 &&
 			dodebcontrol == 0 && dodebdata == 0 &&
 			dounrpminstall == 0 &&
@@ -966,7 +1016,7 @@ LABEL_REREAD:
 				char * name;
 				int found;
 
-				UINFILE_E_DEBUG("");
+				UINFILE_E_DEBUG("Looking for install directory");
 				/* The only binary tarball recognized is slackware */
 
 				/* This code path handles the ./install directory placed anywhere
@@ -1020,12 +1070,14 @@ LABEL_REREAD:
 
 				swvarfs_close(swvarfs);
 				if (found) {
+					UINFILE_E_DEBUG("FOUND SLACK");
 					(*uinformat)->ztypeM = UINFILE_COMPRESSED_NOT;
 					(*uinformat)->typeM = SLACK_FILEFORMAT;
 					ret =  uinfile_handle_return(*uinformat,
 						UINFILE_DETECT_FORCEUXFIOFD,
 						UXFIO_BUFTYPE_DYNAMIC_MEM);
 				} else {
+					UINFILE_E_DEBUG("error");
 					ret = -1;
 				}
 				UINFILE_E_DEBUG2("returning fd=%d", ret);
@@ -1063,11 +1115,13 @@ LABEL_REREAD:
 				if (
 					(*uinformat)->ztypeM == UINFILE_COMPRESSED_DEB
 				) {
+					UINFILE_E_DEBUG("Found Compressed DEB");
 					(*uinformat)->typeM=DEB_FILEFORMAT;
 					(*uinformat)->has_leading_slashM = 0; /* NO */
 					ret = uinfile_handle_return(*uinformat,
 							oflags, uxfio_buffer_type);
 				} else {
+					UINFILE_E_DEBUG("Found USTAR_FILEFORMAT");
 					(*uinformat)->typeM=USTAR_FILEFORMAT;
 					(*uinformat)->has_leading_slashM =
 						determine_if_has_leading_slash(
@@ -1093,12 +1147,15 @@ LABEL_REREAD:
 				*/
 				return ret;
 			} else {
+				UINFILE_E_DEBUG("TAR");
 				if (dounrpminstall) {
 					/* Try to let swpackage decode it */
+					UINFILE_E_DEBUG("Setting ztypeM to UINFILE_COMPRESSED_RPM");
 					(*uinformat)->ztypeM = UINFILE_COMPRESSED_RPM;
 				} else {
 					uxfio_close(refd);
 					swbis_free(*uinformat);
+					UINFILE_E_DEBUG("Unrecognized tar layout");
 					fprintf(stderr, "%s: error: uinfile: unrecognized tar variant format\n",
 							swlib_utilname_get());
 					return -1;
@@ -1106,20 +1163,24 @@ LABEL_REREAD:
 			}
 			/* } */ /* arbitrary tar */
 		}
-	} else {
+	} else { /* doarb */
 		UINFILE_E_DEBUG("");
 		if (!memcmp(xbuffer, (void *) gzmagic, 2)) {
 						/* gzipped compression */
 			UINFILE_E_DEBUG("");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_GZ;
-		} else if (!memcmp(xbuffer, (void *) bz2magic, 3)) {
+		} else if (!memcmp(xbuffer, (void *) bz2magic, LENGTH_OF_MAGIC)) {
 							/* bz2 compression */
 			UINFILE_E_DEBUG("");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_BZ2;
-		} else if (!memcmp(xbuffer, (void *) lzmamagic, 3)) {
+		} else if (!memcmp(xbuffer, (void *) lzmamagic, LENGTH_OF_MAGIC)) {
 							/* LZMA compression */
 			UINFILE_E_DEBUG("");
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_LZMA;
+		} else if (!memcmp(xbuffer, (void *) xzmamagic, LENGTH_OF_MAGIC)) {
+							/* XZ compression */
+			UINFILE_E_DEBUG("");
+			(*uinformat)->ztypeM = UINFILE_COMPRESSED_XZ;
 		} else if (!memcmp(xbuffer, (void *) Zmagic, 2)) {
 							/* Unix compress */
 			UINFILE_E_DEBUG("");
@@ -1172,12 +1233,12 @@ LABEL_REREAD:
 	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_DEB) {
 		UINFILE_E_DEBUG("in deb code");
 		if (memcmp(((char*)(xbuffer)) + deb_gz_offset,
-				gzmagic, 3) == 0) {
+				gzmagic, LENGTH_OF_MAGIC) == 0) {
 			unzip[0] = shcmd_open();
 			shcmd_add_arg(unzip[0], "gzip");
 			shcmd_add_arg(unzip[0], "-d");
 		} else if (memcmp(((char*)(xbuffer)) + deb_gz_offset,
-				bz2magic, 3) == 0) {
+				bz2magic, LENGTH_OF_MAGIC) == 0) {
 			/*
 			 * preemptive support for bzip2'ed debian packages
 			 */
@@ -1212,6 +1273,11 @@ LABEL_REREAD:
 		unzip[0] = shcmd_open();
 		shcmd_add_arg(unzip[0], "lzma");
 		shcmd_add_arg(unzip[0], "-d");
+	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_XZ) {
+		UINFILE_E_DEBUG("in xz code");
+		unzip[0] = shcmd_open();
+		shcmd_add_arg(unzip[0], "xz");
+		shcmd_add_arg(unzip[0], "-d");
 	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_GPG) {
 		UINFILE_E_DEBUG("in gpg code");
 		unzip[0] = shcmd_open();
@@ -1227,10 +1293,23 @@ LABEL_REREAD:
 		shcmd_add_arg(unzip[0], SWBISLIBEXECDIR "/swbis/arf2arf");
 		shcmd_add_arg(unzip[0], "-H");
 		shcmd_add_arg(unzip[0], "ustar");
+	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_SLACK_WITH_NAME) {
+		UINFILE_E_DEBUG("in UINFILE_COMPRESSED_SLACK_WITH_NAME");
+		/* fprintf(stderr, "JL you're a SLACKER: %s:%s at line %d\n", __FILE__, __FUNCTION__, __LINE__); */
+		/* shcmd_add_arg(unzip[0], "swbis"); */
+		unzip[0] = shcmd_open();
+		shcmd_add_arg(unzip[0], SWBISBINDIR "/swpackage");
+		shcmd_add_arg(unzip[0], "--to-swbis");
+		shcmd_add_arg(unzip[0], "-s");
+		shcmd_add_arg(unzip[0], "-");
+		shcmd_add_arg(unzip[0], "--slackware-pkg-name");
+		shcmd_add_arg(unzip[0], slack_name);
+		shcmd_add_arg(unzip[0], "--catalog-owner=0");
+		shcmd_add_arg(unzip[0], "--catalog-group=0");
+		shcmd_add_arg(unzip[0], "@-");
 	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_RPM) {
 		UINFILE_E_DEBUG("in unrpm code");
 		unzip[0] = shcmd_open();
-		/* shcmd_add_arg(unzip[0], "swbis"); */
 		shcmd_add_arg(unzip[0], SWBISBINDIR "/swpackage");
 		shcmd_add_arg(unzip[0], "--to-swbis");
 		/*
@@ -1462,6 +1541,7 @@ LABEL_REREAD:
 	fprintf(stderr, " uinfile: internal error 85.002\n");
 	return -1;
 }
+
 /*
  * -------------- Public Functions ------------------
  */
@@ -1474,10 +1554,39 @@ uinfile_open(char *filename,  mode_t mode, UINFORMAT ** uinformat, int oflags)
 	UINFILE_E_DEBUG("");
 	buftype = uinfile_decode_buftype(oflags, UXFIO_BUFTYPE_DYNAMIC_MEM);
 	ret = uinfile_i_open(filename, -1, mode, uinformat,
-					oflags, -1, buftype);
+					oflags, -1, buftype, NULL);
 	return ret;
 }
 
+int
+uinfile_open_with_name(char *filename,  mode_t mode, UINFORMAT ** uinformat, int oflags, char * name)
+{
+	int ret;
+	int buftype;
+	UINFILE_E_DEBUG("");
+	buftype = uinfile_decode_buftype(oflags, UXFIO_BUFTYPE_DYNAMIC_MEM);
+	ret = uinfile_i_open(filename, -1, mode, uinformat,
+					oflags, -1, buftype, name);
+	return ret;
+}
+
+int
+uinfile_opendup_with_name(int xdupfd, mode_t mode, UINFORMAT ** uinformat, int oflags, char * name)
+{
+	int ret;
+	int buftype;
+	UINFILE_E_DEBUG("");
+
+	if (name == NULL)
+		return uinfile_opendup(xdupfd, mode, uinformat, oflags);
+
+	buftype = uinfile_decode_buftype(oflags, UXFIO_BUFTYPE_DYNAMIC_MEM);
+	ret = uinfile_i_open((char *)NULL, -1, mode,
+				uinformat, oflags, xdupfd, buftype, name);
+	return ret;
+}
+
+
 int
 uinfile_opendup(int xdupfd, mode_t mode, UINFORMAT ** uinformat, int oflags)
 {
@@ -1486,7 +1595,7 @@ uinfile_opendup(int xdupfd, mode_t mode, UINFORMAT ** uinformat, int oflags)
 	UINFILE_E_DEBUG("");
 	buftype = uinfile_decode_buftype(oflags, UXFIO_BUFTYPE_DYNAMIC_MEM);
 	ret = uinfile_i_open((char *)NULL, -1, mode,
-				uinformat, oflags, xdupfd, buftype);
+				uinformat, oflags, xdupfd, buftype, NULL);
 	return ret;
 }
 
@@ -1622,6 +1731,7 @@ int
 uinfile_close(UINFORMAT * uinformat)
 {
 	int ret;
+	UINFILE_E_DEBUG("entering");
 	ahsStaticDeleteFilehdr(uinformat->file_hdrM);
 	/* uxfio_close(uinformat->fdM); */
 	ret = uinfile_wait_on_all_pid(uinformat, 0);
@@ -1630,6 +1740,11 @@ uinfile_close(UINFORMAT * uinformat)
 			"Leaving uinfile_close with status %d\n", ret);
 	if (uinformat->taruM)
 		taru_delete(uinformat->taruM);
+	if (uinformat->swpathM)
+		swpath_close(uinformat->swpathM);
+	if (uinformat->slack_nameM)
+		free(uinformat->slack_nameM);
 	swbis_free(uinformat);
+	UINFILE_E_DEBUG("");
 	return ret;
 }
