@@ -37,7 +37,6 @@
 
 #include "debug_config.h"
 
-
 #define UXFIONEEDDEBUG 1
 #undef UXFIONEEDDEBUG 
 
@@ -67,6 +66,8 @@ static unsigned char uxg_status_table[UXFIO_MAX_OPEN];
 static int uxg_desc_table[UXFIO_MAX_OPEN];
 static UXFIO * uxg_file_table[UXFIO_MAX_OPEN];
 static int uxg_did_init = 0;
+static int uxg_nullfd = 0;
+
 
 #define UXFIO_REMOVE 0
 #define UXFIO_ADD 1
@@ -194,7 +195,7 @@ alloc_file(int i)
 
 static 
 int 
-table_find(int uxfio_fildes, UXFIO ** uxfio_addr)
+table_find(int uxfio_fildes, UXFIO ** uxfio_addr, int v)
 {
 	static int last_used_fd_index;
 	int  i;
@@ -224,7 +225,7 @@ table_find(int uxfio_fildes, UXFIO ** uxfio_addr)
 	}
 	*uxfio_addr = (void *) (NULL);
 	UXFIO_E_DEBUG2("error %d", uxfio_fildes);
-	fprintf(stderr,
+	if (v) fprintf(stderr,
 		"uxfio error: descriptor %d not found.\n", uxfio_fildes);
 	return -1;
 }
@@ -240,7 +241,7 @@ table_mon(int cmd, int *uxfio_fildes, UXFIO ** uxfio_addr)
 	/* make uxfio descriptors begin at UXFIO_FD_MIN */
 
 	if (cmd == UXFIO_FIND) {
-		return table_find(p_index, uxfio_addr);
+		return table_find(p_index, uxfio_addr, 1);
 	} else if (cmd == UXFIO_REMOVE) {
 		for (i = 0; i < UXFIO_MAX_OPEN; i++) {
 			if (uxg_desc_table[i] == p_index) {
@@ -262,7 +263,6 @@ table_mon(int cmd, int *uxfio_fildes, UXFIO ** uxfio_addr)
 				alloc_file(i);
 				*uxfio_addr = (void *)(uxg_file_table[i]);
 				*uxfio_fildes = UXFIO_FD_MIN + i;
-				/* fprintf(stderr, "JL descriptor index = %d\n", i); */
 				return 0;
 			}
 		}
@@ -621,12 +621,15 @@ uxfio__close(int uxfio_fildes, int closeit)
 	int fd;
 	int ret = 0;
 
+	if (uxfio_fildes == uxg_nullfd)
+		return 0;
+
 	if (uxfio_fildes < UXFIO_FD_MIN && uxfio_fildes >= 0) {
 		UXFIO_E_DEBUG2("unix filedes=%d", uxfio_fildes);
 		return close(uxfio_fildes);
 	}
 
-	if (table_find(uxfio_fildes, &uxfio)) {
+	if (table_find(uxfio_fildes, &uxfio, 1)) {
 		return -1;
 	}
 
@@ -685,7 +688,7 @@ uxfio_common_read(UXFIO *uxfio, int uxfio_fildes, void *buf,
 	p = (char *)buf;
 
 	if (uxfio == NULL) {
-		if (table_find(uxfio_fildes, &uxfio)) {
+		if (table_find(uxfio_fildes, &uxfio, 1)) {
 			UXFIO_E_DEBUG2("error %d", uxfio_fildes);
 			fprintf(stderr,
 		"uxfio error: uxfio_common_read: file desc %d not found.\n",
@@ -927,10 +930,10 @@ enforce_veof(UXFIO * uxfio, Xint nbyte, int * einval)
 	if (remaining <= nbyte) {
 		UXFIO_E_DEBUG("setting einval");
 		*einval=1;
-		UXFIO_E_DEBUG2("enforcing veof %d\n", remaining);
+		UXFIO_E_DEBUG2("enforcing veof %d\n", (int)remaining);
 		return remaining;
 	} else {
-		UXFIO_E_DEBUG2("no end found. returning %d", nbyte);
+		UXFIO_E_DEBUG2("no end found. returning %d", (int)nbyte);
 		return nbyte; 
 	}
 }
@@ -979,7 +982,7 @@ enforce_set_position(UXFIO * uxfio, Xint offset, int *einval)
 			} 
 		}
 	} 
-	UXFIO_E_DEBUG2("returning %d", offset);
+	UXFIO_E_DEBUG2("returning %d", (int)offset);
 	return offset;
 }
 
@@ -1068,7 +1071,7 @@ uxfio__init_buffer_file(UXFIO * uxfio, void *buf, int len)
 		uxfio->startM = uxfio->endM = uxfio->posM = 0;
 	
 		if (buf && (len > 0)) {
-			UXFIO_E_DEBUG3("fd=%d len=%d", uxfio->buffdM, len);
+			UXFIO_E_DEBUG3("fd=%d len=%d", (int)(uxfio->buffdM), (int)len);
 			if ((ret=uxfio_unix_atomic_write(uxfio->buffdM, buf, len))
 								!= len) {
 				close(uxfio->buffdM);
@@ -1091,7 +1094,7 @@ uxfio__init_buffer_file(UXFIO * uxfio, void *buf, int len)
 					UXFIO_E_FAIL2("%s",
 							strerror(errno));
 				}
-				UXFIO_E_DEBUG2("uxfio->endM=%d", uxfio->endM);
+				UXFIO_E_DEBUG2("uxfio->endM=%d", (int)(uxfio->endM));
 			}
 		}
 		return 0;
@@ -1184,7 +1187,7 @@ uxfio_internal_ftruncate(int uxfio_fildes, off_t nbyte)
 	intmax_t amount;
 	off_t pos;
 
-	if (table_find(uxfio_fildes, &uxfio)) {
+	if (table_find(uxfio_fildes, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr,
 			"uxfio error:  file desc %d not found.\n",
@@ -1360,6 +1363,23 @@ do_buffered_flush(UXFIO * uxfio)
 /* ------------------------------------------------------  */
 /* ------------- Public Function Definitions ------------  */
 
+
+void
+uxfio_show_all_open_fd(FILE * f)
+{
+	UXFIO * uxfio;
+	int i;
+	int n = 0;
+	for (i=UXFIO_FD_MIN; i<(UXFIO_MAX_OPEN + UXFIO_FD_MIN); i++) {
+		if (table_find(i, &uxfio, 0)) {
+			/* error, fd not found */
+			;
+		} else {
+			fprintf(f, "%d %d\n", n++, i);
+		}
+	}
+}
+
 int
 uxfio_uxfio_get_nopen(void)
 {
@@ -1436,7 +1456,7 @@ uxfio_debug_get_object_address(int uxfiofd)
 {
 	UXFIO *uxfio;
 	UXFIO_E_DEBUG("");
-	if (table_find(uxfiofd, &uxfio)) {
+	if (table_find(uxfiofd, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr, "uxfio error:  file desc %d not found.\n",
 				uxfiofd);
@@ -1451,6 +1471,24 @@ uxfio_get_object_address(int uxfiofd)
 	return uxfio_debug_get_object_address(uxfiofd);
 }
 
+int 
+uxfio_devnull_close(int fd)
+{
+	return 0;
+}
+
+int 
+uxfio_devnull_open(char *path, int oflag, mode_t mode)
+{
+	if (uxg_nullfd == 0)
+		uxg_nullfd = open ("/dev/null", O_RDWR, 0);
+	if (uxg_nullfd < 0) {
+		fprintf (stderr, "fatal error: can't open /dev/null: %s\n", strerror(errno));
+		exit(1);
+	}
+	return uxg_nullfd;
+}
+
 int 
 uxfio_open(char *path, int oflag, mode_t mode)
 {
@@ -1502,7 +1540,7 @@ uxfio_opendup(int uxfd, int buffertype)
 	if ((r = uxfio_fcntl(uxfio_fd, UXFIO_F_ATTACH_FD, uxfd)) < 0)
 		return -1;
 	
-	if (table_find(uxfio_fd, &uxfio)) {
+	if (table_find(uxfio_fd, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr, "uxfio error: file desc  %d not found.\n",
 			uxfio_fd);
@@ -1536,7 +1574,7 @@ uxfio_opendup(int uxfd, int buffertype)
 			uxfio->uxfd_can_seekM = 1;
 		} else {
 			UXFIO_E_DEBUG("uxfio->uxfd_can_seekM setting to 0");
-			UXFIO_E_DEBUG2("UXFD___\n%s\nUXFD^^^\n", uxfio_dump_string(uxfd));
+			/* UXFIO_E_DEBUG2("UXFD___\n%s\nUXFD^^^\n", uxfio_dump_string(uxfd)); */
 			uxfio->uxfd_can_seekM = 0;
 		}
 	} else {
@@ -1610,7 +1648,7 @@ uxfio_decr_use_count(int fd)
 	if (fd < UXFIO_FD_MIN ){
 		return -1;
 	}	
-	if (table_find(fd, &uxfio)) {
+	if (table_find(fd, &uxfio, 1)) {
 		return -1;
 	}
 	return --(uxfio->use_countM);
@@ -1621,7 +1659,7 @@ uxfio_incr_use_count(int fd)
 {
 	UXFIO *uxfio;
 	UXFIO_E_DEBUG2("uxfio_filedes=%d", fd);
-	if (table_find(fd, &uxfio)) {
+	if (table_find(fd, &uxfio, 1)) {
 		return -1;
 	}
 	return ++(uxfio->use_countM);
@@ -1642,7 +1680,7 @@ uxfio_espipe(int uxfio_fildes)
 		}
 	}
 	
-	if (table_find(uxfio_fildes, &uxfio)) {
+	if (table_find(uxfio_fildes, &uxfio, 1)) {
 		return -1;
 	}
 	ret = uxfio->uxfd_can_seekM ? 0 : ESPIPE;
@@ -1668,7 +1706,7 @@ uxfio_getfd(int uxfio_fildes, int * use_next)
 	if (uxfio_fildes < UXFIO_FD_MIN) {
 		return uxfio_fildes;
 	}
-	if (table_find(uxfio_fildes, &uxfio)) {
+	if (table_find(uxfio_fildes, &uxfio, 1)) {
 		return -1;
 	}
 	if (use_next) {
@@ -1748,7 +1786,7 @@ uxfio_write(int uxfio_fildes, void *buf, size_t nbyte)
 	} else if (uxfio_fildes < UXFIO_FD_MIN && uxfio_fildes >= 0) {
 		return (size_t) uxfio_unix_safe_write(uxfio_fildes, buf, nbyte);
 	} else {
-		if (table_find(uxfio_fildes, &uxfio)) {
+		if (table_find(uxfio_fildes, &uxfio, 1)) {
 			UXFIO_E_DEBUG("error");
 			fprintf(stderr,
 				"uxfio error:  file desc %d not found.\n",
@@ -1862,7 +1900,7 @@ uxfio_lseek(int uxfio_fildes, off_t poffset, int pwhence)
 	UXFIO_E_DEBUG3("ENTER FUNCTION  fd=%d offset=%d", uxfio_fildes,
 						(int)poffset);
 	UXFIO_E_DEBUG3("%d pwhence=%d", uxfio_fildes, pwhence);
-	UXFIO_E_DEBUG2("\n%s\n.", uxfio_dump_string(uxfio_fildes));
+	/* UXFIO_E_DEBUG2("\n%s\n.", uxfio_dump_string(uxfio_fildes)); */
 	whence = pwhence;
 	if (uxfio_fildes < UXFIO_FD_MIN && uxfio_fildes >= 0) {
 		if (whence == UXFIO_SEEK_VCUR) {
@@ -1870,20 +1908,20 @@ uxfio_lseek(int uxfio_fildes, off_t poffset, int pwhence)
 		}
 		return lseek(uxfio_fildes, poffset, whence);
 	}
-	if (table_find(uxfio_fildes, &uxfio)) {
+	if (table_find(uxfio_fildes, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr, "uxfio error: file desc %d not found.\n",
 			uxfio_fildes);
 		return -1;
 	}
 	
-	UXFIO_E_DEBUG3("%d posM=%d", uxfio_fildes, uxfio->posM);
+	UXFIO_E_DEBUG3("%d posM=%d", uxfio_fildes, (int)(uxfio->posM));
 	if (whence == SEEK_CUR || whence == UXFIO_SEEK_VCUR) {
 		offset=(off_t)(poffset);
 	} else if (whence == SEEK_SET || whence == UXFIO_SEEK_VSET) {
 		UXFIO_E_DEBUG("in SEEK_SET");
 		UXFIO_E_DEBUG3("poffset = %d  posM = %d",
-					(int)poffset, uxfio->posM);
+					(int)poffset, (int)(uxfio->posM));
 		if (
 			virtual_eof_active(uxfio) == 0 &&
 			uxfio->offset_eof_savedM >= 0 &&
@@ -2033,7 +2071,7 @@ uxfio_lseek(int uxfio_fildes, off_t poffset, int pwhence)
 		if (offset > uxfio->posM) {
 			int xxn;
 			UXFIO_E_DEBUG2("reading ahead %d bytes",
-						(int)offset - uxfio->posM);
+						(int)(offset - uxfio->posM));
 			xxn = uxfio__read_bytes(uxfio_fildes,
 						offset - uxfio->posM);
 			/* FIXME check return value */
@@ -2076,8 +2114,7 @@ uxfio_lseek(int uxfio_fildes, off_t poffset, int pwhence)
 		whence = SEEK_CUR;
 	}
 
-	UXFIO_E_DEBUG3("uxfio_lseek bufferstate  left=%d right=%d",
-								left, right);
+	UXFIO_E_DEBUG3("uxfio_lseek bufferstate  left=%d right=%d", (int)left, (int)right);
 	UXFIO_E_DEBUG2("uxfio_lseek offset = %d", (int)offset);
 
 	/* 
@@ -2236,7 +2273,7 @@ uxfio_fsync(int uxfio_fildes)
 	if (uxfio_fildes < UXFIO_FD_MIN && uxfio_fildes >= 0)
 		return fsync(uxfio_fildes);
 
-	if (table_find(uxfio_fildes, &uxfio)) {
+	if (table_find(uxfio_fildes, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr,
 			"uxfio error: file desc %d not found.\n",
@@ -2261,7 +2298,7 @@ uxfio_ioctl(int uxfio_fd, int request, void * arg)
 {
 	UXFIO *uxfio;
 	UXFIO_E_DEBUG2("uxfio_filedes=%d", uxfio_fd);
-	if (table_find(uxfio_fd, &uxfio)) {
+	if (table_find(uxfio_fd, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr, "uxfio error: file desc %d not found.\n",
 				uxfio_fd);
@@ -2298,7 +2335,7 @@ uxfio_fstat(int uxfio_fd, struct stat * buf)
 {
 	UXFIO *uxfio;
 	UXFIO_E_DEBUG2("uxfio_filedes=%d", uxfio_fd);
-	if (table_find(uxfio_fd, &uxfio)) {
+	if (table_find(uxfio_fd, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr, "uxfio error: file desc %d not found.\n",
 						uxfio_fd);
@@ -2322,7 +2359,7 @@ uxfio_fcntl(int uxfio_fildes, int cmd, int value)
 	if (uxfio_fildes < UXFIO_FD_MIN && uxfio_fildes >= 0){
 		return fcntl(uxfio_fildes, cmd, value);
 	}	
-	if (table_find(uxfio_fildes, &uxfio)) {
+	if (table_find(uxfio_fildes, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr, "uxfio error: file desc %d not found.\n",
 						uxfio_fildes);
@@ -2414,11 +2451,11 @@ uxfio_fcntl(int uxfio_fildes, int cmd, int value)
 				   if it causes loss of information */
 				if (uxfio->buffer_activeM && (uxfio->posM < uxfio->endM))
 					return -1;
-				UXFIO_E_DEBUG3("state:  pos=%d end=%d", uxfio->posM, uxfio->endM);
+				UXFIO_E_DEBUG3("state:  pos=%d end=%d", (int)(uxfio->posM), (int)(uxfio->endM));
 				ret = uxfio__delete_buffer_file(uxfio);
 			} else if (value && uxfio->buffer_activeM == 0) {
 				UXFIO_E_DEBUG("in UXFIO_SET_BUFACTIVE on");
-				UXFIO_E_DEBUG3("state:  pos=%d end=%d", uxfio->posM, uxfio->endM);
+				UXFIO_E_DEBUG3("state:  pos=%d end=%d", (int)(uxfio->posM), (int)(uxfio->endM));
 				ret = uxfio__init_buffer_file(uxfio,
 							(void *)(NULL), 0);
 			} else {
@@ -2498,7 +2535,6 @@ uxfio_fcntl(int uxfio_fildes, int cmd, int value)
 				fprintf(stderr,
 					"uxfio internal error: 002.33\n");
 			}
-			fprintf(stderr,"JL in UXFIO_SET_BUFTYPE 3");
 			ret = 0;
 		} else {
 			UXFIO_E_DEBUG(" default else in set buftype");
@@ -2601,7 +2637,7 @@ uxfio_get_dynamic_buffer(int uxfio_fildes, char **buffer_ptr,
 	UXFIO *uxfio;
 	off_t curpos;
 
-	if (table_find(uxfio_fildes, &uxfio)) {
+	if (table_find(uxfio_fildes, &uxfio, 1)) {
 		UXFIO_E_DEBUG("error");
 		fprintf(stderr, "uxfio error: file desc %d not found.\n",
 					uxfio_fildes);
