@@ -1,7 +1,3 @@
-#ifdef DOTO_ID
-static char testpath_cxx[] =
-"$Id: testheader.cxx_v 1.5 1998/06/30 03:47:01 jhl Exp jhl $";
-#endif
 
 #include "swuser_config.h"
 #include <stdio.h>
